@@ -6,16 +6,11 @@
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: ven set 10 2004
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -170,6 +165,7 @@ static void usage()
    stdOut->writeString( "   -E <enc>    Source files are in <enc> encoding (overrides -e)\n" );
    stdOut->writeString( "   -f          force recompilation of modules even when .fam are found\n" );
    stdOut->writeString( "   -h/-?       this help\n" );
+   stdOut->writeString( "   -l <lang>   Set preferential language of loaded modules\n" );
    stdOut->writeString( "   -L <path>   set path for 'load' directive\n" );
    stdOut->writeString( "   -m          do NOT compile in memory (use temporary files)\n" );
    stdOut->writeString( "   -M          do NOT save the compiled modules in '.fam' files\n" );
@@ -184,6 +180,8 @@ static void usage()
    stdOut->writeString( "   -v          print copyright notice and version and exit\n" );
    stdOut->writeString( "   -w          Add an extra console wait after program exit\n" );
    stdOut->writeString( "   -x          execute a binary '.fam' module\n" );
+   stdOut->writeString( "   -y          write string translation table for the module\n" );
+
    stdOut->writeString( "\n" );
    stdOut->writeString( "Paths must be in falcon file name format: directory separatros must be slashes [/] and\n" );
    stdOut->writeString( "multiple entries must be entered separed by a semicomma (';')\n" );
@@ -459,6 +457,13 @@ void parseOptions( int argc, char **argv, int &script_pos )
                   options.load_path = op + 2;
             break;
 
+            case 'l':
+               if ( op[2] == 0 && i + 1 < argc )
+                  options.module_language = argv[++i];
+               else
+                  options.module_language = op + 2;
+            break;
+
             case 'm': options.comp_memory = false; break;
             case 'M': options.save_modules = false; break;
 
@@ -486,6 +491,7 @@ void parseOptions( int argc, char **argv, int &script_pos )
             case 'x': options.run_only = true; break;
             case 'v': version(); exitNow = true; break;
             case 'w': options.wait_after = true; break;
+            case 'y': options.compile_tltable = true; break;
 
             default:
                stdErr->writeString( "falcon: unrecognized option '" );
@@ -705,6 +711,9 @@ int main( int argc, char *argv[] )
    // 1. Ready the module loader
    FlcLoader *modLoader = new FlcLoader( get_load_path() );
 
+   // set the module preferred language; ok also if default ("") is used
+   modLoader->setLanguage( options.module_language );
+
    if ( ! apply_directives( modLoader->compiler() ) )
    {
       exit_sequence(1);
@@ -726,6 +735,11 @@ int main( int argc, char *argv[] )
 
    // save the main module also if compile only option is set
    modLoader->saveModules( options.save_modules || options.compile_only );
+
+   //... but disable if compiling tltables.
+   if ( options.compile_tltable )
+      modLoader->saveModules( false );
+
    modLoader->alwaysRecomp( options.compile_only || options.force_recomp );
    modLoader->sourceEncoding( get_src_encoding() );
    // normally, save is not mandatory, unless we compile them our own
@@ -738,7 +752,8 @@ int main( int argc, char *argv[] )
    // If we have to assemble or compile just a module...
    if ( options.assemble_only || options.compile_only )
    {
-      modLoader->sourceIsAssembly( true );
+      if( options.assemble_only )
+         modLoader->sourceIsAssembly( true );
 
       // force not to save modules, we're saving it on our own
       modLoader->saveModules( false );
@@ -752,7 +767,13 @@ int main( int argc, char *argv[] )
       // if we had a module, save it at the right place.
       if ( mod != 0 )
       {
-         mod->save( openOutputStream( "fam" ) );
+         // should we save the module ?
+         if ( options.assemble_only || options.compile_only )
+         {
+            Stream *modstream = openOutputStream( "fam" );
+            mod->save( modstream );
+            modstream->close();
+         }
       }
 
       // whatever happened, we go to exit now.
@@ -763,16 +784,6 @@ int main( int argc, char *argv[] )
    // with ignore source option, the flcLoader will bypass loadSource and use loadModule.
    modLoader->ignoreSources( options.run_only );
 
-   // Create the runtime using the given module loader.
-   Runtime *runtime = new Runtime( modLoader );
-
-   Module *falcon_rtl = modLoader->loadName( "falcon_rtl" );
-
-   if ( falcon_rtl == 0 )
-   {
-      // mod loader has already raised the error.
-      exit_sequence( 1 );
-   }
    // Load the modules
    Module *mainMod;
 
@@ -793,6 +804,39 @@ int main( int argc, char *argv[] )
       exit_sequence( 1, modLoader->compileErrors() );
    }
 
+   // should we just write the string table?
+   if( options.compile_tltable )
+   {
+      if( mainMod->stringTable().internatCount() > 0 )
+      {
+         Stream *tplstream = openOutputStream( "temp.ftt" );
+         if( tplstream )
+         {
+            // Wrap using default encoding
+            if ( options.source_encoding != "" && options.source_encoding != "C" )
+               tplstream = TranscoderFactory( options.source_encoding, tplstream, false );
+            mainMod->saveTableTemplate( tplstream, options.source_encoding );
+            tplstream->close();
+         }
+         else
+            exit_sequence( 1, 0 );
+      }
+
+      exit_sequence( 0 , 0 );
+   }
+
+   // Create the runtime using the given module loader.
+   Runtime *runtime = new Runtime( modLoader );
+
+   // But before proceding, load the rtl.
+   Module *falcon_rtl = modLoader->loadName( "falcon_rtl" );
+
+   if ( falcon_rtl == 0 )
+   {
+      // mod loader has already raised the error.
+      exit_sequence( 1 );
+   }
+
    // now that we have the main module, inject other requested modules
    ListElement *pliter = options.preloaded.begin();
    while( pliter != 0 )
@@ -838,12 +882,12 @@ int main( int argc, char *argv[] )
 
    // push the core module
    Module *core = core_module_init();
-   bool res = vmachine->link( core );
-   fassert( res );  // should not fail
+   LiveModule *res = vmachine->link( core );
+   fassert( res != 0 );  // should not fail
    core->decref();
 
    res = vmachine->link( falcon_rtl );
-   fassert( res ); // should not fail
+   fassert( res != 0 ); // should not fail
    // abandon our reference to the rtl.
    falcon_rtl->decref();
 
@@ -939,7 +983,7 @@ int main( int argc, char *argv[] )
       delete stdOut;
    }
 
-   exit_sequence( exitSeq ? 0 : exitVal );
+   exit_sequence( exitSeq ? exitVal : 0 );
    return 0; // to make the compiler happy
 }
 
