@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: faldisass.cpp
-   $Id: flcdis.cpp,v 1.13 2007/08/18 11:08:08 jonnymind Exp $
 
    Falcon disassembler
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: 2004-08-01
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #include <falcon/setup.h>
@@ -182,8 +176,8 @@ void write_operand( Stream *output, byte *instruction, int opnum, Module *mod )
             if (
                ( opnum == 1 && ( opcode == P_JMP || opcode == P_TRAL || opcode == P_TRY
                                  || opcode == P_JTRY || opcode == P_IFT || opcode == P_IFF
-                                 || opcode == P_ONCE || opcode == P_TRAV || opcode == P_FORI
-                                 || opcode == P_FORN || opcode == P_TRAN || opcode == P_SWCH ) ) ||
+                                 || opcode == P_ONCE || opcode == P_TRAV
+                                 || opcode == P_TRAN || opcode == P_SWCH ) ) ||
                ( opnum == 2 && ( opcode == P_FORK || opcode == P_TRAN ) )
                )
             {
@@ -210,6 +204,8 @@ void write_operand( Stream *output, byte *instruction, int opnum, Module *mod )
          output->writeString(temp );
          break;
       case P_PARAM_NIL: output->writeString( "NIL" ); break;
+      case P_PARAM_TRUE: output->writeString( "T" ); break;
+      case P_PARAM_FALSE: output->writeString( "F" ); break;
       case P_PARAM_INT64:
          temp.writeNumber( (int64) endianInt64(*((int64*) (instruction + offset ))) );
          output->writeString(temp );
@@ -229,6 +225,8 @@ void write_operand( Stream *output, byte *instruction, int opnum, Module *mod )
       case P_PARAM_REGB: output->writeString( "B" ); break;
       case P_PARAM_REGS1: output->writeString( "S1" ); break;
       case P_PARAM_REGS2: output->writeString( "S2" ); break;
+      case P_PARAM_REGL1: output->writeString( "L1" ); break;
+      case P_PARAM_REGL2: output->writeString( "L2" ); break;
    }
 
 }
@@ -313,6 +311,7 @@ void gen_propdef( Stream *m_out, const VarDef &def )
    switch( def.type() )
    {
       case VarDef::t_nil: m_out->writeString( "NIL" ); break;
+      case VarDef::t_bool: m_out->writeString( def.asBool() ? "T": "F" ); break;
       case VarDef::t_int:
          temp.writeNumber( def.asInteger() );
          m_out->writeString( temp );
@@ -586,11 +585,11 @@ void disassembler( Module *module, Stream *out, const t_labelMap &labels, const
          case P_STPR: csOpName = "STPR"; break;
          case P_STVR: csOpName = "STVR"; break;
          case P_TRAV: csOpName = "TRAV"; break;
+         case P_INCP: csOpName = "INCP"; break;
+         case P_DECP: csOpName = "DECP"; break;
          case P_PASS: csOpName = "PASS"; break;
          case P_PSIN: csOpName = "PSIN"; break;
 
-         case P_FORI: csOpName = "FORI"; break;
-         case P_FORN: csOpName = "FORN"; break;
          case P_SHR : csOpName = "SHR "; break;
          case P_SHL : csOpName = "SHL "; break;
          case P_SHRS: csOpName = "SHRS"; break;
@@ -602,7 +601,8 @@ void disassembler( Module *module, Stream *out, const t_labelMap &labels, const
          case P_INDI: csOpName = "INDI"; break;
          case P_STEX: csOpName = "STEX"; break;
          case P_TRAC: csOpName = "TRAC"; break;
-         case P_WRT: csOpName = "WRT"; break;
+         case P_WRT : csOpName = "WRT "; break;
+         case P_STO : csOpName = "STO "; break;
 
          default:
             csOpName = "????";
@@ -816,8 +816,6 @@ void Analizer( Module *module, t_labelMap &labels  )
          case P_IFF :
          case P_ONCE:
          case P_TRAV:
-         case P_FORI:
-         case P_FORN:
             labels[ endianInt32( *(uint32 *)(code + sizeof(int32)) ) ] = 0;
          break;
       }
@@ -909,7 +907,7 @@ void write_strtable( e_tabmode mode , Stream *out, Module *mod )
             temp.size(0);
          break;
          case e_mode_table:
-            out->writeString( ".string " );
+            out->writeString( str->exported() ? ".istring " : ".string " );
          break;
       }
 
@@ -1051,6 +1049,10 @@ using namespace Falcon;
 
 int main( int argc, char *argv[] )
 {
+   // initialize the engine
+   EngineData data;
+   Init( data );
+
    Stream *stdErr = stdErrorStream();
 
    // option decoding
