@@ -1,23 +1,17 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: verion.h
-   $Id: version.h,v 1.1 2007/07/25 12:23:08 jonnymind Exp $
 
    Falcon clt
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: 2007-07-25
 
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #ifndef FDIS_VERSION_H
@@ -25,7 +19,7 @@
 
 #define FDIS_VERSION_MAJOR        0
 #define FDIS_VERSION_MINOR        8 
-#define FDIS_VERSION_REVISION     0
+#define FDIS_VERSION_REVISION     10
 
 #endif
 
