@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: faltest.cpp
-   $Id: faltest.cpp,v 1.27 2007/08/11 19:02:32 jonnymind Exp $
 
    Testsuite interpreter
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: lun feb 13 2006
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -212,6 +206,7 @@ static void usage()
    stdOut->writeString( "   -C <subcat> only perform test in this subcategory\n" );
    stdOut->writeString( "   -d <path>   tests are in specified directory\n" );
    stdOut->writeString( "   -l          Just list available tests and exit\n" );
+   stdOut->writeString( "   -L          Changes Falcon load path.\n" );
    stdOut->writeString( "   -h/-?       Show this help\n" );
    stdOut->writeString( "   -m          do NOT compile in memory\n" );
    stdOut->writeString( "   -M          Check for memory allocation correctness.\n" );
@@ -1290,6 +1285,8 @@ int main( int argc, char *argv[] )
    delete stdOut;
    delete stdErr;
 
+   if ( failedCount > 0 )
+       return 2;
    return 0;
 }
 
