@@ -6,16 +6,11 @@
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: 01-08-2004
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #include <falcon/compiler.h>
@@ -45,6 +40,8 @@ Compiler::Compiler( Module *mod, Stream* in ):
    m_stream( in ),
    m_constants( &traits::t_string, &traits::t_voidp ),
    m_strict( false ),
+   m_language( "C" ),
+   m_modVersion( 0 ),
    m_defContext( false ),
    m_delayRaise( false ),
    m_rootError( 0 ),
@@ -70,11 +67,14 @@ Compiler::Compiler():
    m_stream( 0 ),
    m_constants( &traits::t_string, &traits::t_voidp ),
    m_strict( false ),
+   m_language( "C" ),
+   m_modVersion( 0 ),
    m_defContext( false ),
    m_delayRaise( false ),
    m_rootError( 0 ),
    m_lexer(0),
    m_root(0),
+   m_lambdaCount(0),
    m_bParsingFtd(false)
 {
    reset();
@@ -112,6 +112,7 @@ void Compiler::init()
    m_enumId = 0;
    m_staticPrefix = 0;
    m_lambdaCount = 0;
+   m_closureContexts = 0;
    m_tempLine = 0;
 
    // reset the require def status
@@ -143,6 +144,13 @@ void Compiler::reset()
 
    addPredefs();
 
+   m_errors = 0;
+   m_enumId = 0;
+   m_staticPrefix = 0;
+   m_lambdaCount = 0;
+   m_closureContexts = 0;
+   m_tempLine = 0;
+
    // reset FTD parsing mode
    parsingFtd(false);
 }
@@ -204,12 +212,18 @@ bool Compiler::compile()
 
    // save directives
    bool bSaveStrict = m_strict;
+   String savedLanguage = m_language;
+   int64 modver = m_modVersion;
 
    // parse
    flc_src_parse( this );
+   m_module->language( m_language );
+   m_module->version( (uint32) m_modVersion );
 
    // restore directives
    m_strict = bSaveStrict;
+   m_language = savedLanguage;
+   m_modVersion = modver;
 
    // If the context is not empty, then we have something unclosed.
    if ( ! m_context.empty() ) {
@@ -217,7 +231,6 @@ bool Compiler::compile()
       String temp = "from line ";
       temp.writeNumber( (int64) stmt->line() );
       raiseError( e_unclosed_cs, temp );
-      return false;
    }
 
    // leadout sequence
@@ -233,7 +246,6 @@ bool Compiler::compile()
       {
          m_module->entry( 0 );
       }
-
       return true;
    }
 
@@ -344,15 +356,49 @@ void Compiler::defineVal( ArrayDecl *val )
 void Compiler::defineVal( Value *val )
 {
    // raise error for read-only expressions
-   if ( val->isExpr() && val->asExpr()->type() == Expression::t_array_byte_access )
+   if ( val->isExpr() )
    {
-      raiseError( e_byte_access, lexer()->previousLine() );
-      // but proceed
+      if ( val->asExpr()->type() == Expression::t_array_byte_access )
+      {
+         raiseError( e_byte_access, lexer()->previousLine() );
+         // but proceed
+      }
+      else {
+         Expression *expr = val->asExpr();
+         defineVal( expr->first() );
+         if( expr->second() != 0 )
+         {
+            if ( expr->second()->isExpr() &&
+                 expr->second()->asExpr()->type() == Expression::t_assign
+                 )
+            {
+               defineVal( expr->second() );
+            }
+            else if ( expr->isBinaryOperator() )
+            {
+               raiseError( e_assign_const, lexer()->previousLine() );
+            }
+         }
+      }
    }
-
-   if ( val->isSymdef() )
+   else if ( val->isArray() )
    {
-      if ( staticPrefix() == 0 ) {
+      ListElement *it_s = val->asArray()->begin();
+      while( it_s != 0 )
+      {
+         Value *t = (Value *) it_s->data();
+         defineVal( t );
+         it_s = it_s->next();
+      }
+   }
+   else if ( val->isImmediate() )
+   {
+      raiseError( e_assign_const, lexer()->previousLine() );
+   }
+   else if ( val->isSymdef() )
+   {
+      if ( staticPrefix() == 0 )
+      {
          Symbol *sym;
          if ( getFunction() != 0 ) {
             // addlocal must also define the symbol
@@ -433,10 +479,36 @@ Symbol *Compiler::addLocalSymbol( const String *symname, bool parameter )
 bool Compiler::checkLocalUndefined()
 {
    List *l = (List *) m_statementVals.back();
-   while( ! l->empty() ) {
+   while( ! l->empty() )
+   {
       Value *val = (Value *) l->front();
-      if ( val->isSymdef() ) {
-         Symbol *sym = addGlobalSymbol( val->asSymdef() );
+      if ( val->isSymdef() )
+      {
+         Symbol *sym = 0;
+         if ( m_closureContexts > 0 )
+         {
+            fassert( m_functions.end() );
+            if ( m_functions.begin() != m_functions.end() )
+            {
+               const FuncDef *fd_parent = reinterpret_cast<const FuncDef *>
+                  ( m_functions.end()->prev()->data() );
+
+               if( fd_parent != 0 )
+               {
+                  if( fd_parent->symtab().findByName( *val->asSymdef() ) != 0 )
+                  {
+                     sym = getFunction()->addUndefined(
+                        m_module->addSymbol(*val->asSymdef()) );
+                  }
+               }
+            }
+         }
+
+         // still nothing?
+         if (sym == 0)
+         {
+            sym = addGlobalSymbol( val->asSymdef() );
+         }
          val->setSymbol( sym );
       }
       l->popFront();
@@ -447,7 +519,6 @@ bool Compiler::checkLocalUndefined()
 
 Symbol *Compiler::searchLocalSymbol( const String *symname )
 {
-   // should not happen.
    if( m_functions.empty() )
       return searchGlobalSymbol( symname );
 
@@ -462,6 +533,33 @@ Symbol *Compiler::searchLocalSymbol( const String *symname )
    return fd->symtab().findByName( *symname );
 }
 
+Symbol *Compiler::searchOuterSymbol( const String *symname )
+{
+   ListElement *aliasIter = m_alias.end();
+   ListElement *funcIter = m_functions.end();
+
+   while( aliasIter != 0 && funcIter != 0 )
+   {
+      AliasMap *map = (AliasMap *) aliasIter->data();
+
+      // first search the local symbol aliases
+      Symbol **sympp = (Symbol **) map->find( symname );
+      if ( sympp != 0 )
+         return *sympp;
+
+      // then try in the local symtab or just return 0.
+      FuncDef *fd = (FuncDef *) funcIter->data();
+      Symbol *sym = fd->symtab().findByName( *symname );
+      if ( sym != 0 )
+         return sym;
+
+      aliasIter = aliasIter->prev();
+      funcIter = funcIter->prev();
+   }
+
+   return searchGlobalSymbol( symname );
+}
+
 
 Symbol *Compiler::searchGlobalSymbol( const String *symname )
 {
@@ -596,21 +694,20 @@ void Compiler::addPredefs()
 {
 
    addIntConstant( "NilType", FLC_ITEM_NIL );
+   addIntConstant( "BooleanType", FLC_ITEM_BOOL );
    addIntConstant( "IntegerType", FLC_ITEM_INT );
    addIntConstant( "NumericType", FLC_ITEM_NUM );
    addIntConstant( "RangeType", FLC_ITEM_RANGE );
    addIntConstant( "AttributeType", FLC_ITEM_ATTRIBUTE );
    addIntConstant( "FunctionType", FLC_ITEM_FUNC );
    addIntConstant( "StringType", FLC_ITEM_STRING );
+   addIntConstant( "MemBufType", FLC_ITEM_MEMBUF );
    addIntConstant( "ArrayType", FLC_ITEM_ARRAY );
    addIntConstant( "DictionaryType", FLC_ITEM_DICT );
    addIntConstant( "ObjectType", FLC_ITEM_OBJECT );
    addIntConstant( "ClassType", FLC_ITEM_CLASS );
    addIntConstant( "MethodType", FLC_ITEM_METHOD );
    addIntConstant( "ClassMethodType", FLC_ITEM_CLSMETHOD );
-   // Should never be received by the applications.
-   addIntConstant( "true", 1 );
-   addIntConstant( "false", 0 );
 }
 
 void Compiler::addIntConstant( const String &name, int64 value, uint32 line )
@@ -719,6 +816,11 @@ bool Compiler::setDirective( const String &directive, const String &value, bool
 
       bWrongVal = true;
    }
+   else if ( directive == "lang" )
+   {
+      m_language = value;
+      return true;
+   }
 
    // ...
    // if we're here we have either a wrong directive or a wrong value.
@@ -738,10 +840,15 @@ bool Compiler::setDirective( const String &directive, int64 value, bool bRaise )
 {
    bool bWrongVal = false;
 
-   if ( directive == "strict" )
+   if ( directive == "strict" || directive == "lang" )
    {
       bWrongVal = true;
    }
+   else if ( directive == "version" )
+   {
+      m_modVersion = value;
+      return true;
+   }
 
    // if we're here we have either a wrong directive or a wrong value.
    if ( bRaise )
@@ -760,6 +867,133 @@ bool Compiler::setDirective( const String &directive, int64 value, bool bRaise )
    return true;
 }
 
+Value *Compiler::closeClosure()
+{
+   // first, close it as a normal function, but without mangling with the
+   // local function table.
+   StmtFunction *func = static_cast<Falcon::StmtFunction *>( getContext() );
+   FuncDef *fd = func->symbol()->getFuncDef();
+
+   // has this function a static block?
+   if ( func->hasStatic() )
+   {
+      fd->onceItemId( m_module->addGlobal( "_once_" + func->symbol()->name(), false )->itemId() );
+   }
+
+   popContext();
+   popFunctionContext();
+   popContextSet();
+   popFunction();
+   decClosureContext();
+
+   // we're going to need a lambda call
+   Value *lambda_call = new Value( new Falcon::Expression( Expression::t_lambda , new Value( func->symbol() ) ) );
+
+   // Is there some undefined?
+   if( fd->undefined() > 0 )
+   {
+      // we have to find all the local variables that exist in the upper context and
+      // transform them in parameters.
+      SymbolTable &funcTable = fd->symtab();
+      ArrayDecl *closureDecl = new ArrayDecl;
+
+      //First; put parameters away, so that we can reorder them.
+      const Map &symbols = funcTable.map();
+      MapIterator iter = symbols.begin();
+      int moved = 0;
+
+      while( iter.hasCurrent() )
+      {
+         Symbol *sym = *(Symbol **) iter.currentValue();
+         if ( sym->isLocalUndef() )
+         {
+            // ok, this must become a parameter...
+            sym->setParam();
+            sym->itemId( moved );
+            moved ++;
+
+            Symbol *parentSym;
+            if ( (parentSym = searchLocalSymbol( &sym->name() )) != 0 )
+            {
+               //... and the parent symbol must be stored in parametric array...
+               closureDecl->pushBack( new Value( parentSym ) );
+            }
+            else {
+               // closures can't have undefs.
+               raiseError( e_undef_sym, "", sym->declaredAt() );
+            }
+         }
+         else if ( sym->isParam() )
+         {
+            // push forward all parameters
+            sym->itemId( fd->undefined() + sym->itemId() );
+         }
+
+         iter.next();
+      }
+
+      // no more undefs -- now they are params
+      fd->params( fd->params() + fd->undefined() );
+      fd->undefined( 0 );
+
+      // ... put it in front of our array and return it.
+      closureDecl->pushFront( lambda_call );
+      return new Value( closureDecl );
+   }
+   else
+   {
+      // just create create a lambda call for this function
+      return lambda_call;
+   }
+
+}
+
+
+void Compiler::addEnumerator( const String &str, Value *val )
+{
+   StmtClass *stmt = static_cast< StmtClass *>( getContext() );
+   ClassDef *cd = stmt->symbol()->getClassDef();
+
+   if ( cd->hasProperty( str ) )
+   {
+      raiseError( e_already_def, str, lexer()->previousLine() );
+   }
+   else
+   {
+      VarDef *vd = m_module->addClassProperty( stmt->symbol(), str );
+      switch( val->type() )
+      {
+         case Value::t_nil :
+            // nothing to do
+            break;
+
+         case Value::t_imm_integer:
+            vd->setInteger( val->asInteger() );
+            m_enumId = val->asInteger() + 1;
+            break;
+
+         case Value::t_imm_num:
+            vd->setNumeric( val->asNumeric() );
+            m_enumId = int( val->asNumeric() ) + 1;
+            break;
+
+         case Value::t_imm_string:
+            vd->setString( m_module->addString( *val->asString() ) );
+            break;
+
+         case Value::t_imm_bool:
+            vd->setBool( val->asBool() );
+            break;
+      }
+   }
+}
+
+
+void Compiler::addEnumerator( const String &str )
+{
+   Value dummy( (int64) m_enumId );
+   addEnumerator( str, &dummy );
+}
 
 }
 
