@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language
    FILE: error.cpp
-   $Id: error.cpp,v 1.21 2007/08/18 17:59:33 jonnymind Exp $
 
    Error management.
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: lun ago 28 2006
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -30,6 +24,7 @@
 #include <falcon/sys.h>
 #include <falcon/vm.h>
 #include <falcon/cobject.h>
+#include <falcon/enginedata.h>
 
 namespace Falcon {
 
@@ -155,7 +150,7 @@ const String &errorDesc( int code )
       case e_syn_for: return getMessage( msg::err_syn_for );
       case e_syn_forfirst: return getMessage( msg::err_syn_forfirst );
       case e_syn_forlast: return getMessage( msg::err_syn_forlast );
-      case e_syn_forall: return getMessage( msg::err_syn_forall );
+      case e_syn_formiddle: return getMessage( msg::err_syn_formiddle );
       case e_syn_give: return getMessage( msg::err_syn_give );
       case e_syn_try: return getMessage( msg::err_syn_try );
       case e_syn_catch: return getMessage( msg::err_syn_catch );
@@ -190,6 +185,7 @@ const String &errorDesc( int code )
       case e_syn_fordot: return getMessage( msg::err_syn_fordot );
       case e_syn_self_print: return getMessage( msg::err_syn_self_print );
       case e_syn_directive: return getMessage( msg::err_syn_directive );
+      case e_syn_import: return getMessage( msg::err_syn_import );
 
       case e_nl_in_lit: return getMessage( msg::err_nl_in_lit );
       case e_catch_clash: return getMessage( msg::err_catch_clash );
@@ -204,10 +200,14 @@ const String &errorDesc( int code )
       case e_unclosed_string: return getMessage( msg::err_unclosed_string );
       case e_directive_unk: return getMessage( msg::err_directive_unk );
       case e_directive_value: return getMessage( msg::err_directive_value );
+      case e_noninst_cls: return getMessage( msg::err_noninst_cls );
+      case e_unserializable: return getMessage( msg::err_unserializable );
+      case e_wait_in_atomic: return getMessage( msg::err_wait_in_atomic );
+
 
       case e_already_forfirst: return getMessage( msg::err_already_forfirst );
       case e_already_forlast: return getMessage( msg::err_already_forlast );
-      case e_already_forall: return getMessage( msg::err_already_forall );
+      case e_already_formiddle: return getMessage( msg::err_already_formiddle );
 
 
       case e_open_file: return getMessage( msg::err_open_file);
@@ -215,7 +215,8 @@ const String &errorDesc( int code )
       case e_nofile: return getMessage( msg::err_nofile);
       case e_invformat: return getMessage( msg::err_invformat );
       case e_loader_unsupported: return getMessage( msg::err_loader_unsupported );
-
+      case e_io_error: return getMessage( msg::err_io_error );
+      case e_unknown_encoding: return getMessage( msg::err_unknown_encoding );
 
       case e_inv_params: return getMessage( msg::err_inv_params );
       case e_missing_params: return getMessage( msg::err_missing_params );
@@ -312,13 +313,12 @@ Error::~Error()
 
 void Error::incref()
 {
-   m_refCount++;
+   Engine::atomicInc( m_refCount );
 }
 
 void Error::decref()
 {
-   --m_refCount;
-	if( m_refCount == 0 )
+   if( Engine::atomicDec( m_refCount ) <= 0 )
    {
 		delete this;
    }
@@ -531,8 +531,6 @@ CoreObject *Error::scriptize( VMachine *vm )
 }
 
 
-
-
 ErrorCarrier::ErrorCarrier( Error *carried ):
    m_error( carried )
 {
@@ -555,12 +553,12 @@ ErrorCarrier::~ErrorCarrier()
    m_error->decref();
 }
 
-bool ErrorCarrier::isReflective()
+bool ErrorCarrier::isReflective() const
 {
    return true;
 }
 
-void ErrorCarrier::getProperty( const String &propName, Item &prop )
+void ErrorCarrier::getProperty( VMachine *vm, const String &propName, Item &prop )
 {
    if ( m_error == 0 )
       return;
@@ -568,7 +566,15 @@ void ErrorCarrier::getProperty( const String &propName, Item &prop )
    if ( propName == "code" )
       prop = (int64) m_error->errorCode();
    else if ( propName == "description" )
-      prop = const_cast<String *>(&m_error->errorDescription());
+   {
+      const String &desc = m_error->errorDescription();
+      if ( desc.size() == 0 && prop.isNil() )
+      {
+         prop = new GarbageString( vm, errorDesc( m_error->errorCode() ) );
+      }
+      else
+         prop = const_cast<String *>(&m_error->errorDescription());
+   }
    else if ( propName == "message" )
       prop = const_cast<String *>( &m_error->extraDescription() );
    else if ( propName == "systemError" )
@@ -588,7 +594,7 @@ void ErrorCarrier::getProperty( const String &propName, Item &prop )
 
 }
 
-void ErrorCarrier::setProperty( const String &propName, Item &prop )
+void ErrorCarrier::setProperty( VMachine *, const String &propName, Item &prop )
 {
    if ( m_error == 0 )
       return;
