@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: compiler.cpp
-   $Id: fasm_comp.cpp,v 1.17 2007/08/18 11:08:07 jonnymind Exp $
 
    Assembly compiler
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: 2005-08-22
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #include <falcon/setup.h>
@@ -42,6 +36,8 @@ Pseudo *AsmCompiler::regA_Inst() { return m_lexer->regA_Inst(); }
 Pseudo *AsmCompiler::regB_Inst() { return m_lexer->regB_Inst(); }
 Pseudo *AsmCompiler::regS1_Inst() { return m_lexer->regS1_Inst(); }
 Pseudo *AsmCompiler::regS2_Inst() { return m_lexer->regS2_Inst(); }
+Pseudo *AsmCompiler::regL1_Inst() { return m_lexer->regL1_Inst(); }
+Pseudo *AsmCompiler::regL2_Inst() { return m_lexer->regL2_Inst(); }
 Pseudo *AsmCompiler::nil_Inst() { return m_lexer->nil_Inst(); }
 
 AsmCompiler::AsmCompiler( Module *mod, Stream *in, Stream *out ):
@@ -779,6 +775,8 @@ unsigned char AsmCompiler::paramDesc( Pseudo *op1 ) const
    else {
       switch( op1->type() )
       {
+         case Pseudo::imm_true: return P_PARAM_TRUE;
+         case Pseudo::imm_false: return P_PARAM_FALSE;
          case Pseudo::imm_int: return P_PARAM_INT64;
          case Pseudo::imm_double: return P_PARAM_NUM;
          case Pseudo::imm_string: return P_PARAM_STRID;
@@ -792,6 +790,8 @@ unsigned char AsmCompiler::paramDesc( Pseudo *op1 ) const
          case Pseudo::tregB: return P_PARAM_REGB;
          case Pseudo::tregS1: return P_PARAM_REGS1;
          case Pseudo::tregS2: return P_PARAM_REGS2;
+         case Pseudo::tregL1: return P_PARAM_REGL1;
+         case Pseudo::tregL2: return P_PARAM_REGL2;
          case Pseudo::tnil: return P_PARAM_NIL;
       }
    }
