@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language
    FILE: fasm_lexer.cpp
-   $Id: fasm_lexer.cpp,v 1.12 2007/08/18 11:08:07 jonnymind Exp $
 
    Assembly lexer implementation.
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: dom ago 27 2006
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -50,8 +44,11 @@ AsmLexer::AsmLexer( Module *mod, AsmCompiler *cmp, Stream *in ):
       m_regb( Pseudo::tregB, false ),
       m_regs1( Pseudo::tregS1, false ),
       m_regs2( Pseudo::tregS2, false ),
-      m_regs3( Pseudo::tregS3, false ),
-      m_nil( Pseudo::tnil, false )
+      m_regl1( Pseudo::tregL1, false ),
+      m_regl2( Pseudo::tregL2, false ),
+      m_nil( Pseudo::tnil, false ),
+      m_true( Pseudo::imm_true, false ),
+      m_false( Pseudo::imm_false, false )
 {}
 
 int AsmLexer::lex()
@@ -345,7 +342,7 @@ int AsmLexer::lex()
             else if ( chr == '"' )
             {
                // closed string
-               const Falcon::String *str = m_module->addString( m_string );
+               Falcon::String *str = m_module->addString( m_string );
                *VALUE = new Falcon::Pseudo( m_line, Falcon::Pseudo::imm_string, str );
                m_state = e_line;
                return STRING;
@@ -460,7 +457,7 @@ int AsmLexer::lex()
                // else, it's a name
                if ( m_bDirective )
                {
-                  *VALUE = new Falcon::Pseudo( m_line, Falcon::Pseudo::imm_string, m_compiler->addString( m_string ) );
+                  *VALUE = new Falcon::Pseudo( m_line, Falcon::Pseudo::imm_string, (String *) m_compiler->addString( m_string ) );
                   return NAME;
                }
 
@@ -568,11 +565,21 @@ int AsmLexer::checkTokens()
             *VALUE = regA_Inst();
             return REG_A;
          }
-         else if ( m_string.compareIgnoreCase( "B" ) == 0 )
+         if ( m_string.compareIgnoreCase( "B" ) == 0 )
          {
             *VALUE = regB_Inst();
             return REG_B;
          }
+         if ( m_string.compareIgnoreCase( "T" ) == 0 )
+         {
+            *VALUE = true_Inst();
+            return TRUE_TOKEN;
+         }
+         if ( m_string.compareIgnoreCase( "F" ) == 0 )
+         {
+            *VALUE = false_Inst();
+            return FALSE_TOKEN;
+         }
       break;
 
       case 2:
@@ -602,6 +609,16 @@ int AsmLexer::checkTokens()
             *VALUE = regS2_Inst();
             return REG_S2;
          }
+         else if ( m_string.compareIgnoreCase( "L1" ) == 0 )
+         {
+            *VALUE = regL1_Inst();
+            return REG_L1;
+         }
+         else if ( m_string.compareIgnoreCase( "L2" ) == 0 )
+         {
+            *VALUE = regL2_Inst();
+            return REG_L2;
+         }
       break;
 
       case 3:
@@ -672,6 +689,8 @@ int AsmLexer::checkTokens()
              return I_WRT;
          else if ( m_string.compareIgnoreCase( "MOD" ) == 0 )
              return I_MOD;
+         else if ( m_string.compareIgnoreCase( "STO" ) == 0 )
+             return I_STO;
       break;
 
      case 4:
@@ -713,6 +732,10 @@ int AsmLexer::checkTokens()
              return I_STPS;
          if ( m_string.compareIgnoreCase( "TRAV" ) == 0 )
              return I_TRAV;
+         if ( m_string.compareIgnoreCase( "INCP" ) == 0 )
+             return I_INCP;
+         if ( m_string.compareIgnoreCase( "DECP" ) == 0 )
+             return I_DECP;
          if ( m_string.compareIgnoreCase( "TRAN" ) == 0 )
              return I_TRAN;
          if ( m_string.compareIgnoreCase( "TRAL" ) == 0 )
@@ -779,10 +802,6 @@ int AsmLexer::checkTokens()
              return I_PSIN;
          if ( m_string.compareIgnoreCase( "PASS" ) == 0 )
              return I_PASS;
-         if ( m_string.compareIgnoreCase( "FORI" ) == 0 )
-             return I_FORI;
-         if ( m_string.compareIgnoreCase( "FORN" ) == 0 )
-             return I_FORN;
          if ( m_string.compareIgnoreCase( "SHLS" ) == 0 )
              return I_SHLS;
          if ( m_string.compareIgnoreCase( "SHRS" ) == 0 )
@@ -892,6 +911,8 @@ int AsmLexer::checkDirectives()
             return DENDFUNC;
          if ( m_string == "inherit" )
             return DINHERIT;
+         if ( m_string == "istring" )
+            return DISTRING;
       break;
 
       case 8:
