@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language
    FILE: format.cpp
-   $Id: format.cpp,v 1.10 2007/06/30 11:53:22 jonnymind Exp $
 
    Format base class
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: mar apr 17 2007
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -1063,7 +1057,7 @@ void Format::applyPad( String &target, uint32 extraSize )
    }
 }
 
-bool Format::isReflective()
+bool Format::isReflective() const
 {
    return true;
 }
