@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: gencode.cpp
-   $Id: gencode.cpp,v 1.32 2007/08/18 11:08:07 jonnymind Exp $
 
    Short description
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: $DATE
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #include <falcon/setup.h>
@@ -157,7 +151,7 @@ void GenCode::generate( const SourceTree *st  )
    {
       // entry point
       gen_block( &st->statements() );
-      gen_pcode( P_END );
+      gen_pcode( P_RET );
    }
 
    // No need to generate the classes, as they are just definitions in the
@@ -311,6 +305,7 @@ void GenCode::gen_operand( const Value *stmt )
       }
       break;
 
+      case Value::t_imm_bool:
       case Value::t_self:
       case Value::t_sender:
          // do nothing
@@ -333,6 +328,7 @@ byte GenCode::gen_pdef( const c_varpar &elem )
          const Value *val = elem.m_content.value;
          switch( val->type() ) {
             case Value::t_nil: return P_PARAM_NIL;
+            case Value::t_imm_bool: return val->asBool() ? P_PARAM_TRUE : P_PARAM_FALSE;
             case Value::t_imm_integer: return P_PARAM_INT64;
             case Value::t_imm_string: return P_PARAM_STRID;
             case Value::t_imm_num: return P_PARAM_NUM;
@@ -355,6 +351,7 @@ byte GenCode::gen_pdef( const c_varpar &elem )
          const VarDef *vd = elem.m_content.vd;
          switch( vd->type() ) {
             case VarDef::t_nil: return P_PARAM_NIL;
+            case VarDef::t_bool: return vd->asBool() ? P_PARAM_TRUE : P_PARAM_FALSE;
             case VarDef::t_int: return P_PARAM_INT64;
             case VarDef::t_num: return P_PARAM_NUM;
             case VarDef::t_string: return P_PARAM_STRID;
@@ -383,8 +380,12 @@ byte GenCode::gen_pdef( const c_varpar &elem )
       case e_parNTD64: return P_PARAM_NTD64;
       case e_parA: return P_PARAM_REGA;
       case e_parB: return P_PARAM_REGB;
+      case e_parL1: return P_PARAM_REGL1;
+      case e_parL2: return P_PARAM_REGL2;
       case e_parS1: return P_PARAM_REGS1;
       case e_parS2: return P_PARAM_REGS2;
+      case e_parTRUE: return P_PARAM_TRUE;
+      case e_parFALSE: return P_PARAM_FALSE;
       case e_parSTRID: return P_PARAM_STRID;
    }
    // should not get here
@@ -651,88 +652,90 @@ void GenCode::gen_statement( const Statement *stmt )
       case Statement::t_give:
       {
          const StmtGive *give = static_cast<const StmtGive *>( stmt );
+         const ArrayDecl *give_to = give->objects();
 
-         if ( ! give->object()->isSimple() )
+         const ListElement *elem = give_to->begin();
+         while( elem != 0 )
          {
-            gen_complex_value( give->object() );
-         }
+            const Value *object = (const Value *) elem->data();
 
-         const ArrayDecl *attribs = give->attributes();
-         ListElement *iter = attribs->begin();
-         bool pushed = false;
-         bool peek = false;
-
-         while( iter != 0 )
-         {
-            const Value *val = (const Value *) iter->data();
-            byte mode;
-            ListElement *iter_next = iter->next();
-
-            if ( val->isExpr() && val->asExpr()->type() == Expression::t_not ) {
-               val = val->asExpr()->first();
-               mode = P_GIVN;
-            }
-            else
-               mode = P_GIVE;
-
-
-            if( val->isSimple() )
+            if ( ! object->isSimple() )
             {
-               if ( give->object()->isSimple() ) {
-                  gen_pcode( mode, give->object(), val );
-               }
-               else {
-                  if ( peek ) {
-                     if ( iter_next != 0 )
-                        gen_pcode( P_PEEK, e_parA );
-                     else {
-                        gen_pcode( P_POP, e_parA );
-                        pushed = false;
-                     }
-                     peek = false;
-                  }
-                  gen_pcode( mode, e_parA, val );
-               }
-            }
-            else {
-               if ( give->object()->isSimple() ) {
-                  gen_value( val );
-                  gen_pcode( mode, give->object(), e_parA );
-               }
-               else {
-                  if ( ! pushed ) {
-                     gen_pcode( P_PUSH, e_parA );
-                     pushed = true;
-                  }
-
-                  gen_value( val );
-                     if ( iter_next != 0 )
-                        gen_pcode( P_PEEK, e_parB );
-                     else {
-                        gen_pcode( P_POP, e_parB );
-                        pushed = false;
-                     }
-                  peek = true;
-
-                  gen_pcode( mode, e_parB, e_parA );
-               }
+               gen_complex_value( object );
             }
 
-            iter = iter_next;
-         }
-         if ( pushed ) {
-            gen_pcode( P_IPOP, c_param_fixed( 1 ) );
-         }
+            const ArrayDecl *attribs = give->attributes();
+            ListElement *iter = attribs->begin();
+            bool pushed = false;
+            bool peek = false;
 
+            while( iter != 0 )
+            {
+               const Value *val = (const Value *) iter->data();
+               byte mode;
+               ListElement *iter_next = iter->next();
+
+               if ( val->isExpr() && val->asExpr()->type() == Expression::t_not ) {
+                  val = val->asExpr()->first();
+                  mode = P_GIVN;
+               }
+               else
+                  mode = P_GIVE;
+
+
+               if( val->isSimple() )
+               {
+                  if ( object->isSimple() ) {
+                     gen_pcode( mode, object, val );
+                  }
+                  else {
+                     if ( peek ) {
+                        if ( iter_next != 0 )
+                           gen_pcode( P_PEEK, e_parA );
+                        else {
+                           gen_pcode( P_POP, e_parA );
+                           pushed = false;
+                        }
+                        peek = false;
+                     }
+                     gen_pcode( mode, e_parA, val );
+                  }
+               }
+               else {
+                  if ( object->isSimple() ) {
+                     gen_value( val );
+                     gen_pcode( mode, object, e_parA );
+                  }
+                  else {
+                     if ( ! pushed ) {
+                        gen_pcode( P_PUSH, e_parA );
+                        pushed = true;
+                     }
+
+                     gen_value( val );
+                        if ( iter_next != 0 )
+                           gen_pcode( P_PEEK, e_parB );
+                        else {
+                           gen_pcode( P_POP, e_parB );
+                           pushed = false;
+                        }
+                     peek = true;
+
+                     gen_pcode( mode, e_parB, e_parA );
+                  }
+               }
+
+               iter = iter_next;
+            }
+            if ( pushed ) {
+               gen_pcode( P_IPOP, c_param_fixed( 1 ) );
+            }
+
+            elem = elem->next();
+         }
       }
       break;
 
-      case Statement::t_assignment:
-      {
-         const StmtAssignment *ass = static_cast<const StmtAssignment *>(stmt);
-         gen_load( ass->destination(), ass->value() );
-      }
-      break;
 
       case Statement::t_unref:
       {
@@ -747,80 +750,6 @@ void GenCode::gen_statement( const Statement *stmt )
       }
       break;
 
-      case Statement::t_autoadd:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( P_ADDS, assign->destination(), assign->value() );
-      }
-      break;
-      case Statement::t_autosub:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( P_SUBS, assign->destination(), assign->value() );
-      }
-      break;
-
-      case Statement::t_automul:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( P_MULS, assign->destination(), assign->value() );
-      }
-      break;
-
-      case Statement::t_autodiv:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( P_DIVS, assign->destination(), assign->value() );
-      }
-      break;
-
-      case Statement::t_automod:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( P_MODS, assign->destination(), assign->value() );
-      }
-      break;
-
-      case Statement::t_autoband:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( P_ANDS, assign->destination(), assign->value() );
-      }
-      break;
-
-      case Statement::t_autobor:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( P_ORS, assign->destination(), assign->value() );
-      }
-      break;
-      case Statement::t_autobxor:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( P_XORS, assign->destination(), assign->value() );
-      }
-      break;
-
-      case Statement::t_autopow:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( P_POWS, assign->destination(), assign->value() );
-      }
-      break;
-
-      case Statement::t_autoshl:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( P_SHLS, assign->destination(), assign->value() );
-      }
-      break;
-
-      case Statement::t_autoshr:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( P_SHRS, assign->destination(), assign->value() );
-      }
-      break;
 
       case Statement::t_if:
       {
@@ -1063,59 +992,6 @@ void GenCode::gen_statement( const Statement *stmt )
       }
       break;
 
-      case Statement::t_for:
-      {
-         const StmtFor *elem = static_cast<const StmtFor *>( stmt );
-
-         c_jmptag tag( m_out );
-         m_labels_loop.pushBack( &tag );
-
-         if( elem->to()->isSimple() ) {
-            gen_pcode( P_PUSH, elem->to() );
-         }
-         else {
-            gen_value( elem->to() );
-            gen_pcode( P_PUSH, e_parA );
-         }
-
-         // save the step value
-         if( elem->step() != 0 )
-         {
-            if( elem->step()->isSimple() ) {
-               gen_pcode( P_PUSH, elem->step() );
-            }
-            else {
-               gen_value( elem->step() );
-               gen_pcode( P_PUSH, e_parA );
-            }
-         }
-         else
-            gen_pcode( P_PUSH, 0 );
-
-         // create the from value
-         if( elem->from()->isSimple() ) {
-            gen_pcode( P_FORI, c_param_fixed( tag.addQueryEnd() ), elem->counter(), elem->from() );
-         }
-         else
-         {
-            gen_value( elem->from() );
-            gen_pcode( P_FORI, c_param_fixed( tag.addQueryEnd() ), elem->counter(), e_parA );
-         }
-
-         tag.defineBegin();
-
-         if( ! elem->children().empty() ) {
-            gen_block( &elem->children() );
-         }
-
-         tag.defineNext();
-         gen_pcode( P_FORN, c_param_fixed( tag.addQueryBegin() ), elem->counter() );
-         tag.defineEnd();
-
-         m_labels_loop.popBack();
-      }
-      break;
-
       case Statement::t_propdef:
       {
          const StmtVarDef *pdef = static_cast<const StmtVarDef *>( stmt );
@@ -1156,7 +1032,7 @@ void GenCode::gen_statement( const Statement *stmt )
          //    [loop body]
          //    [If we have a forlast block  --
          //      TRAL (last)]
-         // [FORALL block]
+         // [FORMIDDLE block]
          //
          // TRAN begin
          // last:
@@ -1226,18 +1102,15 @@ void GenCode::gen_statement( const Statement *stmt )
             gen_block( &loop->children() );
          }
 
-         // if we have a last block, it must be executed after last element.
-         if( ! loop->lastBlock().empty() ) {
+         // generate the formiddle block
+         if( ! loop->middleBlock().empty() ) {
+            // skip it for the last element
             gen_pcode( P_TRAL, c_param_fixed( tag.addQueryElif( 0 ) ) );
+
+            gen_block( &loop->middleBlock() );
          }
 
-         // generate a forall block (after tral, so it won't bexecuted for the last element)
-         if( ! loop->allBlock().empty() )
-         {
-            gen_block( &loop->allBlock() );
-         }
-
-         // after the forall block we have the next element pick
+         // after the formiddle block we have the next element pick
          tag.defineNext();
 
          // second parameter 0 --> do not delete.
@@ -1416,72 +1289,71 @@ void GenCode::gen_statement( const Statement *stmt )
 }
 
 
-void GenCode::gen_inc_prefix( const Value *val, bool asExpr )
+void GenCode::gen_inc_prefix( const Value *val )
 {
    if ( val->isSimple() ) {
       gen_pcode( P_INC, val );
-      if( asExpr )
-         gen_pcode( P_LD, e_parA, val );
    }
    else {
-      gen_complex_value( val,true );
+      t_valType xValue = l_value;
+      gen_complex_value( val, xValue );
       gen_pcode( P_INC, e_parA );
+
+      if ( xValue == p_value )
+         gen_pcode( P_STP, e_parL1, e_parL2, e_parA );
+      else if ( xValue == v_value )
+         gen_pcode( P_STV, e_parL1, e_parL2, e_parA );
    }
 }
 
-void GenCode::gen_inc_postfix( const Value *val, bool asExpr )
+void GenCode::gen_inc_postfix( const Value *val )
 {
    if ( val->isSimple() ) {
-      if ( asExpr ) {
-         gen_pcode( P_LD, e_parA, val );
-      }
-      gen_pcode( P_INC, val );
+      gen_pcode( P_INCP, val );
    }
    else {
-      gen_complex_value( val, true );
-      if ( asExpr ) {
-         gen_pcode( P_LD, e_parB, e_parA );
-         gen_pcode( P_PUSH, e_parB );
-      }
+      t_valType xValue = l_value;
+      gen_complex_value( val, xValue );
+      gen_pcode( P_INCP, e_parA );
 
-      gen_pcode( P_INC, e_parA );
-
-      if ( asExpr )
-         gen_pcode( P_POP, e_parA );
+      if ( xValue == p_value )
+         gen_pcode( P_STP, e_parL1, e_parL2, e_parB );
+      else if ( xValue == v_value )
+         gen_pcode( P_STV, e_parL1, e_parL2, e_parB );
    }
 }
 
-void GenCode::gen_dec_prefix( const Value *val, bool asExpr )
+void GenCode::gen_dec_prefix( const Value *val )
 {
    if ( val->isSimple() ) {
       gen_pcode( P_DEC, val );
-      if( asExpr )
-         gen_pcode( P_LD, e_parA, val );
    }
    else {
-      gen_complex_value( val, true );
+      t_valType xValue = l_value;
+      gen_complex_value( val, xValue );
       gen_pcode( P_DEC, e_parA );
+
+      if ( xValue == p_value )
+         gen_pcode( P_STP, e_parL1, e_parL2, e_parA );
+      else if ( xValue == v_value )
+         gen_pcode( P_STV, e_parL1, e_parL2, e_parA );
    }
 }
 
-void GenCode::gen_dec_postfix( const Value *val, bool asExpr )
+void GenCode::gen_dec_postfix( const Value *val )
 {
    if ( val->isSimple() ) {
-      if ( asExpr ) {
-         gen_pcode( P_LD, e_parA, val );
-      }
-      gen_pcode( P_DEC, val );
+      gen_pcode( P_DECP, val );
    }
    else {
-      gen_complex_value( val, true );
-      if ( asExpr ) {
-         gen_pcode( P_LD, e_parB, e_parA );
-         gen_pcode( P_PUSH, e_parB );
-      }
+      t_valType xValue = l_value;
+      gen_complex_value( val, xValue );
+      gen_pcode( P_DECP, e_parA );
 
-      gen_pcode( P_DEC, e_parA );
-      if ( asExpr )
-         gen_pcode( P_POP, e_parA );
+      if ( xValue == p_value )
+         gen_pcode( P_STP, e_parL1, e_parL2, e_parB );
+      else if ( xValue == v_value )
+         gen_pcode( P_STV, e_parL1, e_parL2, e_parB );
    }
 }
 
@@ -1494,19 +1366,33 @@ void GenCode::gen_autoassign( byte opcode, const Value *target, const Value *sou
    {
       gen_complex_value( source );
       gen_pcode( opcode, target, e_parA );
-
    }
    else if ( source->isSimple() )
    {
-      gen_complex_value( target, true );
+      t_valType xValue = l_value;
+      gen_complex_value( target, xValue );
+
       gen_pcode( opcode, e_parA, source );
+
+      if ( xValue == p_value )
+         gen_pcode( P_STP, e_parL1, e_parL2, e_parA );
+      else if ( xValue == v_value )
+         gen_pcode( P_STV, e_parL1, e_parL2, e_parA );
    }
    else {
       gen_complex_value( source );
       gen_pcode( P_PUSH, e_parA );
-      gen_complex_value( target, true );
+
+      t_valType xValue = l_value;
+      gen_complex_value( target, xValue );
+
       gen_pcode( P_POP, e_parB );
       gen_pcode( opcode, e_parA, e_parB );
+
+      if ( xValue == p_value )
+         gen_pcode( P_STP, e_parL1, e_parL2, e_parA );
+      else if ( xValue == v_value )
+         gen_pcode( P_STV, e_parL1, e_parL2, e_parA );
    }
 }
 
@@ -1537,7 +1423,7 @@ void GenCode::gen_condition( const Value *stmt, int mode )
 void GenCode::gen_value( const Value *stmt )
 {
    if ( stmt->isSimple() ) {
-      gen_pcode( P_LD, e_parA, stmt );
+      gen_pcode( P_STO, e_parA, stmt );
    }
    else {
       gen_complex_value( stmt );
@@ -1546,7 +1432,7 @@ void GenCode::gen_value( const Value *stmt )
 
 
 
-void GenCode::gen_complex_value( const Value *stmt, bool assign )
+void GenCode::gen_complex_value( const Value *stmt, t_valType &xValue )
 {
    switch( stmt->type() )
    {
@@ -1572,7 +1458,7 @@ void GenCode::gen_complex_value( const Value *stmt, bool assign )
       break;
 
       case Value::t_expression:
-         gen_expression( stmt->asExpr(), assign );
+         gen_expression( stmt->asExpr(), xValue );
       break;
 
       case Value::t_range_decl:
@@ -1606,7 +1492,7 @@ void GenCode::gen_push( const Value *val )
 }
 
 
-void GenCode::gen_expression( const Expression *exp, bool assign )
+void GenCode::gen_expression( const Expression *exp, t_valType &xValue )
 {
 
    byte opname;
@@ -1625,6 +1511,7 @@ void GenCode::gen_expression( const Expression *exp, bool assign )
       case Expression::t_and:
       case Expression::t_or:
       {
+         xValue = l_value;
          char ifmode;
 
          if ( exp->type() == Expression::t_or )
@@ -1709,6 +1596,7 @@ void GenCode::gen_expression( const Expression *exp, bool assign )
       // it is better to handle the rest directly here.
       case Expression::t_iif:
       {
+         xValue = l_value;
          c_jmptag tag( m_out );
          m_labels.pushBack( &tag );
 
@@ -1727,7 +1615,7 @@ void GenCode::gen_expression( const Expression *exp, bool assign )
          // success case
          if( exp->second()->isSimple() )
          {
-            gen_pcode( P_LD, e_parA, exp->second() );
+            gen_pcode( P_STO, e_parA, exp->second() );
          }
          else
          {
@@ -1742,7 +1630,7 @@ void GenCode::gen_expression( const Expression *exp, bool assign )
          // success case
          if( exp->third()->isSimple() )
          {
-            gen_pcode( P_LD, e_parA, exp->third() );
+            gen_pcode( P_STO, e_parA, exp->third() );
          }
          else
          {
@@ -1754,48 +1642,106 @@ void GenCode::gen_expression( const Expression *exp, bool assign )
       }
       return;
 
-      case Expression::t_let:
+      case Expression::t_assign:
          // handle it as a load...
+         // don't change x-value
          gen_load( exp->first(), exp->second() );
-         // and eventually store the assignand to A...
-         // first cannot be A, as A cannot be expressly used in let.
-         if ( exp->second()->isSimple() ) {
-            gen_pcode( P_LD, e_parA, exp->second() );
-         }
-         // else already in a
       return;
 
-      case Expression::t_pre_inc: gen_inc_prefix( exp->first(), true ); return;
-      case Expression::t_pre_dec: gen_dec_prefix( exp->first(), true ); return;
-      case Expression::t_post_inc: gen_inc_postfix( exp->first(), true ); return;
-      case Expression::t_post_dec: gen_dec_postfix( exp->first(), true ); return;
+      case Expression::t_aadd:
+         xValue = l_value;
+         gen_autoassign( P_ADDS, exp->first(), exp->second() );
+      return;
+
+      case Expression::t_asub:
+         xValue = l_value;
+         gen_autoassign( P_SUBS, exp->first(), exp->second() );
+      return;
+
+      case Expression::t_amul:
+         xValue = l_value;
+         gen_autoassign( P_MULS, exp->first(), exp->second() );
+      return;
+
+      case Expression::t_adiv:
+         xValue = l_value;
+         gen_autoassign( P_DIVS, exp->first(), exp->second() );
+      return;
+
+      case Expression::t_amod:
+         xValue = l_value;
+         gen_autoassign( P_MODS, exp->first(), exp->second() );
+      return;
+
+      case Expression::t_aband:
+         xValue = l_value;
+         gen_autoassign( P_ANDS, exp->first(), exp->second() );
+      return;
+
+      case Expression::t_abor:
+         xValue = l_value;
+         gen_autoassign( P_ORS, exp->first(), exp->second() );
+      return;
+
+      case Expression::t_abxor:
+         xValue = l_value;
+         gen_autoassign( P_XORS, exp->first(), exp->second() );
+      return;
+
+      case Expression::t_apow:
+         xValue = l_value;
+         gen_autoassign( P_POWS, exp->first(), exp->second() );
+      return;
+
+      case Expression::t_ashl:
+         xValue = l_value;
+         gen_autoassign( P_SHLS, exp->first(), exp->second() );
+      return;
+
+      case Expression::t_ashr:
+         xValue = l_value;
+         gen_autoassign( P_SHRS, exp->first(), exp->second() );
+      return;
+
+      case Expression::t_pre_inc: gen_inc_prefix( exp->first() ); return;
+      case Expression::t_pre_dec: gen_dec_prefix( exp->first() ); return;
+      case Expression::t_post_inc: gen_inc_postfix( exp->first() ); return;
+      case Expression::t_post_dec: gen_dec_postfix( exp->first() ); return;
 
       case Expression::t_obj_access:
-         gen_load_from_deep( assign? P_LDPR: P_LDP, exp->first(), exp->second() );
+         xValue = p_value;
+         gen_load_from_deep( P_LDP, exp->first(), exp->second() );
       return;
 
       case Expression::t_array_access:
-         gen_load_from_deep( assign ? P_LDVR : P_LDV, exp->first(), exp->second() );
+         xValue = v_value;
+         gen_load_from_deep( P_LDV, exp->first(), exp->second() );
       return;
 
       case Expression::t_array_byte_access:
+         xValue = l_value;
          gen_load_from_deep( P_LSB, exp->first(), exp->second() );
       return;
 
       case Expression::t_funcall:
       case Expression::t_inherit:
       {
+         xValue = l_value;
          gen_funcall( exp, false );
       }
       // funcall is complete here
       return;
 
       case Expression::t_lambda:
-         gen_pcode( P_LD, e_parA, exp->first()->asSymbol() );
+         xValue = l_value;
+         gen_pcode( P_STO, e_parA, exp->first()->asSymbol() );
       return;
-
    }
 
+   // post-processing unary and binary operators.
+   // ++, -- and accessors are gone; safely change the l-value status.
+   xValue = l_value;
+
    // then, if there is still something to do, put the operands in place.
    if ( mode == 1 ) {  // unary?
       if ( exp->first()->isSimple() ) {
@@ -1879,19 +1825,38 @@ void GenCode::gen_range_decl( const RangeDecl *dcl )
    }
    else
    {
+      Value dummy; // defaults to nil
+      Value *rangeStep;
+      if ( dcl->rangeStep() == 0 )
+      {
+         dummy.setInteger( 0 );
+         rangeStep = &dummy;
+      }
+      else if ( dcl->rangeStep()->isSimple() )
+      {
+         rangeStep = dcl->rangeStep();
+      }
+      else
+      {
+         gen_complex_value( dcl->rangeStep() );
+         gen_pcode( P_PUSH, e_parA );
+         // we'll instruct GENR to get it via NIL as parameter
+         rangeStep = &dummy;
+      }
+
       if ( dcl->rangeStart()->isSimple() && dcl->rangeEnd()->isSimple() )
       {
-         gen_pcode( P_GENR, dcl->rangeStart(), dcl->rangeEnd() );
+         gen_pcode( P_GENR, dcl->rangeStart(), dcl->rangeEnd(), rangeStep );
       }
       else if ( dcl->rangeStart()->isSimple() )
       {
          gen_complex_value( dcl->rangeEnd() );
-         gen_pcode( P_GENR, dcl->rangeStart(), e_parA );
+         gen_pcode( P_GENR, dcl->rangeStart(), e_parA, rangeStep );
       }
       else if ( dcl->rangeEnd()->isSimple() )
       {
          gen_complex_value( dcl->rangeStart() );
-         gen_pcode( P_GENR, e_parA, dcl->rangeEnd() );
+         gen_pcode( P_GENR, e_parA, dcl->rangeEnd(), rangeStep );
 
       }
       else {
@@ -1899,7 +1864,7 @@ void GenCode::gen_range_decl( const RangeDecl *dcl )
          gen_pcode( P_PUSH, e_parA );
          gen_complex_value( dcl->rangeEnd() );
          gen_pcode( P_POP, e_parB );
-         gen_pcode( P_GENR, e_parB, e_parA );
+         gen_pcode( P_GENR, e_parB, e_parA, rangeStep );
       }
    }
 }
