@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: genhasm.cpp
-   $Id: genhasm.cpp,v 1.30 2007/08/18 11:08:08 jonnymind Exp $
 
    Generate Falcon Assembly from a Falcon syntactic tree.
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: $DATE
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #include <falcon/setup.h>
@@ -76,7 +70,7 @@ void GenHAsm::generate( const SourceTree *st )
       m_out->writeString( ".entry\n" );
 
       gen_block( &st->statements() );
-      m_out->writeString( "\tEND\n" );
+      m_out->writeString( "\tRET\n" );
    }
 
    // generate functions
@@ -280,7 +274,12 @@ void GenHAsm::gen_stringTable( const Module *mod )
       // for now all cstrings.
       String temp;
       str->escape( temp );
-      m_out->writeString( ".string \"" + temp + "\"\n" );
+      if ( str->exported() )
+         m_out->writeString( ".istring \"" );
+      else
+         m_out->writeString( ".string \"" );
+
+      m_out->writeString( temp + "\"\n" );
       ++count;
       str = mod->getString( count);
    }
@@ -685,95 +684,97 @@ void GenHAsm::gen_statement( const Statement *stmt )
       case Statement::t_give:
       {
          const StmtGive *give = static_cast<const StmtGive *>( stmt );
+         const ArrayDecl *give_to = give->objects();
+         const ListElement *elem = give_to->begin();
 
-         if ( ! give->object()->isSimple() )
+         while( elem != 0 )
          {
-            gen_complex_value( give->object() );
-         }
+            const Value *object = (const Value *) elem->data();
 
-         const ArrayDecl *attribs = give->attributes();
-         ListElement *iter = attribs->begin();
-         bool pushed = false;
-         bool peek = false;
-
-         while( iter != 0 )
-         {
-            const Value *val = (const Value *) iter->data();
-            String mode;
-            ListElement *iter_next = iter->next();
-
-            if ( val->isExpr() && val->asExpr()->type() == Expression::t_not ) {
-               val = val->asExpr()->first();
-               mode = "\tGIVN\t";
-            }
-            else
-               mode = "\tGIVE\t";
-
-
-            if( val->isSimple() )
+            if ( ! object->isSimple() )
             {
-               if ( give->object()->isSimple() ) {
-                  m_out->writeString( mode );
-                  gen_operand( give->object() );
-                  m_out->writeString( ", " );
-                  gen_operand( val );
+               gen_complex_value( object );
+            }
+
+            const ArrayDecl *attribs = give->attributes();
+            ListElement *iter = attribs->begin();
+            bool pushed = false;
+            bool peek = false;
+
+            while( iter != 0 )
+            {
+               const Value *val = (const Value *) iter->data();
+               String mode;
+               ListElement *iter_next = iter->next();
+
+               if ( val->isExpr() && val->asExpr()->type() == Expression::t_not ) {
+                  val = val->asExpr()->first();
+                  mode = "\tGIVN\t";
+               }
+               else
+                  mode = "\tGIVE\t";
+
+
+               if( val->isSimple() )
+               {
+                  if ( object->isSimple() ) {
+                     m_out->writeString( mode );
+                     gen_operand( object );
+                     m_out->writeString( ", " );
+                     gen_operand( val );
+                  }
+                  else {
+                     if ( peek ) {
+                        if ( iter_next != 0 )
+                           m_out->writeString( "\tPEEK\tA\n" );
+                        else {
+                           m_out->writeString( "\tPOP\tA\n" );
+                           pushed = false;
+                        }
+                        peek = false;
+                     }
+                     m_out->writeString( mode + "A, " );
+                     gen_operand( val );
+                  }
                }
                else {
-                  if ( peek ) {
+                  if ( object->isSimple() ) {
+                     gen_value( val );
+                     m_out->writeString( mode );
+                     gen_operand( object );
+                     m_out->writeString( ", A" );
+                  }
+                  else {
+                     if ( ! pushed ) {
+                        m_out->writeString( "\tPUSH A\n" );
+                        pushed = true;
+                     }
+
+                     gen_value( val );
                      if ( iter_next != 0 )
-                        m_out->writeString( "\tPEEK\tA\n" );
+                        m_out->writeString( "\tPEEK\tB\n" );
                      else {
-                        m_out->writeString( "\tPOP\tA\n" );
+                        m_out->writeString( "\tPOP\tB\n" );
                         pushed = false;
                      }
-                     peek = false;
+                     peek = true;
+
+                     m_out->writeString( mode + "B, A" );
                   }
-                  m_out->writeString( mode + "A, " );
-                  gen_operand( val );
                }
+
+               m_out->writeString( "\n" );
+               iter = iter_next;
             }
-            else {
-               if ( give->object()->isSimple() ) {
-                  gen_value( val );
-                  m_out->writeString( mode );
-                  gen_operand( give->object() );
-                  m_out->writeString( ", A" );
-               }
-               else {
-                  if ( ! pushed ) {
-                     m_out->writeString( "\tPUSH A\n" );
-                     pushed = true;
-                  }
-
-                  gen_value( val );
-                  if ( iter_next != 0 )
-                     m_out->writeString( "\tPEEK\tB\n" );
-                  else {
-                     m_out->writeString( "\tPOP\tB\n" );
-                     pushed = false;
-                  }
-                  peek = true;
-
-                  m_out->writeString( mode + "B, A" );
-               }
+            if ( pushed ) {
+               m_out->writeString( "\tIPOP\t1\n" );
             }
 
-            m_out->writeString( "\n" );
-            iter = iter_next;
+            elem = elem->next();
          }
-         if ( pushed ) {
-            m_out->writeString( "\tIPOP\t1\n" );
-         }
-
       }
       break;
 
-      case Statement::t_assignment:
-      {
-         const StmtAssignment *ass = static_cast<const StmtAssignment *>(stmt);
-         gen_load( ass->destination(), ass->value() );
-      }
-      break;
 
       case Statement::t_unref:
       {
@@ -790,81 +791,6 @@ void GenHAsm::gen_statement( const Statement *stmt )
       }
       break;
 
-      case Statement::t_autoadd:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( "ADDS", assign->destination(), assign->value() );
-      }
-      break;
-      case Statement::t_autosub:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( "SUBS", assign->destination(), assign->value() );
-      }
-      break;
-
-      case Statement::t_automul:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( "MULS", assign->destination(), assign->value() );
-      }
-      break;
-
-      case Statement::t_autodiv:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( "DIVS", assign->destination(), assign->value() );
-      }
-      break;
-
-      case Statement::t_automod:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( "MODS", assign->destination(), assign->value() );
-      }
-      break;
-
-      case Statement::t_autopow:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( "POWS", assign->destination(), assign->value() );
-      }
-      break;
-
-      case Statement::t_autoband:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( "ANDS", assign->destination(), assign->value() );
-      }
-      break;
-
-      case Statement::t_autobor:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( "ORS", assign->destination(), assign->value() );
-      }
-      break;
-      case Statement::t_autobxor:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( "XORS", assign->destination(), assign->value() );
-      }
-      break;
-
-      case Statement::t_autoshl:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( "SHLS", assign->destination(), assign->value() );
-      }
-      break;
-
-      case Statement::t_autoshr:
-      {
-         const StmtAssignment *assign = static_cast<const StmtAssignment *>(stmt);
-         gen_autoassign( "SHRS", assign->destination(), assign->value() );
-      }
-      break;
-
       case Statement::t_if:
       {
          const StmtIf *elem = static_cast<const StmtIf *>( stmt );
@@ -1048,74 +974,6 @@ void GenHAsm::gen_statement( const Statement *stmt )
       }
       break;
 
-      case Statement::t_for:
-      {
-         const StmtFor *elem = static_cast<const StmtFor *>( stmt );
-
-         int branch = m_loop_id++;
-         m_loops.pushBack( (void *) branch );
-         String branchStr;
-         branchStr.writeNumber( (int64) branch );
-
-         if( elem->to()->isSimple() ) {
-            m_out->writeString( "\tPUSH\t" );
-            gen_operand( elem->to() );
-            m_out->writeString( "\n" );
-         }
-         else {
-            gen_complex_value( elem->to() );
-            m_out->writeString( "\tPUSH\tA\n" );
-         }
-
-         // save the step value
-         if( elem->step() != 0 )
-         {
-            if( elem->step()->isSimple() ) {
-               m_out->writeString( "\tPUSH\t" );
-               gen_operand( elem->step() );
-               m_out->writeString( "\n" );
-            }
-            else {
-               gen_complex_value( elem->step() );
-               m_out->writeString( "\tPUSH\tA\n" );
-            }
-         }
-         else
-            m_out->writeString( "\tPUSH\t0\n" );
-
-         // create the from value
-         if( elem->from()->isSimple() ) {
-            m_out->writeString( "\tFORI\t_loop_end_" + branchStr + ", " );
-            gen_operand( elem->counter() );
-            m_out->writeString( ", " );
-            gen_operand( elem->from() );
-            m_out->writeString( "\n" );
-         }
-         else
-         {
-            gen_value( elem->from() );
-            m_out->writeString( "\tFORI\t_loop_end_" + branchStr + ", " );
-            gen_operand( elem->counter() );
-            m_out->writeString( ", A\n" );
-         }
-
-         m_out->writeString( "_loop_begin_" + branchStr + ":\n" );
-
-         if( ! elem->children().empty() ) {
-            gen_block( &elem->children() );
-         }
-
-         m_out->writeString( "_loop_next_" + branchStr + ":\n" );
-
-         m_out->writeString( "\tFORN\t_loop_begin_" + branchStr + ", " );
-         gen_operand( elem->counter() );
-         m_out->writeString( "\n" );
-         m_out->writeString( "_loop_end_" + branchStr + ":\n" );
-
-         // FORI and FORN will pop themselves if they are activated
-         m_loops.popBack();
-      }
-      break;
 
       case Statement::t_propdef:
       {
@@ -1162,7 +1020,7 @@ void GenHAsm::gen_statement( const Statement *stmt )
          //    [loop body]
          //    [If we have a forlast block  --
          //      TRAL (last)]
-         // [FORALL block]
+         // [FORMIDDLE block]
          //
          // TRAN begin
          // last:
@@ -1245,16 +1103,12 @@ void GenHAsm::gen_statement( const Statement *stmt )
             gen_block( &loop->children() );
          }
 
-         // do we have a forall block?
-
-         // do we have a last block? -- in this case, skip the all block and next loop
-         if( ! loop->lastBlock().empty() ) {
+         // generate the middle block
+         if( ! loop->middleBlock().empty() ) {
+            // skip it for the last element
             m_out->writeString( "\tTRAL\t_loop_tral_" + loopStr + "\n" );
-         }
 
-         // generate the all block
-         if( ! loop->allBlock().empty() ) {
-            gen_block( &loop->allBlock() );
+            gen_block( &loop->middleBlock() );
          }
 
          m_out->writeString( "_loop_next_" + loopStr + ":\n" );
@@ -1433,91 +1287,80 @@ void GenHAsm::dump_cases( int branch, const MapIterator &begin1 )
 }
 
 
-void GenHAsm::gen_inc_prefix( const Value *val, bool asExpr )
+void GenHAsm::gen_inc_prefix( const Value *val )
 {
    if ( val->isSimple() ) {
       m_out->writeString( "\tINC \t" );
       gen_operand( val );
       m_out->writeString( "\n" );
-      if( asExpr ) {
-         m_out->writeString( "\tLD  \tA, " );
-         gen_operand( val );
-         m_out->writeString( "\n" );
-      }
    }
    else {
-      gen_complex_value( val, true );
+      t_valType xValue = l_value;
+      gen_complex_value( val, xValue );
       m_out->writeString( "\tINC \tA\n" );
+      if ( xValue == p_value )
+         m_out->writeString( "\tSTP \tL1, L2, A\n" );
+      else if ( xValue == v_value )
+         m_out->writeString( "\tSTV \tL1, L2, A\n" );
    }
 }
 
-void GenHAsm::gen_inc_postfix( const Value *val, bool asExpr )
+void GenHAsm::gen_inc_postfix( const Value *val )
 {
    if ( val->isSimple() ) {
-      if ( asExpr ) {
-         m_out->writeString( "\tLD  \tA, " );
-         gen_operand( val );
-         m_out->writeString( "\n" );
-      }
-      m_out->writeString( "\tINC \t" );
+      m_out->writeString( "\tINCP\t" );
       gen_operand( val );
       m_out->writeString( "\n" );
    }
    else {
-      gen_complex_value( val, true );
-      if ( asExpr ) {
-         m_out->writeString( "\tLD \tB, A\n" );
-         m_out->writeString( "\tPUSH\tB\n" );
-      }
+      t_valType xValue = l_value;
+      gen_complex_value( val, xValue );
+      m_out->writeString( "\tINCP\tA\n" );
 
-      m_out->writeString( "\tINC \tA\n" );
-
-      if ( asExpr )
-         m_out->writeString( "\tPOP \tA\n" );
+      if ( xValue == p_value )
+         m_out->writeString( "\tSTP \tL1, L2, B\n" );
+      else if ( xValue == v_value )
+         m_out->writeString( "\tSTV \tL1, L2, B\n" );
    }
 }
 
-void GenHAsm::gen_dec_prefix( const Value *val, bool asExpr )
+void GenHAsm::gen_dec_prefix( const Value *val )
 {
    if ( val->isSimple() ) {
       m_out->writeString( "\tDEC \t" );
       gen_operand( val );
       m_out->writeString( "\n" );
-      if ( asExpr ) {
-         m_out->writeString( "\tLD  \tA, " );
-         gen_operand( val );
-         m_out->writeString( "\n" );
-      }
    }
    else {
-      gen_complex_value( val, true );
+      t_valType xValue = l_value;
+      gen_complex_value( val, xValue );
+
       m_out->writeString( "\tDEC \tA\n" );
+
+      if ( xValue == p_value )
+         m_out->writeString( "\tSTP \tL1, L2, A\n" );
+      else if ( xValue == v_value )
+         m_out->writeString( "\tSTV \tL1, L2, A\n" );
    }
 }
 
-void GenHAsm::gen_dec_postfix( const Value *val, bool asExpr )
+void GenHAsm::gen_dec_postfix( const Value *val )
 {
    if ( val->isSimple() ) {
-      if ( asExpr ) {
-         m_out->writeString( "\tLD  \tA, " );
-         gen_operand( val );
-         m_out->writeString( "\n" );
-      }
-      m_out->writeString( "\tDEC \t" );
+      m_out->writeString( "\tDECP\t" );
       gen_operand( val );
       m_out->writeString( "\n" );
    }
    else {
-      gen_complex_value( val, true );
-      if ( asExpr ) {
-         m_out->writeString( "\tLD  \tB, A\n" );
-         m_out->writeString( "\tPUSH\tB\n" );
-      }
+      t_valType xValue = l_value;
+      gen_complex_value( val, xValue );
 
-      m_out->writeString( "\tDEC \tA\n" );
+      m_out->writeString( "\tDECP\tA\n" );
 
-      if ( asExpr )
-         m_out->writeString( "\tPOP \tA\n" );
+      if ( xValue == p_value )
+         m_out->writeString( "\tSTP \tL1, L2, B\n" );
+      else if ( xValue == v_value )
+         m_out->writeString( "\tSTV \tL1, L2, B\n" );
    }
 }
 
@@ -1540,17 +1383,32 @@ void GenHAsm::gen_autoassign( const char *op, const Value *target, const Value *
    }
    else if ( source->isSimple() )
    {
-      gen_complex_value( target, true );
+      t_valType xValue = l_value;
+      gen_complex_value( target, xValue );
+
       m_out->writeString( "\t" + opstr + "\tA, " );
       gen_operand( source );
       m_out->writeString( "\n" );
+
+      if ( xValue == p_value )
+         m_out->writeString( "\tSTP \tL1, L2, A\n" );
+      else if ( xValue == v_value )
+         m_out->writeString( "\tSTV \tL1, L2, A\n" );
    }
    else {
       gen_complex_value( source );
       m_out->writeString( "\tPUSH\tA\n" );
-      gen_complex_value( target, true );
+
+      t_valType xValue = l_value;
+      gen_complex_value( target, xValue );
+
       m_out->writeString( "\tPOP \tB\n" );
       m_out->writeString( "\t" + opstr + "\tA, B\n" );
+
+      if ( xValue == p_value )
+         m_out->writeString( "\tSTP \tL1, L2, A\n" );
+      else if ( xValue == v_value )
+         m_out->writeString( "\tSTV \tL1, L2, A\n" );
    }
 }
 
@@ -1603,8 +1461,7 @@ void GenHAsm::gen_value( const Value *stmt, const char *prefix, const char *cpl_
 }
 
 
-
-void GenHAsm::gen_complex_value( const Value *stmt, bool assign )
+void GenHAsm::gen_complex_value( const Value *stmt, t_valType &xValue )
 {
    switch( stmt->type() )
    {
@@ -1629,7 +1486,7 @@ void GenHAsm::gen_complex_value( const Value *stmt, bool assign )
       break;
 
       case Value::t_expression:
-         gen_expression( stmt->asExpr(), assign );
+         gen_expression( stmt->asExpr(), xValue );
       break;
 
       case Value::t_range_decl:
@@ -1653,7 +1510,7 @@ void GenHAsm::gen_push( const Value *val )
          m_out->writeString( "\tPSHR\t$" + val->asReference()->asSymbol()->name() );
       else {
          gen_value( val->asReference() );
-         m_out->writeString( "\tPSHR\t$A" );
+         m_out->writeString( "\tPSHR\tA" );
       }
    }
    else {
@@ -1681,6 +1538,10 @@ void GenHAsm::gen_operand( const Value *stmt )
       }
       break;
 
+      case Value::t_imm_bool:
+         m_out->writeString( stmt->asBool() ? "T" : "F" );
+      break;
+
       case Value::t_imm_integer:
       {
          String intStr;
@@ -1718,7 +1579,7 @@ void GenHAsm::gen_operand( const Value *stmt )
    }
 }
 
-void GenHAsm::gen_expression( const Expression *exp, bool assign )
+void GenHAsm::gen_expression( const Expression *exp, t_valType &xValue )
 {
 
    String opname;
@@ -1737,6 +1598,8 @@ void GenHAsm::gen_expression( const Expression *exp, bool assign )
       case Expression::t_and:
       case Expression::t_or:
       {
+         xValue = l_value;
+
          String opname, ifmode;
          if ( exp->type() == Expression::t_or )
          {
@@ -1762,7 +1625,7 @@ void GenHAsm::gen_expression( const Expression *exp, bool assign )
             String branchStr;
             branchStr.writeNumber( (int64) branch );
 
-            m_out->writeString( "\tLD  \tA, " );
+            m_out->writeString( "\tSTO \tA, " );
             gen_operand(  exp->first() );
             m_out->writeString( "\n" );
             m_out->writeString( "\t" + ifmode + "\t_branch_orand_" );
@@ -1779,7 +1642,7 @@ void GenHAsm::gen_expression( const Expression *exp, bool assign )
             gen_value(  exp->first() );
             m_out->writeString( "\t" + ifmode + "\t_branch_orand_" + branchStr + ", A" + "\n" );
             // else we have to load the second in A
-            m_out->writeString( "\tLD  \tA, " );
+            m_out->writeString( "\tSTO \tA, " );
             gen_operand(  exp->second() );
             m_out->writeString( "\n" );
             m_out->writeString( "_branch_orand_" + branchStr + ":\n" );
@@ -1836,6 +1699,8 @@ void GenHAsm::gen_expression( const Expression *exp, bool assign )
       // it is better to handle the rest directly here.
       case Expression::t_iif:
       {
+         xValue = l_value;
+
          int32 branch = m_branch_id++;
          String branchStr;
          branchStr.writeNumber( (int64) branch );
@@ -1854,7 +1719,7 @@ void GenHAsm::gen_expression( const Expression *exp, bool assign )
          // success case
          if( exp->second()->isSimple() )
          {
-            m_out->writeString( "\tLD  \tA, " );
+            m_out->writeString( "\tSTO  \tA, " );
             gen_operand( exp->second() );
             m_out->writeString( "\n" );
          }
@@ -1870,7 +1735,7 @@ void GenHAsm::gen_expression( const Expression *exp, bool assign )
          // success case
          if( exp->third()->isSimple() )
          {
-            m_out->writeString( "\tLD  \tA, " );
+            m_out->writeString( "\tSTO  \tA, " );
             gen_operand( exp->third() );
             m_out->writeString( "\n" );
          }
@@ -1884,69 +1749,108 @@ void GenHAsm::gen_expression( const Expression *exp, bool assign )
       }
       return;
 
-      case Expression::t_let:
+      case Expression::t_assign:
          // handle it as a load...
+         // don't change x-value
          gen_load( exp->first(), exp->second() );
-         // and eventually store the assignand to a...
-         // first cannot be A, as A cannot be expressly used in let.
-         if ( exp->second()->isSimple() ) {
-            mode = 3; // unary for A...
-            m_out->writeString( "\tLD  \tA, " );
-            gen_operand( exp->second() );
-            m_out->writeString( "\n" );
-         }
-         // else already in A
          return;
-      break;
 
-      case Expression::t_pre_inc: gen_inc_prefix( exp->first(), true ); return;
-      case Expression::t_pre_dec: gen_dec_prefix( exp->first(), true ); return;
-      case Expression::t_post_inc: gen_inc_postfix( exp->first(), true ); return;
-      case Expression::t_post_dec: gen_dec_postfix( exp->first(), true ); return;
+      case Expression::t_aadd:
+         xValue = l_value;
+         gen_autoassign( "ADDS", exp->first(), exp->second() );
+         return;
+
+      case Expression::t_asub:
+         xValue = l_value;
+         gen_autoassign( "SUBS", exp->first(), exp->second() );
+         return;
+
+      case Expression::t_amul:
+         xValue = l_value;
+         gen_autoassign( "MULS", exp->first(), exp->second() );
+         return;
+
+      case Expression::t_adiv:
+         xValue = l_value;
+         gen_autoassign( "DIVS", exp->first(), exp->second() );
+         return;
+
+      case Expression::t_amod:
+         xValue = l_value;
+         gen_autoassign( "MODS", exp->first(), exp->second() );
+         return;
+
+      case Expression::t_apow:
+         xValue = l_value;
+         gen_autoassign( "POWS", exp->first(), exp->second() );
+         return;
+
+      case Expression::t_aband:
+         xValue = l_value;
+         gen_autoassign( "ANDS", exp->first(), exp->second() );
+         return;
+
+      case Expression::t_abor:
+         xValue = l_value;
+         gen_autoassign( "ORS", exp->first(), exp->second() );
+         return;
+
+      case Expression::t_abxor:
+         xValue = l_value;
+         gen_autoassign( "XORS", exp->first(), exp->second() );
+         return;
+
+      case Expression::t_ashl:
+         xValue = l_value;
+         gen_autoassign( "SHLS", exp->first(), exp->second() );
+         return;
+
+      case Expression::t_ashr:
+         xValue = l_value;
+         gen_autoassign( "SHRS", exp->first(), exp->second() );
+         return;
+
+      case Expression::t_pre_inc: gen_inc_prefix( exp->first() ); return;
+      case Expression::t_pre_dec: gen_dec_prefix( exp->first() ); return;
+      case Expression::t_post_inc: gen_inc_postfix( exp->first() ); return;
+      case Expression::t_post_dec: gen_dec_postfix( exp->first() ); return;
 
       case Expression::t_obj_access:
-         gen_load_from_deep( assign ? "LDPR" : "LDP ", exp->first(), exp->second() );
+         xValue = p_value;
+         gen_load_from_deep( "LDP ", exp->first(), exp->second() );
       return;
 
       case Expression::t_array_access:
-         gen_load_from_deep( assign ? "LDVR" : "LDV ", exp->first(), exp->second() );
+         xValue = v_value;
+         gen_load_from_deep( "LDV ", exp->first(), exp->second() );
       return;
 
       case Expression::t_array_byte_access:
+         xValue = l_value;
          gen_load_from_deep( "LSB ", exp->first(), exp->second() );
       return;
 
       case Expression::t_funcall:
       case Expression::t_inherit:
       {
+         xValue = l_value;
          gen_funcall( exp, false );
       }
       // funcall is complete here
       return;
 
-      case Expression::t_funval:
-      {
-         if ( exp->first()->isSimple() )
-         {
-            m_out->writeString( "\tLVAL\t" );
-            gen_operand( exp->first() );
-            m_out->writeString( ", A\n" );
-         }
-         else {
-            gen_complex_value( exp->first() );
-            m_out->writeString( "\tLVAL\tA, A\n" );
-         }
-      }
-      return;
-
       case Expression::t_lambda:
       {
-         m_out->writeString( "\tLD  \tA, $" + exp->first()->asSymbol()->name() + "\n" );
+         xValue = l_value;
+         m_out->writeString( "\tSTO \tA, $" + exp->first()->asSymbol()->name() + "\n" );
       }
       return;
-
    }
 
+   // post-processing unary and binary operators.
+   // ++, -- and accessors are gone; safely change the l-value status.
+   xValue = l_value;
+
    // then, if there is still something to do, put the operands in place.
    if ( mode == 1 ) {  // unary?
       if ( exp->first()->isSimple() ) {
@@ -1986,7 +1890,6 @@ void GenHAsm::gen_expression( const Expression *exp, bool assign )
    }
 
    m_out->writeString( "\n" );
-
 }
 
 void GenHAsm::gen_dict_decl( const DictDecl *dcl )
@@ -2046,12 +1949,33 @@ void GenHAsm::gen_range_decl( const RangeDecl *dcl )
    }
    else
    {
+      Value dummy; // defaults to nil
+      Value *rangeStep;
+      if ( dcl->rangeStep() == 0 )
+      {
+         dummy.setInteger( 0 );
+         rangeStep = &dummy;
+      }
+      else if ( dcl->rangeStep()->isSimple() )
+      {
+         rangeStep = dcl->rangeStep();
+      }
+      else
+      {
+         gen_complex_value( dcl->rangeStep() );
+         m_out->writeString( "\tPUSH\tA\n" );
+         // we'll instruct GENR to get it via NIL as parameter
+         rangeStep = &dummy;
+      }
+
       if ( dcl->rangeStart()->isSimple() && dcl->rangeEnd()->isSimple() )
       {
          m_out->writeString( "\tGENR\t" );
          gen_operand( dcl->rangeStart() );
          m_out->writeString( ", " );
          gen_operand( dcl->rangeEnd() );
+         m_out->writeString( ", " );
+         gen_operand( rangeStep );
          m_out->writeString( "\n" );
       }
       else if ( dcl->rangeStart()->isSimple() )
@@ -2059,13 +1983,17 @@ void GenHAsm::gen_range_decl( const RangeDecl *dcl )
          gen_complex_value( dcl->rangeEnd() );
          m_out->writeString( "\tGENR\t" );
          gen_operand( dcl->rangeStart() );
-         m_out->writeString( ", A\n" );
+         m_out->writeString( ", A, " );
+         gen_operand( rangeStep );
+         m_out->writeString( "\n" );
       }
       else if ( dcl->rangeEnd()->isSimple() )
       {
          gen_complex_value( dcl->rangeStart() );
          m_out->writeString( "\tGENR\tA, " );
          gen_operand( dcl->rangeEnd() );
+         m_out->writeString( ", " );
+         gen_operand( rangeStep );
          m_out->writeString( "\n" );
       }
       else {
@@ -2073,7 +2001,9 @@ void GenHAsm::gen_range_decl( const RangeDecl *dcl )
          m_out->writeString( "\tPUSH\tA\n" );
          gen_complex_value( dcl->rangeEnd() );
          m_out->writeString( "\tPOP \tB\n" );
-         m_out->writeString( "\tGENR\tB, A\n" );
+         m_out->writeString( "\tGENR\tB, A, ");
+         gen_operand( rangeStep );
+         m_out->writeString( "\n" );
       }
    }
 }
