@@ -1,7 +1,6 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: gentree.cpp
-   $Id: gentree.cpp,v 1.16 2007/08/01 13:18:01 jonnymind Exp $
 
    Short description
    -------------------------------------------------------------------
@@ -13,10 +12,6 @@
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #include <falcon/gentree.h>
@@ -161,11 +156,10 @@ void GenTree::generate( const Statement *cmp, const char *specifier, bool sameli
       case Statement::t_give:
       {
          const StmtGive *give = static_cast< const StmtGive *>( cmp );
-
          m_out->writeString( "GIVE " );
-         gen_value( give->object() );
-         m_out->writeString( " " );
          gen_array( give->attributes() );
+         m_out->writeString( " to " );
+         gen_array( give->objects() );
          m_out->writeString( "\n" );
       }
       break;
@@ -181,47 +175,6 @@ void GenTree::generate( const Statement *cmp, const char *specifier, bool sameli
       }
       break;
 
-      case Statement::t_assignment:
-         m_out->writeString( "ASSIGN " );
-         gen_value( static_cast< const StmtAssignment *>( cmp )->destination() );
-         m_out->writeString( " = " );
-         gen_value( static_cast< const StmtAssignment *>( cmp )->value() );
-         m_out->writeString( "\n" );
-      break;
-
-
-      case Statement::t_autoadd:
-      case Statement::t_autosub:
-      case Statement::t_automul:
-      case Statement::t_autodiv:
-      case Statement::t_automod:
-      case Statement::t_autoband:
-      case Statement::t_autobor:
-      case Statement::t_autobxor:
-      case Statement::t_autoshl:
-      case Statement::t_autoshr:
-      {
-         const char *op;
-         switch (  cmp->type() ) {
-            case Statement::t_autoadd: op = " += "; break;
-            case Statement::t_autosub: op = " -= "; break;
-            case Statement::t_automul: op = " *= "; break;
-            case Statement::t_autodiv: op = " /= "; break;
-            case Statement::t_automod: op = " %= "; break;
-            case Statement::t_autoband: op = " &= "; break;
-            case Statement::t_autobor: op = " |= "; break;
-            case Statement::t_autobxor: op = " ^= "; break;
-            case Statement::t_autoshl: op = " <<= "; break;
-            case Statement::t_autoshr: op = " >>= "; break;
-         }
-         m_out->writeString( "AUTO_ASSIGN " );
-         gen_value( static_cast< const StmtAssignment *>( cmp )->destination() );
-         m_out->writeString( op);
-         gen_value( static_cast< const StmtAssignment *>( cmp )->value() );
-         m_out->writeString( "\n" );
-      }
-      break;
-
       case Statement::t_if:
       {
          m_out->writeString( "IF " );
@@ -404,22 +357,6 @@ void GenTree::generate( const Statement *cmp, const char *specifier, bool sameli
       }
       break;
 
-      case Statement::t_for:
-      {
-         m_out->writeString( "FOR " );
-         const StmtFor *sfor = static_cast< const StmtFor *>( cmp );
-         gen_value( sfor->counter() );
-         m_out->writeString( " = " );
-         gen_value( sfor->from() );
-         m_out->writeString( " TO " );
-         gen_value( sfor->to() );
-         m_out->writeString( " STEP " );
-         gen_value( sfor->step() );
-         m_out->writeString( "\n" );
-         gen_block( sfor->children(), depth );
-      }
-      break;
-
       case Statement::t_global:
       {
          m_out->writeString( "GLOBAL " );
@@ -444,7 +381,7 @@ void GenTree::generate( const Statement *cmp, const char *specifier, bool sameli
          m_out->writeString( "\n" );
          gen_block( sfor->children(), depth );
          gen_block( sfor->firstBlock(), depth, "FORFIRST" );
-         gen_block( sfor->allBlock(), depth, "FORALL" );
+         gen_block( sfor->middleBlock(), depth, "FORMIDDLE" );
          gen_block( sfor->lastBlock(), depth, "FORLAST" );
       }
       break;
@@ -573,6 +510,10 @@ void GenTree::gen_value( const Value *val )
    switch( val->type() )
    {
       case Value::t_nil: m_out->writeString( "nil" ); break;
+      case Value::t_imm_bool:
+            m_out->writeString( val->asBool() ? "true": "false" );
+      break;
+
       case Value::t_imm_integer: {
          String intStr;
          intStr.writeNumber( val->asInteger() );
@@ -601,6 +542,12 @@ void GenTree::gen_value( const Value *val )
          if ( ! val->asRange()->isOpen() )
          {
             gen_value(  val->asRange()->rangeEnd() ) ;
+            if ( val->asRange()->rangeStep() != 0 )
+            {
+               m_out->writeString( ":" );
+               gen_value(  val->asRange()->rangeStep() );
+            }
+
          }
          m_out->writeString( "]" );
       break;
@@ -680,13 +627,25 @@ void GenTree::gen_expression( const Expression *exp )
       case Expression::t_and: type = 2; name = "and"; break;
 
       case Expression::t_iif: type = 3; break;
-      case Expression::t_let: type = 4; break;
+
+      case Expression::t_assign: type = 4; name = " = "; break;
+      case Expression::t_aadd: type = 4; name = " += "; break;
+      case Expression::t_asub: type = 4; name = " -= "; break;
+      case Expression::t_amul: type = 4; name = " *= "; break;
+      case Expression::t_adiv: type = 4; name = " /= "; break;
+      case Expression::t_amod: type = 4; name = " %= "; break;
+      case Expression::t_apow: type = 4; name = " *= "; break;
+      case Expression::t_aband: type = 4; name = " &= "; break;
+      case Expression::t_abor: type = 4; name = " |= "; break;
+      case Expression::t_abxor: type = 4; name = " ^= "; break;
+      case Expression::t_ashl: type = 4; name = " <<= "; break;
+      case Expression::t_ashr: type = 4; name = " >>= "; break;
+
       case Expression::t_array_access: type = 5; break;
       case Expression::t_array_byte_access: type = 10; break;
       case Expression::t_obj_access: type = 6; break;
       case Expression::t_funcall: type = 7; break;
       case Expression::t_lambda: type = 8; break;
-      case Expression::t_funval: type = 9; break;
    }
 
    switch( type )
@@ -720,9 +679,9 @@ void GenTree::gen_expression( const Expression *exp )
       break;
 
       case 4:
-         m_out->writeString( "let(" );
+         m_out->writeString( "( " );
          gen_value( exp->first() );
-         m_out->writeString( " = " );
+         m_out->writeString( name );
          gen_value( exp->second() );
          m_out->writeString( " )" );
       break;
@@ -752,11 +711,6 @@ void GenTree::gen_expression( const Expression *exp )
          m_out->writeString( exp->first()->asSymbol()->name() );
       break;
 
-      case 9:
-         m_out->writeString( "VALUE OF " );
-         gen_value( exp->first() );
-      break;
-
       case 10:
          gen_value( exp->first() );
          m_out->writeString( "[[ " );
