@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language
    FILE: itemserial.cpp
-   $Id: itemserial.cpp,v 1.12 2007/08/11 00:11:54 jonnymind Exp $
 
    Item serialization methods
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: dom gen 28 2007
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -31,6 +25,7 @@
 #include <falcon/stream.h>
 #include <falcon/lineardict.h>
 #include <falcon/attribute.h>
+#include <falcon/membuf.h>
 
 namespace Falcon {
 
@@ -50,7 +45,7 @@ bool Item::serialize_symbol( Stream *file, const Symbol *sym ) const
 }
 
 
-bool Item::serialize_function( Stream *file, const Symbol *func, VMachine *vm ) const
+bool Item::serialize_function( Stream *file, const Symbol *func ) const
 {
    byte type = FLC_ITEM_FUNC;
    file->write( &type, 1 );
@@ -76,9 +71,19 @@ bool Item::serialize_function( Stream *file, const Symbol *func, VMachine *vm )
    return true;
 }
 
-bool Item::serialize_object( Stream *file, const CoreObject *obj, VMachine *vm ) const
+bool Item::serialize_object( Stream *file, const CoreObject *obj, bool bLive ) const
 {
+   // if we're not live, we can't have user data
+   if ( ! bLive && obj->getUserData() != 0 )
+   {
+      obj->origin()->raiseError( e_unserializable, "Item::serialize" );
+      return false;
+   }
+
    byte type = FLC_ITEM_OBJECT;
+   if ( bLive )
+      type |= 0x80;
+
    file->write( &type, 1 );
 
    // write the class symbol so that it can be rebuilt back.
@@ -89,7 +94,7 @@ bool Item::serialize_object( Stream *file, const CoreObject *obj, VMachine *vm )
    file->write((byte *) &len, sizeof(len) );
 
    for( uint32 i = 0; i < len; i ++ ) {
-      if ( obj->getPropertyAt( i ).serialize( file, vm ) != sc_ok )
+      if ( obj->getPropertyAt( i ).serialize( file, bLive ) != sc_ok )
          return false;
    }
 
@@ -117,20 +122,75 @@ bool Item::serialize_object( Stream *file, const CoreObject *obj, VMachine *vm )
       head = head->next();
    }
 
+   // finally, clone if live
+   if ( bLive )
+   {
+      if ( obj->getUserData() != 0 )
+      {
+         UserData *cloned = obj->getUserData()->clone();
+         if ( cloned == 0 )
+         {
+            obj->origin()->raiseError( e_uncloneable, "Item::serialize" );
+            return false;
+         }
+         file->write( (byte *) &cloned, sizeof( cloned ) );
+      }
+      else {
+         void *data = 0;
+         file->write( (byte *) &data, sizeof( data ) );
+      }
+
+      if ( ! file->good() )
+      {
+         return false;
+      }
+   }
+
    return true;
 }
 
 
-Item::e_sercode Item::serialize( Stream *file, VMachine *vm ) const
+bool Item::serialize_class( Stream *file, const CoreClass *cls ) const
+{
+   byte type = FLC_ITEM_CLASS;
+   file->write( &type, 1 );
+
+   LiveModule *lmod = cls->liveModule();
+
+   // Write the live module name
+   lmod->name().serialize( file );
+   if ( ! file->good() )
+         return false;
+
+   // and the class name
+   cls->symbol()->name().serialize( file );
+   if ( ! file->good() )
+         return false;
+
+   return true;
+}
+
+
+Item::e_sercode Item::serialize( Stream *file, bool bLive ) const
 {
    if( file->bad() )
       return sc_ferror;
 
    switch( this->type() )
    {
+      case FLC_ITEM_BOOL:
+      {
+         byte type = FLC_ITEM_BOOL;
+         file->write((byte *) &type, 1 );
+
+         byte bval = this->asBoolean() ? 1 : 0;
+         file->write( (byte *) &bval, sizeof( bval ) );
+      }
+      break;
+
       case FLC_ITEM_INT:
       {
-         char type = FLC_ITEM_INT;
+         byte type = FLC_ITEM_INT;
          file->write((byte *) &type, 1 );
 
          int64 val = endianInt64( this->asInteger() );
@@ -140,22 +200,24 @@ Item::e_sercode Item::serialize( Stream *file, VMachine *vm ) const
 
       case FLC_ITEM_RANGE:
       {
-         char type = FLC_ITEM_RANGE;
+         byte type = FLC_ITEM_RANGE;
          file->write((byte *) &type, 1 );
 
          int32 val1 = endianInt32(this->asRangeStart());
          int32 val2 = endianInt32(this->asRangeEnd());
+         int32 val3 = endianInt32(this->asRangeStep());
          byte isOpen = this->asRangeIsOpen() ? 1 : 0;
 
          file->write( (byte *) &val1, sizeof( val1 ) );
          file->write( (byte *) &val2, sizeof( val2 ) );
+         file->write( (byte *) &val3, sizeof( val3 ) );
          file->write( (byte *) &isOpen, sizeof( isOpen ) );
       }
       break;
 
       case FLC_ITEM_NUM:
       {
-         char type = FLC_ITEM_NUM;
+         byte type = FLC_ITEM_NUM;
          file->write((byte *) &type, 1 );
 
          numeric val = endianNum( this->asNumeric() );
@@ -165,8 +227,8 @@ Item::e_sercode Item::serialize( Stream *file, VMachine *vm ) const
 
       case FLC_ITEM_ATTRIBUTE:
       {
-         char type = FLC_ITEM_ATTRIBUTE;
-         file->write((byte *) &type, 1 );
+         byte type = FLC_ITEM_ATTRIBUTE;
+         file->write( &type, 1 );
 
          this->asAttribute()->name().serialize( file );
       }
@@ -174,16 +236,26 @@ Item::e_sercode Item::serialize( Stream *file, VMachine *vm ) const
 
       case FLC_ITEM_STRING:
       {
-         char type = FLC_ITEM_STRING;
+         byte type = FLC_ITEM_STRING;
          file->write((byte *) &type, 1 );
-
          this->asString()->serialize( file );
       }
       break;
 
+      case FLC_ITEM_MEMBUF:
+      {
+         byte type = FLC_ITEM_MEMBUF;
+         if ( bLive )
+            type |= 0x80;
+
+         file->write( &type, 1 );
+         this->asMemBuf()->serialize( file, bLive );
+      }
+      break;
+
       case FLC_ITEM_FBOM:
       {
-         char type = FLC_ITEM_FBOM;
+         byte type = FLC_ITEM_FBOM;
          file->write((byte *) &type, 1 );
 
          type = this->getFbomMethod();
@@ -191,20 +263,20 @@ Item::e_sercode Item::serialize( Stream *file, VMachine *vm ) const
 
          Item ifbom;
          getFbomItem( ifbom );
-         ifbom.serialize( file, vm );
+         ifbom.serialize( file, bLive );
       }
       break;
 
       case FLC_ITEM_ARRAY:
       {
-         char type = FLC_ITEM_ARRAY;
+         byte type = FLC_ITEM_ARRAY;
          file->write((byte *) &type, 1 );
 
-         CoreArray *array = this->asArray();
-         int32 len = endianInt32( array->length() );
+         CoreArray &array = *this->asArray();
+         int32 len = endianInt32( array.length() );
          file->write( (byte *) &len, sizeof( len ) );
-         for( uint32 i = 0; i < array->length(); i ++ ) {
-            array->elements()[i].serialize( file, vm );
+         for( uint32 i = 0; i < array.length(); i ++ ) {
+            array[i].serialize( file, bLive );
             if( ! file->good() )
                return sc_ferror;
          }
@@ -213,8 +285,8 @@ Item::e_sercode Item::serialize( Stream *file, VMachine *vm ) const
 
       case FLC_ITEM_DICT:
       {
-         char type = FLC_ITEM_DICT;
-         file->write((byte *) &type, 1 );
+         byte type = FLC_ITEM_DICT;
+         file->write( &type, 1 );
 
          CoreDict *dict = this->asDict();
          int32 len = endianInt32( dict->length() );
@@ -224,10 +296,10 @@ Item::e_sercode Item::serialize( Stream *file, VMachine *vm ) const
          dict->traverseBegin();
          while( dict->traverseNext( key, value ) )
          {
-            key.serialize( file, vm );
+            key.serialize( file, bLive );
             if( ! file->good() )
                return sc_ferror;
-            value.serialize( file, vm );
+            value.serialize( file, bLive );
             if( ! file->good() )
                return sc_ferror;
          }
@@ -235,30 +307,47 @@ Item::e_sercode Item::serialize( Stream *file, VMachine *vm ) const
       break;
 
       case FLC_ITEM_FUNC:
-         serialize_function( file, this->asFunction(), vm );
+         serialize_function( file, this->asFunction() );
       break;
 
       case FLC_ITEM_METHOD:
       {
          byte type = FLC_ITEM_METHOD;
-         file->write( &type, 1 );
-
-         serialize_object( file, this->asMethodObject(), vm );
-         serialize_function( file, this->asMethodFunction(), vm );
+         if ( bLive )
+         {
+            type |= 0x80;
+            file->write( &type, 1 );
+            void *obj = this->asMethodObject();
+            file->write( &obj, sizeof( obj ) );
+         }
+         else {
+            // TODO
+            serialize_object( file, this->asMethodObject(), bLive );
+         }
+         serialize_function( file, this->asMethodFunction() );
       }
       break;
 
       case FLC_ITEM_OBJECT:
-         serialize_object( file, this->asObject(), vm );
+         serialize_object( file, this->asObject(), bLive );
       break;
 
       case FLC_ITEM_REFERENCE:
-         asReference()->origin().serialize( file, vm );
+         asReference()->origin().serialize( file, bLive );
+      break;
+
+      case FLC_ITEM_CLASS:
+         serialize_class( file, this->asClass() );
+      break;
+
+       case FLC_ITEM_CLSMETHOD:
+         serialize_class( file, this->asMethodClass() );
+         serialize_function( file, this->asMethodFunction() );
       break;
 
       default:
       {
-         char type = FLC_ITEM_NIL;
+         byte type = FLC_ITEM_NIL;
          file->write((byte *) &type, 1 );
       }
    }
@@ -340,10 +429,42 @@ Item::e_sercode Item::deserialize_function( Stream *file, VMachine *vm )
    return sc_ok;
 }
 
+Item::e_sercode Item::deserialize_class( Stream *file, VMachine *vm )
+{
+   if ( vm == 0 )
+      return sc_missvm;
+
+   String modName, className;
+   if ( ! modName.deserialize( file ) || ! className.deserialize( file ) )
+      return file->bad() ? sc_ferror : sc_invformat;
+
+   // find the module in the vm
+   LiveModule *origMod = vm->findModule( modName );
+   if( origMod == 0 )
+   {
+      return sc_misssym;
+   }
+
+   // find the class item in the module
+   Item *clitem = origMod->findModuleItem( className );
+   if ( clitem == 0 )
+      return sc_misssym;
+
+   if ( clitem->isReference() )
+   {
+      if( ! clitem->dereference()->isClass() )
+         return sc_misssym;
+   }
+   else if ( ! clitem->isClass() )
+      return sc_misssym;
+
+   *this = *clitem;
+   return sc_ok;
+}
 
 Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
 {
-   char type;
+   byte type;
    file->read((byte *) &type, 1 );
 
    if( ! file->good() )
@@ -355,6 +476,18 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
          setNil();
       return sc_ok;
 
+      case FLC_ITEM_BOOL:
+      {
+         byte bval;
+         file->read( (byte *) &bval, sizeof( bval ) );
+         if ( file->good() ) {
+            setBoolean( bval != 0 );
+            return sc_ok;
+         }
+         return sc_ferror;
+      }
+      return sc_ok;
+
       case FLC_ITEM_INT:
       {
          int64 val;
@@ -371,13 +504,15 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
       {
          int32 val1;
          int32 val2;
+         int32 val3;
          byte isOpen;
 
          file->read( (byte *) &val1, sizeof( val1 ) );
          file->read( (byte *) &val2, sizeof( val2 ) );
+         file->read( (byte *) &val3, sizeof( val3 ) );
          file->read( (byte *) &isOpen, sizeof( isOpen ) );
          if ( file->good() ) {
-            setRange( val1, val2, isOpen != 0 );
+            setRange( val1, val2, val3, isOpen != 0 );
             return sc_ok;
          }
          return sc_ferror;
@@ -432,6 +567,39 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
       }
       break;
 
+      case FLC_ITEM_MEMBUF |0x80:
+      {
+         // get the function pointer in the stream
+         MemBuf *(*deserializer)( VMachine *, Stream * );
+         file->read( &deserializer, sizeof( deserializer ) );
+         if ( ! file->good() ) {
+            return sc_ferror;
+         }
+
+         MemBuf *mb = deserializer( vm, file );
+         if( mb == 0 )
+         {
+            return sc_invformat;
+         }
+
+         setMemBuf( mb );
+         return sc_ok;
+      }
+
+      case FLC_ITEM_MEMBUF:
+      {
+         MemBuf *mb = MemBuf::deserialize( vm, file );
+         if ( file->good() && mb != 0 ) {
+            setMemBuf( mb );
+            return sc_ok;
+         }
+
+         delete mb; // may be 0, but it's ok
+         return sc_ferror;
+      }
+      break;
+
+
       case FLC_ITEM_FBOM:
       {
          byte methodId;
@@ -525,17 +693,29 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
          return deserialize_function( file, vm );
       break;
 
+      case FLC_ITEM_METHOD | 0x80:
       case FLC_ITEM_METHOD:
       {
          if( vm == 0 )
             return sc_missvm;
 
-         Item obj, func;
-         e_sercode sc = obj.deserialize( file, vm );
-         if ( sc != sc_ok )
-            return sc;
-         if ( ! obj.isObject() )
-            return sc_invformat;
+         Item obj;
+         Item func;
+         e_sercode sc;
+         if( type == FLC_ITEM_METHOD )
+         {
+            // TODO
+            sc = obj.deserialize( file, vm );
+            if ( sc != sc_ok )
+               return sc;
+            if ( ! obj.isObject() )
+               return sc_invformat;
+         }
+         else {
+            CoreObject *memObj;
+            file->read( &memObj, sizeof( memObj ) );
+            obj = memObj;
+         }
 
          sc = func.deserialize( file, vm );
          if ( sc != sc_ok )
@@ -547,8 +727,11 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
          return sc_ok;
       }
 
+      case FLC_ITEM_OBJECT | 0x80:
       case FLC_ITEM_OBJECT:
       {
+         bool bLive = type != FLC_ITEM_OBJECT;
+
          if( vm == 0 )
             return sc_missvm;
 
@@ -607,6 +790,15 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
          }
 
          if ( sc == sc_ok ) {
+            // if the object was serialized live, get the user data.
+            if( bLive )
+            {
+               UserData *data;
+               if ( file->read( (byte *) &data, sizeof( data ) ) != sizeof( data ) )
+                  return sc_ferror;
+               object->setUserData( data );
+            }
+
             setObject( object );
             return sc_ok;
          }
@@ -615,6 +807,19 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
       }
       break;
 
+      case FLC_ITEM_CLASS:
+         return deserialize_class( file, vm );
+
+
+       case FLC_ITEM_CLSMETHOD:
+       {
+         e_sercode sc = deserialize_class( file, vm );
+         if ( sc != sc_ok )
+            return sc;
+         return deserialize_function( file, vm );
+      }
+      break;
+
       default:
          return sc_invformat;
    }
