@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language
    FILE: src_lexer.cpp
-   $Id: src_lexer.cpp,v 1.21 2007/08/19 07:25:53 jonnymind Exp $
 
    Short description
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: sab ago 26 2006
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -53,11 +47,9 @@ SrcLexer::SrcLexer( Compiler *comp, Stream *in ):
    m_compiler( comp ),
    m_contexts(0),
    m_squareContexts(0),
-   m_firstEq( true ),
    m_character( 0 ),
    m_state( e_line ),
    m_done( false ),
-   m_firstSym( true ),
    m_addEol( false ),
    m_lineFilled( false ),
    m_mode( t_mNormal ),
@@ -140,13 +132,20 @@ int SrcLexer::lex_outscape()
                // and break from the loop so to return the string to print.
                break;
             }
-            else if ( chr == ' ' || chr == '\t' || chr == '\r' || chr == '\n' )
+            else if ( chr == ' ' || chr == '\t' || chr == '\r' )
             {
                // we enter now the normal mode; we start to consider a standard program.
                m_mode = t_mNormal;
                // and break from the loop so to return the string to print.
                break;
             }
+            else if ( chr == '\n' )
+            {
+               // as above, but now count theline
+               m_mode = t_mNormal;
+               // and break from the loop so to return the string to print.
+               break;
+            }
             else if ( chr == 'f' )
             {
                state = e_escF;
@@ -190,13 +189,24 @@ int SrcLexer::lex_outscape()
          break;
 
          case e_escL:
-            if ( chr == ' ' || chr == '\t' || chr == '\r' || chr == '\n' )
+            if ( chr == ' ' || chr == '\t' || chr == '\r' )
             {
                // we enter now the normal mode; we start to consider a standard program.
                m_mode = t_mNormal;
                // and break from the loop so to return the string to print.
                break;
             }
+            else if( chr == '\n' )
+            {
+               // as above, but now count theline
+               m_previousLine = m_line;
+               m_line++;
+               m_character = 0;
+               m_bIsDirectiveLine = false;
+               m_mode = t_mNormal;
+               // and break from the loop so to return the string to print.
+               break;
+            }
             else {
                state = e_normal;
                m_string.append( '<' );
@@ -214,6 +224,7 @@ int SrcLexer::lex_outscape()
    {
       VALUE->stringp = m_compiler->addString( m_string );
       m_string.size( 0 );
+      m_string.exported( false );
       return OUTER_STRING;
    }
 
@@ -246,11 +257,13 @@ int SrcLexer::lex_normal()
    // generate an extra eol?
    if ( m_addEol )
    {
-      m_firstEq = true;
       m_addEol = false;
       m_lineFilled = false;
-      m_bIsDirectiveLine = false;
-      return EOL;
+      if ( m_contexts == 0 && m_squareContexts == 0 )
+      {
+         m_bIsDirectiveLine = false;
+         return EOL;
+      }
    }
 
    if ( m_done )
@@ -324,6 +337,13 @@ int SrcLexer::lex_normal()
             if( ! isWhiteSpace( chr ) )
             {
                m_previousLine = m_line;
+               if ( m_bIsDirectiveLine && chr == '.' )
+               {
+                  // just ignore it
+                  m_string.append( chr );
+                  break;
+               }
+
                // whitespaces and '\n' can't follow a valid symbol,
                // as since they are token limiters, and they would be read
                // ahead after token begin, valid symbols and token has already
@@ -343,6 +363,22 @@ int SrcLexer::lex_normal()
             if ( isTokenLimit( chr ) )
             {
                // end of symbol
+               // special x" notation?
+               if( m_string.size() == 1 && (chr == '"' || chr == '\''))
+               {
+                  // recognize only the i" for now
+                  if ( m_string[0] == 'i' )
+                  {
+                     // starting a string context
+                     m_string.size(0); // remove first char.
+                     m_state = chr == '"' ? e_string : e_litString;;
+                     m_string.exported( true );
+                     m_chrEndString = chr; //... up to the matching "
+                     break;
+                  }
+                  // else, just let it through.
+               }
+
                // unless we have a dot in a load directive
                if ( m_bIsDirectiveLine && chr == '.' )
                {
@@ -359,9 +395,6 @@ int SrcLexer::lex_normal()
                // it may be a named token
                int token = checkLimitedTokens();
 
-               // we have a first symbol, that is, can't be a directive anymore
-               m_firstSym = false;
-
                if ( token != 0 )
                {
                   return token;
@@ -396,31 +429,7 @@ int SrcLexer::lex_normal()
                // great, we have a token
                m_lineFilled = true;
                m_state = e_line;
-
-               // a bit of galore: discard extra "\n" or "\r\n" in case of outer escape
-               if ( m_mode == t_mOutscape )
-               {
-                  if ( chr != '\n' ) {
-
-                     // ok, not a newline; but is it an INET newline?
-                     if ( chr == '\r')
-                     {
-                        uint32 ch1 = 0;
-                        m_in->get( ch1 );
-                        if ( ch1 != '\n' )
-                        {
-                           m_in->unget( ch1 );
-                           m_in->unget( chr );
-                        }
-                        // else silently discard
-                     }
-                     else
-                        m_in->unget( chr );
-                  }
-
-               }
-               else
-                  m_in->unget( chr );
+               m_in->unget( chr );
 
                return token;
             }
@@ -455,8 +464,6 @@ int SrcLexer::lex_normal()
                // a real EOL has been provided here.
                if ( m_state == e_line && m_contexts == 0  && m_squareContexts == 0 )
                {
-                  m_firstSym = true;
-                  m_firstEq = true;
                   m_bIsDirectiveLine = false;
                   if ( m_lineFilled )
                   {
@@ -668,6 +675,7 @@ int SrcLexer::lex_normal()
             {
                m_state = e_line;
                VALUE->stringp = m_compiler->addString( m_string );
+               m_string.exported( false );
                return STRING;
             }
             else
@@ -774,6 +782,7 @@ int SrcLexer::lex_normal()
                // end of input?
                if ( m_done ) {
                   VALUE->stringp = m_compiler->addString( m_string );
+                  m_string.exported( false );
                   m_state = e_line;
                   return STRING;
                }
@@ -808,6 +817,7 @@ int SrcLexer::lex_normal()
                   // end of input.
                   m_done = true;
                   VALUE->stringp = m_compiler->addString( m_string );
+                  m_string.exported( false );
                   return STRING;
                }
 
@@ -817,6 +827,7 @@ int SrcLexer::lex_normal()
                }
                else if ( chr1 == '/' ) {
                   VALUE->stringp = m_compiler->addString( m_string );
+                  m_string.exported( false );
                   m_state = e_eolComment;
                   return STRING;
                }
@@ -825,15 +836,16 @@ int SrcLexer::lex_normal()
                   m_in->unget( chr1 );
                   m_in->unget( chr );
                   VALUE->stringp = m_compiler->addString( m_string );
+                  m_string.exported( false );
                   m_state = e_line;
                   return STRING;
                }
             }
             else if ( ! isWhiteSpace( chr ) )
             {
-
                m_in->unget( chr );
                VALUE->stringp = m_compiler->addString( m_string );
+               m_string.exported( false );
                m_state = e_line;
                return STRING;
             }
@@ -903,14 +915,11 @@ int SrcLexer::state_line( uint32 chr )
       m_character = 0;
 
       // a real EOL has been provided here.
-      m_firstEq = true;
-      m_firstSym = true;
       m_bIsDirectiveLine = false;
-      if ( m_lineFilled )
+      if ( m_lineFilled && m_contexts == 0 && m_squareContexts == 0 )
       {
          m_lineFilled = false;
-         if ( m_contexts == 0 && m_squareContexts == 0 )
-            return EOL;
+         return EOL;
       }
    }
    else if ( chr == '\\' )
@@ -1007,7 +1016,6 @@ int SrcLexer::checkUnlimitedTokens( uint32 nextChar )
          uint32 chr = m_string.getCharAt( 0 );
          if ( chr == ';'  )
          {
-            m_firstEq = true;
             m_bIsDirectiveLine = false;
             // but not first sym
             if ( m_lineFilled )
@@ -1026,14 +1034,18 @@ int SrcLexer::checkUnlimitedTokens( uint32 nextChar )
             return SLASH;
          else if ( chr == '%' && nextChar != '=' )
             return PERCENT;
-         else if ( chr == '&' && nextChar != '=' )
+         else if ( chr == '&' && nextChar != '=' && nextChar != '&' )
             return AMPER;
-         else if ( chr == '|' && nextChar != '=' )
+         else if ( chr == '~' && nextChar != '=' )
+               return TILDE;
+         /*
+            This may come useful in future.
+         else if ( chr == '|' && nextChar != '=' && nextChar != '|' )
             return VBAR;
-         else if ( chr == '^' && nextChar != '=' )
+         else if ( chr == '^' && nextChar != '=' && nextChar != '^' )
             return CAP;
          else if ( chr == '!' && nextChar != '=' )
-               return BANG;
+               return BANG;*/
 
          else if ( chr == '$' )
          {
@@ -1041,9 +1053,6 @@ int SrcLexer::checkUnlimitedTokens( uint32 nextChar )
          }
          else if ( chr == ':' )
          {
-            // expressions after : are reassigned, but only out of contexts
-            if( m_contexts == 0 && m_squareContexts == 0 )
-               m_firstEq = true;
                // but they don't reset first sym
             return COLON;
          }
@@ -1064,12 +1073,7 @@ int SrcLexer::checkUnlimitedTokens( uint32 nextChar )
             return LT;
          else if ( chr == '=' && nextChar != '=' && nextChar != '>' )
          {
-            if ( m_firstEq ) {
-               m_firstEq = false;
-               return OP_ASSIGN;
-            }
-            else
-               return OP_EQ;
+            return OP_EQ;
          }
          else if ( chr == '(' || chr == 0xff08 )
          {
@@ -1146,6 +1150,12 @@ int SrcLexer::checkUnlimitedTokens( uint32 nextChar )
             return ASSIGN_BOR;
          else if ( m_string == "^=" )
             return ASSIGN_BXOR;
+         else if ( m_string == "&&" )
+            return AMPER_AMPER;
+         else if ( m_string == "^^" )
+            return CAP_CAP;
+         else if ( m_string == "||" )
+            return VBAR_VBAR;
          else if ( m_string == ">>" && nextChar != '=' )
             return SHR;
          else if ( m_string == "<<" && nextChar != '=' )
@@ -1165,6 +1175,8 @@ int SrcLexer::checkUnlimitedTokens( uint32 nextChar )
          else if ( m_string == ".[" )
          {
             m_squareContexts++;
+            if ( m_squareContexts == 1 )
+               m_ctxOpenLine = m_line;
             return LISTPAR;
          }
          else if ( m_string == ".\"" )
@@ -1174,7 +1186,6 @@ int SrcLexer::checkUnlimitedTokens( uint32 nextChar )
             m_chrEndString = '"'; //... up to the matching "
             return 0;
          }
-
          else if ( parsingFtd() && m_string == "?>" )
          {
             m_mode = t_mOutscape;
@@ -1220,8 +1231,6 @@ int SrcLexer::checkLimitedTokens()
             return NIL;
          else if ( m_string == "for" )
             return FOR;
-         else if ( m_string == "let" )
-            return LET;
          else if ( m_string == "has" )
             return HAS;
          else if ( m_string == "and" )
@@ -1236,7 +1245,7 @@ int SrcLexer::checkLimitedTokens()
       break;
 
       case 4:
-         if ( m_string == "load" && m_firstSym )  // directive
+         if ( m_string == "load" )  // directive
          {
             m_bIsDirectiveLine = true;
             return LOAD;
@@ -1255,12 +1264,14 @@ int SrcLexer::checkLimitedTokens()
             return SELF;
          if ( m_string == "pass" )
             return PASS;
-         if ( m_string == "step" )
-            return FOR_STEP;
          if ( m_string == "case" )
             return CASE;
          if ( m_string == "loop" )
             return LOOP;
+         if ( m_string == "true" )
+            return TRUE_TOKEN;
+         if ( m_string == "enum" )
+            return ENUM;
       break;
 
       case 5:
@@ -1280,6 +1291,8 @@ int SrcLexer::checkLimitedTokens()
             return CONST_KW;
          if ( m_string == "while" )
             return WHILE;
+         if ( m_string == "false" )
+            return FALSE_TOKEN;
       break;
 
       case 6:
@@ -1299,15 +1312,18 @@ int SrcLexer::checkLimitedTokens()
             return OBJECT;
          if ( m_string == "return" )
             return RETURN;
-         if ( m_string == "export" && m_firstSym ) // directive
+         if ( m_string == "export" ) // directive
          {
             m_bIsDirectiveLine = true;
             return EXPORT;
          }
+         if ( m_string == "import" ) // directive
+         {
+            m_bIsDirectiveLine = true;
+            return IMPORT;
+         }
          if ( m_string == "static" )
             return STATIC;
-         if ( m_string == "forall" )
-            return FORALL;
       break;
 
       case 7:
@@ -1335,13 +1351,17 @@ int SrcLexer::checkLimitedTokens()
          if ( m_string == "directive" )
          {
             // No assigments in directive.
-            m_firstEq = false;
             m_bIsDirectiveLine = true;
             return DIRECTIVE;
          }
+         if ( m_string == "innerfunc" )
+            return INNERFUNC;
+         if ( m_string == "formiddle" )
+            return FORMIDDLE;
       break;
 
       case 10:
+
          if ( m_string == "attributes" )
             return ATTRIBUTES;
       break;
