@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: cstring.cpp
-   $Id: string.cpp,v 1.25 2007/08/16 13:46:40 jonnymind Exp $
 
    Implementation of Core Strings.
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: mer nov 24 2004
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -243,7 +237,8 @@ uint32 Byte::find( const String *str, const String *element, uint32 start, uint3
    uint32 elemLen = element->length();
 
    while( pos + elemLen <= end ) {
-      if ( str->getCharAt( pos ) == keyStart ) {
+      if ( str->getCharAt( pos ) == keyStart )
+      {
          uint32 len = 1;
          while( pos + len < end && len < elemLen && element->getCharAt(len) == str->getCharAt( pos + len ) )
             len++;
@@ -1253,7 +1248,8 @@ void Buffer::destroy( String *str ) const
 String::String( uint32 size ):
    m_class( &csh::handler_buffer ),
    m_garbageable( false ),
-   m_id(String::no_id)
+   m_id(String::no_id),
+   m_bExported( false )
 {
    m_storage = (byte *) memAlloc( size );
    m_allocated = size;
@@ -1265,7 +1261,8 @@ String::String( const char *data ):
    m_garbageable( false ),
    m_allocated( 0 ),
    m_storage( (byte*) const_cast< char *>(data) ),
-   m_id(String::no_id)
+   m_id(String::no_id),
+   m_bExported( false )
 {
    m_size = strlen( data );
 }
@@ -1273,7 +1270,8 @@ String::String( const char *data ):
 String::String( const char *data, int32 len ):
    m_class( &csh::handler_buffer ),
    m_garbageable( false ),
-   m_id(String::no_id )
+   m_id(String::no_id ),
+   m_bExported( false )
 {
    m_size = len >= 0 ? len : strlen( data );
    m_allocated = (( m_size / FALCON_STRING_ALLOCATION_BLOCK ) + 1 ) * FALCON_STRING_ALLOCATION_BLOCK;
@@ -1286,7 +1284,8 @@ String::String( const wchar_t *data ):
    m_garbageable( false ),
    m_allocated( 0 ),
    m_storage( (byte*) const_cast< wchar_t *>(data) ),
-   m_id( String::no_id )
+   m_id( String::no_id ),
+   m_bExported( false )
 {
    if ( sizeof( wchar_t ) == 2 )
       m_class = &csh::handler_static16;
@@ -1304,7 +1303,8 @@ String::String( const wchar_t *data, int32 len ):
    m_allocated( 0 ),
    m_storage( (byte *) const_cast< wchar_t *>( data ) ),
    m_garbageable( false ),
-   m_id( String::no_id )
+   m_id( String::no_id ),
+   m_bExported( false )
 {
    if ( sizeof( wchar_t ) == 2 )
       m_class = &csh::handler_buffer16;
@@ -1334,7 +1334,8 @@ String::String( const String &other, uint32 begin, uint32 end ):
    m_size( 0 ),
    m_storage( 0 ),
    m_garbageable( false ),
-   m_id( String::no_id )
+   m_id( String::no_id ),
+   m_bExported( false )
 {
    // by default, copy manipulator
    m_class = other.m_class;
@@ -1907,7 +1908,8 @@ void String::unescape()
 
 void String::serialize( Stream *out ) const
 {
-   uint32 size = endianInt32(m_size);
+   uint32 size = endianInt32( m_bExported ? m_size | 0x80000000 : m_size );
+
    out->write( (byte *) &size, sizeof(size) );
    if ( m_size != 0 && out->good() )
    {
@@ -1931,7 +1933,7 @@ void String::serialize( Stream *out ) const
                return;
          }
       }
-      else if ( chars == 2 )
+      else if ( chars == 4 )
       {
          for( int i = 0; i < m_size/4; i ++ )
          {
@@ -1953,6 +1955,8 @@ bool String::deserialize( Stream *in, bool bStatic )
 
    in->read( (byte *) &size, sizeof( size ) );
    m_size = endianInt32(size);
+   m_bExported = (m_size & 0x80000000) == 0x80000000;
+   m_size = m_size & 0x7FFFFFFF;
 
    // if the size of the deserialized string is 0, we have an empty string.
    if ( m_size == 0 )
@@ -2027,7 +2031,7 @@ bool String::deserialize( Stream *in, bool bStatic )
                return;
          }
       }
-      else if ( chars == 2 )
+      else if ( chars == 4 )
       {
          for( int i = 0; i < m_size/4; i ++ )
          {
@@ -2289,6 +2293,17 @@ void String::writeNumberOctal( uint64 number )
    append( buffer + pos );
 }
 
+void String::writeNumber( int64 number, const String &format )
+{
+   char buffer[64];
+
+   char bufFormat[32];
+   if ( format.toCString( bufFormat, 32 ) == - 1 )
+      return;
+
+   sprintf( buffer, bufFormat, number );
+   append( buffer );
+}
 
 void String::writeNumber( double number, const String &format )
 {
