@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: syntree.cpp
-   $Id: syntree.cpp,v 1.14 2007/08/01 13:32:26 jonnymind Exp $
 
    Short description
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin:
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #include <falcon/setup.h>
@@ -109,12 +103,18 @@ RangeDecl::RangeDecl( const RangeDecl &other )
    else
       m_rend = new Value( *other.m_rend );
 
+   if ( other.m_step == 0 )
+      m_step = 0;
+   else
+      m_step = new Value( *other.m_step );
+
 }
 
 RangeDecl::~RangeDecl()
 {
    delete m_rstart;
    delete m_rend;
+   delete m_step;
 }
 
 //================================================
@@ -618,13 +618,13 @@ Statement *StmtPass::clone() const
 StmtGive::StmtGive( const StmtGive &other ):
    Statement( other )
 {
-   m_object = other.m_object == 0 ? 0 : new Value( *other.m_object );
+   m_objects = other.m_objects == 0 ? 0 : new ArrayDecl( *other.m_objects );
    m_attribs = other.m_attribs == 0 ? 0 : new ArrayDecl( *other.m_attribs );
 }
 
 StmtGive::~StmtGive()
 {
-   delete m_object;
+   delete m_objects;
    delete m_attribs;
 }
 
@@ -633,80 +633,6 @@ Statement *StmtGive::clone() const
    return new StmtGive( *this );
 }
 
-// StmtBaseAssign statement
-//
-
-StmtBaseAssign::StmtBaseAssign( const StmtBaseAssign &other ):
-   StmtExpression( other )
-{
-   m_dest = other.m_dest == 0 ? 0 : new Value( *other.m_dest );
-}
-
-StmtBaseAssign::~StmtBaseAssign()
-{
-   delete m_dest;
-}
-
-Statement *StmtAssignment::clone() const
-{
-   return new StmtAssignment( *this );
-}
-
-Statement *StmtAutoAdd::clone() const
-{
-   return new StmtAutoAdd( *this );
-}
-
-Statement *StmtAutoSub::clone() const
-{
-   return new StmtAutoSub( *this );
-}
-
-Statement *StmtAutoMul::clone() const
-{
-   return new StmtAutoMul( *this );
-}
-
-Statement *StmtAutoDiv::clone() const
-{
-   return new StmtAutoDiv( *this );
-}
-
-Statement *StmtAutoMod::clone() const
-{
-   return new StmtAutoMod( *this );
-}
-
-Statement *StmtAutoPow::clone() const
-{
-   return new StmtAutoPow( *this );
-}
-
-Statement *StmtAutoBAND::clone() const
-{
-   return new StmtAutoBAND( *this );
-}
-
-Statement *StmtAutoBOR::clone() const
-{
-   return new StmtAutoBOR( *this );
-}
-
-Statement *StmtAutoBXOR::clone() const
-{
-   return new StmtAutoBXOR( *this );
-}
-
-Statement *StmtAutoSHL::clone() const
-{
-   return new StmtAutoSHL( *this );
-}
-
-Statement *StmtAutoSHR::clone() const
-{
-   return new StmtAutoSHR( *this );
-}
-
 // Loop ctrl
 //
 
@@ -786,7 +712,7 @@ StmtForin::StmtForin( const StmtForin &other ):
    StmtBlock( other ),
    m_first( other.m_first ),
    m_last( other.m_last ),
-   m_all( other.m_all )
+   m_middle( other.m_middle )
 {
    m_source = other.m_source == 0 ? 0 : new Value( * other.m_source );
    m_dest = other.m_dest == 0 ? 0 : new Value( * other.m_dest );
@@ -804,36 +730,6 @@ Statement *StmtForin::clone() const
 }
 
 
-// StmtFor statement
-//
-
-StmtFor::StmtFor( const StmtFor &other ):
-   StmtBlock( other )
-{
-  Value *m_counter;
-   Value *m_from;
-   Value *m_to;
-   Value *m_step;
-
-   m_counter = other.m_counter == 0 ? 0 : new Value( * other.m_counter );
-   m_from = other.m_from == 0 ? 0 : new Value( * other.m_from );
-   m_to = other.m_to == 0 ? 0 : new Value( * other.m_to );
-   m_step = other.m_step == 0 ? 0 : new Value( * other.m_step );
-}
-
-StmtFor::~StmtFor()
-{
-   delete m_counter;
-   delete m_from;
-   delete m_to;
-   delete m_step;
-}
-
-Statement *StmtFor::clone() const
-{
-   return new StmtFor( *this );
-}
-
 // StmtCaseBlock statement
 //
 
