@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: TimeStampapi.cpp
-   $Id: timestamp.cpp,v 1.7 2007/08/11 12:15:59 jonnymind Exp $
 
    Short description
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: lun mar 6 2006
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -26,12 +20,17 @@
 #include <falcon/timestamp.h>
 #include <falcon/memory.h>
 #include <falcon/time_sys.h>
-#include <falcon/string.h>
+#include <falcon/autocstring.h>
 #include <falcon/item.h>
 
 #include <stdio.h>
 #include <time.h>
 
+static const char *RFC_2822_days[] = { "Mon","Tue", "Wed","Thu","Fri","Sat","Sun" };
+
+static const char *RFC_2822_months[] = {
+   "Jan","Feb","Mar","Apr","May","Jun","Jul","Aug", "Sep","Oct","Nov","Dec" };
+
 namespace Falcon {
 
 inline bool i_isLeapYear( int year )
@@ -58,12 +57,303 @@ void TimeStamp::copy( const TimeStamp &ts )
    m_msec = ts.m_msec;
 }
 
+const char *TimeStamp::getRFC2822_ZoneName( TimeZone tz, bool bSemantic, bool bSaving )
+{
+   if ( tz == tz_local )
+      tz = Sys::Time::getLocalTimeZone();
+
+   switch( tz )
+   {
+      case tz_local: return "+????";
+      case tz_NONE: case tz_UTC: if ( bSemantic ) return "GMT"; return "+0000";
+      case tz_UTC_E_1: return "+0100";
+      case tz_UTC_E_2: return "+0200";
+      case tz_UTC_E_3: return "+0300";
+      case tz_UTC_E_4: return "+0400";
+      case tz_UTC_E_5: return "+0500";
+      case tz_UTC_E_6: return "+0600";
+      case tz_UTC_E_7: return "+0700";
+      case tz_UTC_E_8: return "+0800";
+      case tz_UTC_E_9: return "+0900";
+      case tz_UTC_E_10: return "+1000";
+      case tz_UTC_E_11: return "+1100";
+      case tz_UTC_E_12: return "+1200";
+
+      case tz_UTC_W_1: return "-0100";
+      case tz_UTC_W_2: return "-0200";
+      case tz_UTC_W_3: return "-0300";
+      case tz_UTC_W_4: if ( bSemantic ) return "EDT"; return "-0400";
+      case tz_UTC_W_5: if ( bSemantic ) return bSaving ? "EST":"CDT"; return "-0500";
+      case tz_UTC_W_6: if ( bSemantic ) return bSaving ? "CST":"MDT"; return "-0600";
+      case tz_UTC_W_7: if ( bSemantic ) return bSaving ? "MST":"PDT"; return "-0700";
+      case tz_UTC_W_8: if ( bSemantic ) return "PST"; return "-0800";
+      case tz_UTC_W_9: return "-0900";
+      case tz_UTC_W_10: return "-1000";
+      case tz_UTC_W_11: return "-1100";
+      case tz_UTC_W_12: return "-1200";
+
+      case tz_NFT: return "+1130";
+      case tz_ACDT: return "+1030";
+      case tz_ACST: return "+0930";
+      case tz_HAT: return "-0230";
+      case tz_NST: return "-0330";
+   }
+   return "+????";
+}
+
+
+TimeZone TimeStamp::getRFC2822_Zone( const char *csZoneName )
+{
+   if( strncmp( "UT", csZoneName, 2 ) == 0 ||
+      strncmp( "GMT", csZoneName, 3 ) == 0 ||
+      strncmp( "+0000", csZoneName, 5 ) == 0 )
+   {
+      return tz_UTC;
+   }
+
+   if ( csZoneName[0] == '+' )
+   {
+      int zone = atoi( csZoneName + 1 );
+      switch( zone )
+      {
+         case 100: return tz_UTC_E_1;
+         case 200: return tz_UTC_E_2;
+         case 300: return tz_UTC_E_3;
+         case 400: return tz_UTC_E_4;
+         case 500: return tz_UTC_E_5;
+         case 600: return tz_UTC_E_6;
+         case 700: return tz_UTC_E_7;
+         case 800: return tz_UTC_E_8;
+         case 900: return tz_UTC_E_9;
+         case 1000: return tz_UTC_E_10;
+         case 1100: return tz_UTC_E_11;
+         case 1200: return tz_UTC_E_12;
+
+         case 1130: return tz_NFT;
+         case 1030: return tz_ACDT;
+         case 930: return tz_ACST;
+      }
+      return tz_NONE;
+   }
+   else if ( csZoneName[0] == '-' )
+   {
+      int zone = atoi( csZoneName + 1 );
+      switch( zone )
+      {
+         case 100: return tz_UTC_W_1;
+         case 200: return tz_UTC_W_2;
+         case 300: return tz_UTC_W_3;
+         case 400: return tz_UTC_W_4;
+         case 500: return tz_UTC_W_5;
+         case 600: return tz_UTC_W_6;
+         case 700: return tz_UTC_W_7;
+         case 800: return tz_UTC_W_8;
+         case 900: return tz_UTC_W_9;
+         case 1000: return tz_UTC_W_10;
+         case 1100: return tz_UTC_W_11;
+         case 1200: return tz_UTC_W_12;
+         case 230: return tz_HAT;
+         case 330: return tz_NST;
+      }
+      return tz_NONE;
+   }
+
+   if( strncmp( "EDT", csZoneName, 3 ) == 0 )
+   {
+      return tz_UTC_W_4;
+   }
+
+   if( strncmp( "EST", csZoneName, 3 ) == 0 ||
+      strncmp( "CDT", csZoneName, 3 ) == 0 )
+   {
+      return tz_UTC_W_5;
+   }
+
+   if( strncmp( "CST", csZoneName, 3 ) == 0 ||
+      strncmp( "MDT", csZoneName, 3 ) == 0 )
+   {
+      return tz_UTC_W_6;
+   }
+
+   if( strncmp( "MST", csZoneName, 3 ) == 0 ||
+      strncmp( "PDT", csZoneName, 3 ) == 0 )
+   {
+      return tz_UTC_W_7;
+   }
+
+   if( strncmp( "PST", csZoneName, 3 ) == 0 )
+   {
+      return tz_UTC_W_8;
+   }
+
+   // failure
+   return tz_NONE;
+}
+
+const char *TimeStamp::getRFC2822_WeekDayName( int16 wd )
+{
+   if ( wd >=0 && wd < 7 )
+   {
+      return RFC_2822_days[wd];
+   }
+   return "???";
+}
+
+const char *TimeStamp::getRFC2822_MonthName( int16 month )
+{
+   if ( month >= 1 && month <= 12 )
+   {
+      return RFC_2822_months[ month - 1 ];
+   }
+   return "???";
+}
+
+
+int16 TimeStamp::getRFC2822_WeekDay( const char *name )
+{
+   for ( int16 i = 0; i < sizeof( RFC_2822_days ) / sizeof( char *); i ++ )
+   {
+      if ( strncmp( RFC_2822_days[i], name, 3) == 0 )
+         return i;
+   }
+   return -1;
+}
+
+int16 TimeStamp::getRFC2822_Month( const char *name )
+{
+   for ( int16 i = 0; i < sizeof( RFC_2822_months ) / sizeof( char *); i ++ )
+   {
+      if ( strncmp( RFC_2822_months[i], name, 3) == 0 )
+         return i+1;
+   }
+   return -1;
+}
+
 TimeStamp &TimeStamp::operator = ( const TimeStamp &ts )
 {
    copy(ts);
    return *this;
 }
 
+
+bool TimeStamp::toRFC2822( String &target, bool bSemantic, bool bDst ) const
+{
+   if ( ! isValid() )
+   {
+      target = "?";
+      return false;
+   }
+
+   target.append( getRFC2822_WeekDayName( dayOfWeek() ) );
+   target.append( ',' );
+   target.append( ' ' );
+
+   target.writeNumber( (int64) m_day, "%02d" );
+
+   target.append( ' ' );
+   target.append( getRFC2822_MonthName( m_month ) );
+   target.append( ' ' );
+   if ( m_year < 0 )
+      target.append( "0000" );
+   else {
+      target.writeNumber( (int64) m_year, "%04d" );
+   }
+
+   target.append( ' ' );
+   target.writeNumber( (int64) m_hour, "%02d" );
+   target.append( ':' );
+   target.writeNumber( (int64) m_minute, "%02d" );
+   target.append( ':' );
+   target.writeNumber( (int64) m_second, "%02d" );
+
+   target.append( ' ' );
+   TimeZone tz = m_timezone;
+
+   if  ( tz == tz_local )
+   {
+      tz = Sys::Time::getLocalTimeZone();
+   }
+
+   target.append( getRFC2822_ZoneName( tz, bSemantic, bDst ) );
+
+   return true;
+}
+
+
+bool TimeStamp::fromRFC2822( TimeStamp &target, const String &source )
+{
+   AutoCString cstr( source );
+   return fromRFC2822( target, cstr.c_str() );
+}
+
+bool TimeStamp::fromRFC2822( TimeStamp &target, const char *source )
+{
+   const char *pos = source;
+
+   // Find the comma
+   while ( *pos != 0 && *pos != ',' ) pos++;
+   if ( *pos == 0 || (pos-source)!= 3 )
+      return false;
+
+   // is this a valid day?
+   if( getRFC2822_WeekDay( source ) < 0 )
+      return false;
+
+   pos++;
+   if ( *pos == 0 )
+      return false;
+   pos++;
+   const char *mon = pos;
+   while( *mon != 0 && *mon != ' ' ) mon++;
+   if ( *mon == 0 || (mon - pos) != 2)
+      return false;
+   target.m_day = atoi( pos );
+
+   mon++;
+   pos = mon;
+   while( *mon != 0 && *mon != ' ' ) mon++;
+   if ( *mon == 0 || (mon - pos) != 3)
+      return false;
+   target.m_month = getRFC2822_Month( pos );
+
+   mon++;
+   pos = mon;
+   while( *mon != 0 && *mon != ' ' ) mon++;
+   if ( *mon == 0 || (mon - pos) != 4)
+      return false;
+   target.m_year = atoi( pos );
+
+   mon++;
+   pos = mon;
+   while( *mon != 0 && *mon != ':' ) mon++;
+   if ( *mon == 0 || (mon - pos) != 2 )
+      return false;
+   target.m_hour = atoi( pos );
+
+   mon++;
+   pos = mon;
+   while( *mon != 0 && *mon != ':' ) mon++;
+   if ( *mon == 0 || (mon - pos) != 2 )
+      return false;
+   target.m_minute = atoi( pos );
+
+   mon++;
+   pos = mon;
+   while( *mon != 0 && *mon != ' ' ) mon++;
+   if ( *mon == 0 || (mon - pos) != 2 )
+      return false;
+   target.m_second = atoi( pos );
+
+   mon++;
+   target.m_timezone = getRFC2822_Zone( mon );
+   if( target.m_timezone == tz_NONE )
+      return false;
+
+   target.m_msec = 0;
+
+   return target.isValid();
+}
+
 bool TimeStamp::isValid() const
 {
    if ( m_msec < 0 || m_msec >= 1000 )
@@ -251,7 +541,7 @@ void TimeStamp::fromSeconds( numeric seconds )
 
 void TimeStamp::add( const TimeStamp &ts )
 {
-   m_day = ts.m_day + dayOfYear();
+   m_day += ts.m_day;
    m_hour += ts.m_hour;
    m_minute += ts.m_minute;
    m_second += ts.m_second;
@@ -442,61 +732,36 @@ void TimeStamp::rollOver( bool onlyDays )
    if ( onlyDays ) {
       return;
    }
-   adjust = m_day;
 
-   if ( adjust <= 0 ) {
-      m_year --;
-      while ( adjust < -366 ) {
-         adjust += 365;
-         if ( i_isLeapYear( m_year ) )
-            adjust++;
-         m_year--;
-      }
+   if ( m_day > 0 )
+   {
+      int16 mdays;
+      while( m_day > (mdays = getDaysOfMonth( m_month )) )
+      {
+         m_day -= mdays;
 
-      if ( adjust == -365 && ! i_isLeapYear( m_year ) ) {
-         adjust = 0;
-         m_year--;
-      }
-
-      m_month = 12;
-      int16 mdays = getDaysOfMonth( m_month );
-      adjust += mdays;
-      while( adjust <= 0 ) {
-         m_month --;
-         if ( m_month == 0 ) {
-            m_month = 12;
-            m_year--;
-         }
-         mdays = getDaysOfMonth( m_month );
-         adjust += mdays;
-      }
-   }
-   else {
-      while ( adjust > 366 ) {
-         adjust -= 365;
-         if ( i_isLeapYear( m_year ) )
-            adjust--;
-         m_year++;
-      }
-
-      if ( adjust == 365 && ! i_isLeapYear( m_year ) ) {
-         adjust = 1;
-         m_year ++;
-      }
-
-      m_month = 1;
-      int16 mdays = getDaysOfMonth( m_month );
-      while( adjust > mdays ) {
-         m_month ++;
-         adjust -= mdays;
-         mdays = getDaysOfMonth( m_month );
-         if ( m_month > 12 ) {
+         if( m_month == 12 )
+         {
             m_month = 1;
             m_year++;
          }
+         else
+            m_month++;
+      }
+   }
+   else {
+      while( m_day < 1 )
+      {
+         if ( m_month == 1 )
+         {
+            m_month = 12;
+            m_year--;
+         }
+         else
+            m_month --;
+         m_day += getDaysOfMonth( m_month );
       }
    }
-   m_day = adjust;
 }
 
 int32 TimeStamp::compare( const TimeStamp &ts ) const
@@ -547,7 +812,12 @@ int32 TimeStamp::compare( const TimeStamp &ts ) const
 void TimeStamp::getTZDisplacement( int16 &hours, int16 &minutes ) const
 {
    TimeZone tz = m_timezone;
+   getTZDisplacement( tz, hours, minutes );
+}
 
+
+void TimeStamp::getTZDisplacement( TimeZone tz, int16 &hours, int16 &minutes )
+{
    if  ( tz == tz_local )
    {
       tz = Sys::Time::getLocalTimeZone();
@@ -555,7 +825,7 @@ void TimeStamp::getTZDisplacement( int16 &hours, int16 &minutes ) const
 
    switch( tz )
    {
-      case tz_NONE: case tz_UTC: hours = 0; minutes = 0; break;
+      case tz_local: case tz_NONE: case tz_UTC: hours = 0; minutes = 0; break;
       case tz_UTC_E_1: hours = 1; minutes = 0; break;
       case tz_UTC_E_2: hours = 2; minutes = 0; break;
       case tz_UTC_E_3: hours = 3; minutes = 0; break;
@@ -623,14 +893,26 @@ bool TimeStamp::toString( String &target, const String &fmt ) const
    {
       target.bufferize( timeTgt );
 
-      uint32 pos = target.find( "%q" );
+      uint32 pos = target.find( "%i" );
+      if( pos !=  String::npos )
+      {
+         String rfc;
+         toRFC2822( rfc );
+         while( pos != String::npos )
+         {
+            target.change( pos, pos + 2, rfc );
+            pos = target.find( "%i", pos + 2 );
+         }
+      }
+
+      pos = target.find( "%q" );
       if( pos !=  String::npos )
       {
          String msecs;
          msecs.writeNumber( (int64) m_msec );
          while( pos != String::npos )
          {
-            target.change( pos, pos + 2, &msecs );
+            target.change( pos, pos + 2, msecs );
             pos = target.find( "%q", pos + 2 );
          }
       }
@@ -648,7 +930,7 @@ bool TimeStamp::toString( String &target, const String &fmt ) const
 
          while( pos != String::npos )
          {
-            target.change( pos, pos + 2, &msecs );
+            target.change( pos, pos + 2, msecs );
             pos = target.find( "%Q", pos + 2 );
          }
       }
@@ -662,15 +944,48 @@ bool TimeStamp::toString( String &target, const String &fmt ) const
 void TimeStamp::currentTime()
 {
    Sys::Time::currentTime( *this );
+   if  ( m_timezone == tz_local )
+   {
+      m_timezone = Sys::Time::getLocalTimeZone();
+   }
 }
 
+void TimeStamp::changeTimezone( TimeZone tz )
+{
+   if  ( m_timezone == tz_local )
+   {
+      m_timezone = Sys::Time::getLocalTimeZone();
 
-bool TimeStamp::isReflective()
+      if (tz == tz_local ) // no shift...
+         return;
+   }
+
+   if ( tz == tz_local )
+      tz = Sys::Time::getLocalTimeZone();
+
+   // no shift
+   if ( tz == m_timezone )
+      return;
+
+   // get the relative total shift.
+   int16 currentHour, currentMin, newHour, newMin;
+   getTZDisplacement( tz, newHour, newMin );
+   getTZDisplacement( m_timezone, currentHour, currentMin );
+
+   m_hour -= currentHour;
+   m_hour += newHour;
+   m_minute -= currentMin;
+   m_minute += newMin;
+   rollOver( false );
+   m_timezone = tz;
+}
+
+bool TimeStamp::isReflective() const
 {
    return true;
 }
 
-void TimeStamp::getProperty( const String &propName, Item &prop )
+void TimeStamp::getProperty( VMachine *, const String &propName, Item &prop )
 {
    if( propName == "year" )
    {
@@ -694,35 +1009,41 @@ void TimeStamp::getProperty( const String &propName, Item &prop )
    else if( propName == "msec" ) {
       prop = (int64) m_msec;
    }
+   else if( propName == "timezone" ) {
+      prop = (int64) m_timezone;
+   }
 }
 
-void TimeStamp::setProperty( const String &propName, Item &prop )
+void TimeStamp::setProperty( VMachine *, const String &propName, Item &prop )
 {
    if( propName == "year" )
    {
-      m_year = (uint16) prop.forceInteger();
+      m_year = (int16) prop.forceInteger();
    }
    else if( propName == "month" ) {
-      m_month = (uint16) prop.forceInteger();
+      m_month = (int16) prop.forceInteger();
    }
    else if( propName == "day" ) {
-      m_day = (uint16) prop.forceInteger();
+      m_day = (int16) prop.forceInteger();
    }
    else if( propName == "hour" ) {
-      m_hour = (uint16) prop.forceInteger();
+      m_hour = (int16) prop.forceInteger();
    }
    else if( propName == "minute" ) {
-      m_minute = (uint16) prop.forceInteger();
+      m_minute = (int16) prop.forceInteger();
    }
    else if( propName == "second" ) {
-      m_second = (uint16) prop.forceInteger();
+      m_second = (int16) prop.forceInteger();
    }
    else if( propName == "msec" ) {
-      m_msec = (uint16) prop.forceInteger();
+      m_msec = (int16) prop.forceInteger();
+   }
+   else if( propName == "timezone" ) {
+      m_timezone = (TimeZone)( prop.forceInteger()%32);
    }
 }
 
-UserData *TimeStamp::clone()
+UserData *TimeStamp::clone() const
 {
    return new TimeStamp( *this );
 }
