@@ -6,16 +6,11 @@
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: sab giu 23 2007
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -29,20 +24,25 @@ namespace Falcon {
 UserData::~UserData()
 {}
 
-bool UserData::isReflective()
+bool UserData::isReflective() const
 {
    return false;
 }
 
-void UserData::getProperty( const String &propName, Item &prop )
+bool UserData::shared() const
+{
+   return false;
+}
+
+void UserData::getProperty( VMachine *vm, const String &propName, Item &prop )
 {
 }
 
-void UserData::setProperty( const String &propName, Item &prop )
+void UserData::setProperty( VMachine *vm, const String &propName, Item &prop )
 {
 }
 
-UserData * UserData::clone()
+UserData * UserData::clone() const
 {
    return 0;
 }
