@@ -2,20 +2,15 @@
    FALCON - The Falcon Programming Language.
    FILE: vm.cpp
 
-   Implementation of virtual - non main loop
+   Implementation of virtual machine - non main loop
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: 2004-09-08
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #include <falcon/item.h>
@@ -38,6 +33,7 @@
 #include <falcon/format.h>
 #include <falcon/attribute.h>
 #include <falcon/bommap.h>
+#include <falcon/vm_sys.h>
 
 namespace Falcon {
 
@@ -201,15 +197,15 @@ void VMachine::internal_construct()
    m_opHandlers[ P_STVR] = opcodeHandler_STVR;
    m_opHandlers[ P_TRAV] = opcodeHandler_TRAV;
 
-   m_opHandlers[ P_FORI] = opcodeHandler_FORI;
-   m_opHandlers[ P_FORN] = opcodeHandler_FORN;
+   m_opHandlers[ P_INCP] = opcodeHandler_INCP;
+   m_opHandlers[ P_DECP] = opcodeHandler_DECP;
 
    m_opHandlers[ P_SHL ] = opcodeHandler_SHL;
    m_opHandlers[ P_SHR ] = opcodeHandler_SHR;
    m_opHandlers[ P_SHLS] = opcodeHandler_SHLS;
    m_opHandlers[ P_SHRS] = opcodeHandler_SHRS;
-   m_opHandlers[ P_LDVR] = opcodeHandler_LDVR;
-   m_opHandlers[ P_LDPR] = opcodeHandler_LDPR;
+//   m_opHandlers[ P_LDVR] = opcodeHandler_LDVR;
+//   m_opHandlers[ P_LDPR] = opcodeHandler_LDPR;
    m_opHandlers[ P_LSB ] = opcodeHandler_LSB;
    m_opHandlers[ P_UNPS ] = opcodeHandler_UNPS;
    m_opHandlers[ P_SELE ] = opcodeHandler_SELE;
@@ -217,6 +213,7 @@ void VMachine::internal_construct()
    m_opHandlers[ P_STEX ] = opcodeHandler_STEX;
    m_opHandlers[ P_TRAC ] = opcodeHandler_TRAC;
    m_opHandlers[ P_WRT ] = opcodeHandler_WRT;
+   m_opHandlers[ P_STO ] = opcodeHandler_STO;
 
    m_fbom = new BomMap();
    m_fbom->add( "toString" );
@@ -235,6 +232,8 @@ void VMachine::internal_construct()
    m_fbom->add( "trim" );
    m_fbom->add( "frontTrim" );
    m_fbom->add( "allTrim" );
+   m_fbom->add( "front" );
+   m_fbom->add( "back" );
 }
 
 
@@ -269,8 +268,7 @@ void VMachine::init()
 
 VMachine::~VMachine()
 {
-
-   delete  m_memPool ;
+   // Free generic tables (quite safe)
    memFree( m_opHandlers );
    delete m_fbom;
 
@@ -301,10 +299,12 @@ VMachine::~VMachine()
    delete m_stdIn;
    delete m_stdOut;
 
+   delete  m_memPool;
    // clear now the global maps
    // this also decrefs the modules and destroys the globals.
    // Notice that this would be done automatically also at destructor exit.
    m_liveModules.clear();
+
 }
 
 void VMachine::errorHandler( ErrorHandler *em, bool own )
@@ -331,7 +331,7 @@ bool VMachine::link( Runtime *rt )
 }
 
 
-bool VMachine::link( Module *mod, bool isMainModule )
+LiveModule *VMachine::link( Module *mod, bool isMainModule )
 {
    ItemVector *globs;
 
@@ -418,6 +418,7 @@ bool VMachine::link( Module *mod, bool isMainModule )
                Item *itm = globs->itemPtrAt( sym->itemId() );
                VarDef *vd = sym->getVarDef();
                switch( vd->type() ) {
+                  case VarDef::t_bool: itm->setBoolean( vd->asBool() ); break;
                   case VarDef::t_int: itm->setInteger( vd->asInteger() ); break;
                   case VarDef::t_num: itm->setNumeric( vd->asNumeric() ); break;
                   case VarDef::t_string:
@@ -431,7 +432,8 @@ bool VMachine::link( Module *mod, bool isMainModule )
          }
 
          // Is this symbol exported?
-         if ( sym->exported() ) {
+         if ( sym->exported() && sym->name().getCharAt(0) != '_' )
+         {
             // as long as the module is referenced, the symbols are alive, and as we
             // hold a reference to the module, we are sure that symbols are alive here.
             // also, in case an entry already exists, the previous item is just overwritten.
@@ -510,7 +512,7 @@ bool VMachine::link( Module *mod, bool isMainModule )
 
    // exit if link failed.
    if ( ! success )
-      return false;
+      return 0;
 
    // now that the symbols in the module have been linked, link the classes.
    ListElement *cls_iter = modClasses.begin();
@@ -549,7 +551,7 @@ bool VMachine::link( Module *mod, bool isMainModule )
                symbol( obj->name() ).
                module( obj->module()->name() ) )
          );
-         return false;
+         return 0;
       }
       else {
          CoreObject *co = clsItem->asClass()->createInstance();
@@ -587,7 +589,7 @@ bool VMachine::link( Module *mod, bool isMainModule )
             m_event = eventNone;
             callItem( *clsItem, 0, e_callInst );
             if ( m_event == eventRisen )
-               return false;
+               return 0;
          }
          obj_iter = obj_iter->next();
       }
@@ -598,7 +600,7 @@ bool VMachine::link( Module *mod, bool isMainModule )
       m_regS2 = oldS2;
    }
 
-   return true;
+   return livemod;
 }
 
 
@@ -620,6 +622,10 @@ PropertyTable *VMachine::createClassTemplate( LiveModule *lmod, const Map &pt )
             itm.setNil();
          break;
 
+         case VarDef::t_bool:
+            itm.setBoolean( vd->asBool() );
+         break;
+
          case VarDef::t_int:
             itm.setInteger( vd->asInteger() );
          break;
@@ -657,7 +663,15 @@ PropertyTable *VMachine::createClassTemplate( LiveModule *lmod, const Map &pt )
       }
 
       String *key = *(String **) iter.currentKey();
-      table->appendSafe( key, itm );
+      if( ! vd->isReflective() )
+         table->appendSafe( key, itm );
+      else {
+         PropertyTable::config conf;
+         conf.m_offset = vd->asReflectiveOffset();
+         conf.m_size = vd->asReflectiveSize();
+         conf.m_isSigned = vd->asReflectiveIsSigned();
+         table->appendSafe( key, itm, conf );
+      }
       iter.next();
    }
 
@@ -1141,7 +1155,7 @@ bool VMachine::callItem( const Item &callable, int32 paramCount, e_callMode call
       {
          m_bomParams = paramCount;
          bool bomRet = callable.callBom( this );
-         //m_stack->resize( m_stack->size() - paramCount );
+         m_stack->resize( m_stack->size() - paramCount );
          return bomRet;
       }
 
@@ -1156,6 +1170,7 @@ bool VMachine::callItem( const Item &callable, int32 paramCount, e_callMode call
          targetMod = callable.asModule();
       break;
 
+      case FLC_ITEM_CLSMETHOD:
       case FLC_ITEM_CLASS:
       {
          CoreClass *cls = callable.asClass();
@@ -1175,6 +1190,12 @@ bool VMachine::callItem( const Item &callable, int32 paramCount, e_callMode call
             return true;
          }
 
+         if ( ! cls->constructor().asModule()->isAlive() )
+         {
+            const_cast<Item *>(&callable)->setNil();
+            return false;
+         }
+
          target = cls->constructor().asFunction();
          targetMod = cls->constructor().asModule();
       }
@@ -1329,6 +1350,7 @@ bool VMachine::callItem( const Item &callable, int32 paramCount, e_callMode call
       // if we aren't in a frame call, call the item directly
       if( callMode == e_callNormal || callMode == e_callInst )
       {
+         regA().setNil(); // clear return value if calling external functions
          target->getExtFuncDef()->call( this );
          if ( callable.isClass() )
             m_regA.setObject( self );
@@ -1514,6 +1536,7 @@ bool VMachine::callItemPass( const Item &callable  )
    else
    {
       m_stack->resize( m_stackBase );
+      regA().setNil(); // clear return value if calling external functions
       target->getExtFuncDef()->call( this );
       m_pc_next = ((StackFrame *)m_stack->at( m_stackBase - VM_FRAME_SPACE ))->m_ret_pc;
       callReturn();
@@ -1542,7 +1565,7 @@ void VMachine::yield( numeric secs )
 {
    if ( m_atomicMode )
    {
-      raiseError( new InterruptedError( ErrorParam( e_interrupted ).origin( e_orig_vm ).
+      raiseError( new InterruptedError( ErrorParam( e_wait_in_atomic ).origin( e_orig_vm ).
             symbol( "yield" ).
             module( "core.vm" ).
             line( __LINE__ ).
@@ -1566,7 +1589,17 @@ void VMachine::yield( numeric secs )
          else
          {
             // else just ask the system to sleep.
-            Sys::_sleep( secs );
+            if ( ! m_systemData.sleep( secs ) )
+            {
+               m_systemData.resetInterrupt();
+
+               raiseError( new InterruptedError(
+                  ErrorParam( e_interrupted ).origin( e_orig_vm ).
+                     symbol( m_symbol->name() ).
+                     module( m_currentModule->name() ).
+                     line( m_currentModule->getLineAt( m_pc ) )
+                  ) );
+            }
          }
       }
    }
@@ -1673,7 +1706,7 @@ void VMachine::electContext()
       m_pc = m_pc_next;
 
       // but eventually sleep.
-	  if ( tgtTime > 0.0 )
+      if ( tgtTime > 0.0 )
       {
          // should we ask the embedding application to wait for us??
          if ( m_sleepAsRequests )
@@ -1682,8 +1715,20 @@ void VMachine::electContext()
             m_yieldTime = tgtTime;
             return;
          }
-         else
-            Sys::_sleep( tgtTime );
+         else {
+            // raise an interrupted error on need.
+            if ( ! m_systemData.sleep( tgtTime ) )
+            {
+               m_systemData.resetInterrupt();
+
+               raiseError( new InterruptedError(
+                  ErrorParam( e_interrupted ).origin( e_orig_vm ).
+                     symbol( m_symbol->name() ).
+                     module( m_currentModule->name() ).
+                     line( m_currentModule->getLineAt( m_pc ) )
+                  ) );
+            }
+         }
       }
    }
 }
@@ -2232,6 +2277,13 @@ Item *VMachine::findLocalSymbolItem( const String &symName ) const
    if( sym == 0 )
    {
       sym = currentModule()->findGlobalSymbol( symName );
+      // still zero? Let's try the global symbol table.
+      if( sym == 0 )
+      {
+         Item *itm = findGlobalItem( symName );
+         if ( itm != 0 )
+            return itm->dereference();
+      }
    }
 
    Item *itm = 0;
@@ -2798,7 +2850,7 @@ int VMachine::compareItems( const Item &first, const Item &second )
       if( fo->getMethod( "compare", comparer ) )
       {
          Item oldA = m_regA;
-         m_regA = (int64)0;
+         m_regA.setBoolean( false );
 
          pushParameter( second );
          // enter atomic mode
@@ -3082,6 +3134,29 @@ bool VMachine::unlink( const Module *module )
    return true;
 }
 
+
+bool VMachine::interrupted( bool raise, bool reset, bool dontCheck )
+{
+   if( dontCheck || m_systemData.interrupted() )
+   {
+      if( reset )
+         m_systemData.resetInterrupt();
+
+      if ( raise )
+         raiseError( new InterruptedError(
+            ErrorParam( e_interrupted ).origin( e_orig_vm ).
+               symbol( m_symbol->name() ).
+               module( m_currentModule->name() ).
+               line( m_currentModule->getLineAt( m_pc ) )
+            ) );
+
+      return true;
+   }
+
+   return false;
+}
+
+
 }
 
 /* end of vm.cpp */
