@@ -6,16 +6,11 @@
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: dom giu 6 2004
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #ifndef FALCON_COMPILER_H
@@ -123,7 +118,7 @@ class FALCON_DYN_CLASS Compiler: public BaseAlloc
    /** This is the module that is being formed in the meanwhile. */
    Module *m_module;
 
-   uint32 m_enumId;
+   int64 m_enumId;
 
    /** Leading instruction that owns currently parsed statements */
    List m_context;
@@ -163,10 +158,18 @@ class FALCON_DYN_CLASS Compiler: public BaseAlloc
    const String *m_staticPrefix;
 
    int m_lambdaCount;
-
+   int m_closureContexts;
    int m_tempLine;
 
+   /** Directive strict. */
    bool m_strict;
+
+   /** Directive language. */
+   String m_language;
+
+   /** Directive version. */
+   int64 m_modVersion;
+
    bool m_defContext;
 
    bool m_bParsingFtd;
@@ -266,6 +269,7 @@ public:
    Symbol *searchGlobalSymbol( const String *symname );
    Symbol *addLocalSymbol( const String *symname, bool parameter );
    Symbol *addGlobalSymbol( const String *symname );
+   Symbol *searchOuterSymbol( const String *symname );
    Symbol *addAttribute( const String *symname );
 
    /** Creates a symbol that will be an initially defined global variable.
@@ -451,6 +455,20 @@ public:
    void defContext( bool ctx ) { m_defContext = ctx; }
    bool defContext() const { return m_defContext; }
 
+   /** Closes the currently worked on closure */
+   Value *closeClosure();
+   void incClosureContext() { m_closureContexts++; }
+   void decClosureContext() { m_closureContexts--; }
+
+   /** Add an enumeration item to current enumeration. */
+   void addEnumerator( const String &name, Value *value );
+
+   /** Add an enumeration item to current enumeration.
+      This version assigns the enumerated value a progressive integer.
+   */
+   void addEnumerator( const String &name );
+
+   void resetEnum() { m_enumId = 0; }
 };
 
 } // end of namespace
