@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: flc_deptab.h
-   $Id: deptab.h,v 1.2 2006/10/15 20:21:50 gian Exp $
 
    Dependency table declaration
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: dom ago 8 2004
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #ifndef flc_DEPTAB_H
