@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: dir_sys_unix.h
-   $Id: dir_sys_unix.h,v 1.1 2007/06/21 21:54:25 jonnymind Exp $
 
    Support for directory oriented operations in unix systems
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: dom nov 7 2004
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
