@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: dll_mac.h
-   $Id: dll_mac.h,v 1.1.1.1 2006/10/08 15:05:39 gian Exp $
 
    libdl Dynamic Link support
    -------------------------------------------------------------------
    Author: Guerra Francesco
    Begin: sab mag 06 2006
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2006: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
