@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: error.h
-   $Id: error.h,v 1.17 2007/08/18 17:59:33 jonnymind Exp $
 
    Error class.
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: dom feb 18 2007
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -155,7 +149,7 @@ const int e_syn_continue =         145;
 const int e_syn_for =              146;
 const int e_syn_forfirst =         147;
 const int e_syn_forlast =          148;
-const int e_syn_forall =           149;
+const int e_syn_formiddle =        149;
 const int e_syn_try =              150;
 const int e_syn_catch =            151;
 const int e_syn_raise =            152;
@@ -191,6 +185,7 @@ const int e_syn_def =              181;
 const int e_syn_fordot =           182;
 const int e_syn_self_print =       183;
 const int e_syn_directive =        184;
+const int e_syn_import =           185;
 
 const int e_nl_in_lit =            200;
 const int e_catch_adef =           201;
@@ -198,7 +193,7 @@ const int e_catch_clash =          202;
 const int e_fmt_convert =          203;
 const int e_already_forfirst =     204;
 const int e_already_forlast =      205;
-const int e_already_forall =       206;
+const int e_already_formiddle =    206;
 const int e_fordot_outside =       207;
 const int e_interrupted =          208;
 const int e_priv_access =          209;
@@ -207,12 +202,17 @@ const int e_square_unbal =         211;
 const int e_unclosed_string =      212;
 const int e_directive_unk =        213;
 const int e_directive_value =      214;
+const int e_noninst_cls =          215;
+const int e_unserializable =       216;
+const int e_wait_in_atomic =       217;
 
 const int e_open_file =            500;
 const int e_loaderror =            501;
 const int e_nofile =               503;
 const int e_invformat =            504;
 const int e_loader_unsupported =   505;
+const int e_io_error =             506;
+const int e_unknown_encoding =     507;
 
 const int e_inv_params =           900;
 const int e_missing_params =       901;
@@ -359,6 +359,8 @@ private:
 class FALCON_DYN_CLASS Error: public BaseAlloc
 {
 protected:
+   volatile long m_refCount;
+   
    int m_errorCode;
    String m_description;
    String m_extra;
@@ -371,8 +373,6 @@ protected:
    uint32 m_pc;
    uint32 m_sysError;
 
-   uint32 m_refCount;
-
    t_origin m_origin;
    bool m_catchable;
    Item m_raised;
@@ -544,15 +544,15 @@ public:
       {}
 };
 
-class RangeError: public Error
+class AccessError: public Error
 {
 public:
-   RangeError():
-      Error( "RangeError" )
+   AccessError():
+      Error( "AccessError" )
    {}
 
-   RangeError( const ErrorParam &params  ):
-      Error( "RangeError", params )
+   AccessError( const ErrorParam &params  ):
+      Error( "AccessError", params )
       {}
 };
 
@@ -652,9 +652,9 @@ public:
    ErrorCarrier( Error *carried );
 
    virtual ~ErrorCarrier();
-   virtual bool isReflective();
-   virtual void getProperty( const String &propName, Item &prop );
-   virtual void setProperty( const String &propName, Item &prop );
+   virtual bool isReflective() const;
+   virtual void getProperty( VMachine *vm, const String &propName, Item &prop );
+   virtual void setProperty( VMachine *vm, const String &propName, Item &prop );
 
    Error *error() const { return m_error; }
 };
@@ -669,7 +669,7 @@ FALCON_FUNC_DYN_SYM Error_init ( ::Falcon::VMachine *vm );
 FALCON_FUNC_DYN_SYM SyntaxError_init ( ::Falcon::VMachine *vm );
 FALCON_FUNC_DYN_SYM CodeError_init ( ::Falcon::VMachine *vm );
 FALCON_FUNC_DYN_SYM IoError_init ( ::Falcon::VMachine *vm );
-FALCON_FUNC_DYN_SYM RangeError_init ( ::Falcon::VMachine *vm );
+FALCON_FUNC_DYN_SYM AccessError_init ( ::Falcon::VMachine *vm );
 FALCON_FUNC_DYN_SYM MathError_init ( ::Falcon::VMachine *vm );
 FALCON_FUNC_DYN_SYM ParamError_init ( ::Falcon::VMachine *vm );
 FALCON_FUNC_DYN_SYM ParseError_init ( ::Falcon::VMachine *vm );
