@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: format.h
-   $Id: format.h,v 1.11 2007/07/20 08:24:49 jonnymind Exp $
 
    Format base class.
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: sab giu 16 2007
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -427,7 +421,7 @@ public:
    //=================================================
    // Reflection settings
    //
-   virtual bool isReflective();
+   virtual bool isReflective() const;
    virtual void getProperty( const String &propName, Item &prop );
    virtual void setProperty( const String &propName, Item &prop );
 };
