@@ -6,16 +6,11 @@
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: lun ott 4 2004
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -46,6 +41,7 @@ class VMachine;
 class Stream;
 class Attribute;
 class LiveModule;
+class MemBuf;
 
 /** Basic item abstraction.*/
 class FALCON_DYN_CLASS Item: public BaseAlloc
@@ -107,17 +103,25 @@ private:
          LiveModule *m_liveMod;
       };
 
+      struct {
+         int32 rstart;
+         int32 rend;
+         int32 rstep;
+      };
+
    } m_data;
 
 
    bool internal_is_equal( const Item &other ) const;
    int internal_compare( const Item &other ) const;
-   bool serialize_object( Stream *file, const CoreObject *obj, VMachine *vm ) const;
+   bool serialize_object( Stream *file, const CoreObject *obj, bool bLive ) const;
    bool serialize_symbol( Stream *file, const Symbol *sym ) const;
-   bool serialize_function( Stream *file, const Symbol *func, VMachine *vm ) const;
+   bool serialize_function( Stream *file, const Symbol *func ) const;
+   bool serialize_class( Stream *file, const CoreClass *cls ) const;
 
    e_sercode deserialize_symbol( Stream *file, VMachine *vm, Symbol **tg_sym, LiveModule **modId );
    e_sercode deserialize_function( Stream *file, VMachine *vm );
+   e_sercode deserialize_class( Stream *file, VMachine *vm );
 
 #ifdef _MSC_VER
 	#if _MSC_VER < 1299
@@ -155,12 +159,26 @@ public:
       copy( other );
    }
 
+   /** Creates a boolean item. */
+   explicit Item( bool tof )
+   {
+      setBoolean( tof );
+   }
+
+   /** Sets this item as boolean */
+   void setBoolean( bool tof )
+   {
+      type( FLC_ITEM_BOOL );
+      m_data.num.val1 = tof?1: 0;
+   }
+
    /** Creates an integer item */
    Item( int64 val )
    {
       setInteger( val );
    }
 
+
    /** Creates an integer item */
    Item( uint64 val )
    {
@@ -192,8 +210,18 @@ public:
    void setRange( int32 val1, int32 val2, bool open )
    {
       type( FLC_ITEM_RANGE );
-      m_data.num.val1 = val1;
-      m_data.num.val2 = val2;
+      m_data.rstart = val1;
+      m_data.rend = val2;
+      m_data.rstep = 0;
+      m_base.bits.flags = open? flagOpenRange : 0;
+   }
+
+   void setRange( int32 val1, int32 val2, int32 step, bool open )
+   {
+      type( FLC_ITEM_RANGE );
+      m_data.rstart = val1;
+      m_data.rend = val2;
+      m_data.rstep = step;
       m_base.bits.flags = open? flagOpenRange : 0;
    }
 
@@ -253,6 +281,17 @@ public:
       m_data.voidp = dict;
    }
 
+   /** Creates a memory buffer. */
+   Item( MemBuf *buf )
+   {
+      setMemBuf( buf );
+   }
+
+   void setMemBuf( MemBuf *b ) {
+      type( FLC_ITEM_MEMBUF );
+      m_data.voidp = b;
+   }
+
    /** Creates a reference to another item. */
    void setReference( GarbageItem *ref ) {
       type( FLC_ITEM_REFERENCE );
@@ -308,7 +347,8 @@ public:
 
    void setClass( CoreClass *cls ) {
       type( FLC_ITEM_CLASS );
-      m_data.voidp = cls;
+      // warning: class in extra to be omologue to methodClass()
+      m_data.m_extra = cls;
    }
 
    /** Sets an item as a FBOM.
@@ -395,24 +435,29 @@ public:
       m_data = other.m_data;
    }
 
-   bool isCallable() const;
-
    /** Tells if this item is callable.
-      The non-const version of this function will turn this object into a nil
-      if the item referenced a dead module. 
+      This function will turn this object into a nil
+      if the item referenced a dead module. As this is a pathological
+      situation, a const cast is forced.
    */
-   bool isCallable();
+   bool isCallable() const;
 
    bool isOrdinal() const {
       return type() == FLC_ITEM_INT || type() == FLC_ITEM_NUM;
    }
 
+   bool asBoolean() const
+   {
+      return m_data.num.val1 != 0;
+   }
+
    int64 asInteger() const { return m_data.val64; }
 
    numeric asNumeric() const { return m_data.number; }
 
-   int32 asRangeStart() const { return m_data.num.val1; }
-   int32 asRangeEnd()  const { return m_data.num.val2; }
+   int32 asRangeStart() const { return m_data.rstart; }
+   int32 asRangeEnd()  const { return m_data.rend; }
+   int32 asRangeStep()  const { return m_data.rstep; }
    bool asRangeIsOpen()  const { return (m_base.bits.flags & flagOpenRange) != 0; }
 
    String *asString() const { return (String *) m_data.voidp; }
@@ -424,8 +469,9 @@ public:
    CoreArray *asArray() const { return (CoreArray *) m_data.voidp; }
    CoreObject *asObject() const { return (CoreObject *) m_data.voidp; }
    CoreDict *asDict() const { return ( CoreDict *) m_data.voidp; }
+   MemBuf *asMemBuf() const { return ( MemBuf *) m_data.voidp; }
 
-   CoreClass *asClass() const { return (CoreClass *) m_data.voidp; }
+   CoreClass *asClass() const { return (CoreClass *) m_data.m_extra; }
    Symbol *asFunction() const { return (Symbol *) m_data.voidp; }
 
    CoreObject *asMethodObject() const { return (CoreObject *) m_data.voidp; }
@@ -452,6 +498,7 @@ public:
    uint32 hash() const;
 
    bool isNil() const { return type() == FLC_ITEM_NIL; }
+   bool isBoolean() const { return type() == FLC_ITEM_BOOL; }
    bool isInteger() const { return type() == FLC_ITEM_INT; }
    bool isNumeric() const { return type() == FLC_ITEM_NUM; }
    bool isScalar() const { return type() == FLC_ITEM_INT || type() == FLC_ITEM_NUM; }
@@ -459,6 +506,7 @@ public:
    bool isString() const { return type() == FLC_ITEM_STRING; }
    bool isArray() const { return type() == FLC_ITEM_ARRAY; }
    bool isDict() const { return type() == FLC_ITEM_DICT; }
+   bool isMemBuf() const { return type() == FLC_ITEM_MEMBUF; }
    bool isObject() const { return type() == FLC_ITEM_OBJECT; }
    bool isReference() const { return type() == FLC_ITEM_REFERENCE; }
    bool isFunction() const { return type() == FLC_ITEM_FUNC; }
@@ -539,15 +587,7 @@ public:
 
 
    /** Serialize this item.
-      This method stores an item on a stream for later retrival. Providing a virtual
-      machine that is optional; if not provided, items requiring the VM for serialization
-      won't be correctly serialized. In example, objects may provide a serialize() method;
-      if that method exists, the VM would call it in order to provide the object with its own
-      personalized serialization.
-
-      Unless certain that the given object is not an object with a serialized method,
-      or does not references one at any level, this function should always be called with
-      a VM ready.
+      This method stores an item on a stream for later retrival.
 
       The function can return true if the serialization succeded. It will return false if
       the serialization failed; in that case, if there is an error on the stream, it can
@@ -557,10 +597,10 @@ public:
       VM received an error during the method call.
 
       \param out the output stream
-      \param vm the virtual machine that can be used for object serialization
+      \param bLive true
       \return an error code in case of error (\see e_sercode).
    */
-   e_sercode serialize( Stream *out, VMachine *vm = 0 ) const;
+   e_sercode serialize( Stream *out, bool bLive = false ) const;
 
 
    /** Loads a serialized item from a stream.
