@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: flc_itemid.h
-   $Id: itemid.h,v 1.4 2007/08/11 22:59:07 jonnymind Exp $
 
    List of item ids
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: ven ott 15 2004
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -31,6 +25,7 @@
 #define FLC_ITEM_INT       1
 #define FLC_ITEM_NUM       2
 #define FLC_ITEM_RANGE     3
+#define FLC_ITEM_BOOL      4
 
 #define FLC_ITEM_FUNC      6
 /* Some shallow callable items. */
@@ -49,6 +44,7 @@
 #define FLC_ITEM_ARRAY     11
 #define FLC_ITEM_DICT      12
 #define FLC_ITEM_OBJECT    13
+#define FLC_ITEM_MEMBUF    14
 
 #define FLC_ITEM_REFERENCE 15
 
