@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: itemtraits.h
-   $Id: itemtraits.h,v 1.5 2007/08/11 00:11:52 jonnymind Exp $
 
    Traits for putting items in generic containers
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: dom ott 29 2006
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
