@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: flc_modloader.h
-   $Id: modloader.h,v 1.9 2007/08/11 18:26:44 jonnymind Exp $
 
    Short description
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: mar ago 3 2004
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #ifndef flc_MODLOADER_H
@@ -68,6 +62,10 @@ private:
 
 protected:
    ErrorHandler *m_errhand;
+   bool m_bSaveIntTemplate;
+
+   /** Required language during load. */
+   String m_language;
 
    Module *loadModule_select_ver( Stream *in );
 
@@ -166,6 +164,9 @@ protected:
 
    virtual t_filetype fileType( const String &path );
 
+   /** Try to load the language table for the given module. */
+   bool applyLangTable( Module *mod, const String &file_path );
+
 public:
 
    /** Creates a module loader.
@@ -442,11 +443,58 @@ public:
       return temp;
    }
 
+   /** Sets the error handler.
+      The module loader never owns the handler; it must be disposed separately.
+   */
    void errorHandler( ErrorHandler *h ) { m_errhand = h; }
+
+   /** Returns the error handler.
+   */
    ErrorHandler *errorHandler() const { return m_errhand; }
+
+   /** Save international templates for loaded modules.
+      If this option is set to true, and if the loaded modules
+      have international strings, then a template for the
+      internationalization file will be saved.
+   */
+   void saveIntTemplates( bool mode /*, bool force=false */ )
+   {
+      m_bSaveIntTemplate = mode;
+   }
+
+   /** Sets the language required to modules during load.
+      This informs the module loader that the owner wishes the string table
+      of the loaded module configured for the given language.
+
+      If the loaded module doesn't declare itself to be written in the
+      desired language, the module loader will try to load \b modulename.ftr
+      binary file, get the table for the desired language and change the
+      strings according to the translation table before returning it to
+      the caller.
+
+      In case of failure, the original string table will be left untouched.
+
+      Language names are the ISO language names in 5 characters: xx_YY.
+
+      Setting the language to "" disables this feature.
+
+      \param langname the name of the language that should be loaded.
+   */
+   void setLanguage( const String &langname ) { m_language = langname; }
+
+   /** Returns the translation language that is searched by this module loader.
+   */
+   const String &getLanguage() const { return m_language; }
+
+   /** Load a determined language table directly into the module.
+      On success, the language table of the module and it's declared language
+      are changed.
+      \return true on success.
+   */
+   bool loadLanguageTable( Module *module, const String &language );
 };
 
 }
 
 #endif
-/* end of flc_modloader.h */
+/* end of modloader.h */
