@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: module.h
-   $Id: module.h,v 1.13 2007/08/19 09:46:44 jonnymind Exp $
 
-   Short description
+   Falcon module extensions.
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: mar ago 3 2004
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #ifndef FLC_MODULE_H
@@ -61,6 +55,7 @@ class Stream;
 */
 class FALCON_DYN_CLASS Module: public BaseAlloc
 {
+   volatile long m_refcount;
 
    /******************************
    * Anagraphic section
@@ -69,7 +64,7 @@ class FALCON_DYN_CLASS Module: public BaseAlloc
    char m_subversion;
    String m_name;
    String m_path;
-   uint32 m_refcount;
+   String m_language;
    uint32 m_modVersion;
    uint32 m_engineVersion;
 
@@ -110,7 +105,7 @@ class FALCON_DYN_CLASS Module: public BaseAlloc
       binary Falcon objects.
    */
    DllLoader *m_loader;
-   
+
    /** Detach the loader for decref.
       If the module has been loaded through a DLL, decreffing it to death
       should unload the module. If this was the last instance of the module
@@ -123,7 +118,7 @@ class FALCON_DYN_CLASS Module: public BaseAlloc
    DllLoader *detachLoader();
 
    Map m_serviceMap;
-   
+
    virtual ~Module();
 
 public:
@@ -139,6 +134,8 @@ public:
    Module &code( byte *code ) { m_code = code; return *this; }
    Module &codeSize( uint32 cs ) { m_codeSize = cs; return *this; }
    Module &entry( uint32 entry ) { m_entry = entry; return *this; }
+   Module &language( const String &lang ) { m_language = lang; return *this; }
+
 
    Module &version( uint32 version ) { m_modVersion = version; return *this; }
    Module &version( int major, int minor, int revision ) {
@@ -152,8 +149,11 @@ public:
       return *this;
    }
 
-   void incref() { m_refcount ++; }
-   void decref(); 
+   virtual char pcodeVersion() const;
+   virtual char pcodeSubVersion() const;
+
+   void incref();
+   void decref();
 
    Symbol *findGlobalSymbol( const String &name ) const
    {
@@ -162,14 +162,43 @@ public:
 
    const String &name() const { return m_name; }
    const String &path() const { return m_path; }
+   const String &language() const { return m_language;}
 
    byte *code() const { return m_code; }
    uint32 codeSize() const { return m_codeSize; }
    uint32 entry() const { return m_entry; }
 
    String *addCString( const char *pos, uint32 size );
+
+   /** Add a string to the module string table.
+      If the string already exists, the previous entry is returned.
+      \note If the string isn't found in the string table and is
+         created on the spod, its exported status
+         (international string) is copied from \b st.
+
+      \param st the string to be added.
+      \return A pointer to the string in the module string table.
+   */
    String *addString( const String &st );
 
+   /** Add a string to the module string table and eventually exports it.
+      If the string already exists, the previous entry is returned.
+      This method also set the "exported" bit of the returned string,
+      so that the string can be directly used in exported intenrationalization
+      tables.
+      \param st the string to be added.
+      \param exported set to true to make this string subject to
+                      internationalization table exports.
+      \return A pointer to the string in the module string table.
+   */
+   String *addString( const String &st, bool exported );
+
+   /** Return a string given an ID in the string table.
+      This function doesn't check for out-of-bounds access, so be careful.
+
+      \param id the ID of the string in the string table (insertion order).
+      \return A pointer to the string.
+   */
    const String *getString( uint32 id ) const { return m_strTab.get(id); }
 
    Symbol *getSymbol( uint32 id ) const
@@ -514,9 +543,6 @@ public:
    */
    virtual bool load( Stream *is, bool skipHeader=true );
 
-   virtual char version() const;
-   virtual char subVersion() const;
-
    const StringTable &stringTable() const { return m_strTab; }
    StringTable &stringTable() { return m_strTab; }
 
@@ -565,12 +591,36 @@ public:
    */
    Symbol *addConstant( const String &name, const String &value, bool exp = true );
 
+   /** Get the version declared by this module */
    void getModuleVersion( int &major, int &minor, int &revision ) const;
+
+   /** Get the version of the engine libraries under which this module has been compiled. */
    void getEngineVersion( int &major, int &minor, int &revision ) const;
+
+   /** Save internationalization infos on required file.
+      Creates an XML file staring with ?xml tag.
+      Shouldn't be called if it's table has no international strintgs.
+   */
+   bool saveTableTemplate( Stream *stream, const String &encoding ) const;
 };
 
 }
 
-#endif
-/* end of flc_module.h */
+//====================================================
+// Module declaration and service macros
+//
 
+// module init and declaration
+#define DEFALUT_FALCON_MODULE_INIT falcon_module_init
+
+#define FALCON_MODULE_DECL \
+   FALCON_MODULE_TYPE DEFALUT_FALCON_MODULE_INIT
+
+// Module string table service functions
+#ifndef FAL_STR
+   #define FAL_STR( id )   vm->moduleString( id )
+#endif
+
+#endif
+
+/* end of module.h */
