@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: stackframe.h
-   $Id: stackframe.h,v 1.5 2007/08/18 11:08:06 jonnymind Exp $
 
    Short description
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: sab mar 18 2006
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -49,7 +43,7 @@ typedef struct tag_StackFrame
    bool m_initFrame;
    bool m_break;
    bool m_suspend;
-   byte m_param_count;
+   uint16 m_param_count;
    uint32 m_stack_base;
    uint32 m_try_base;
    Item m_sender;
