@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: string.h
-   $Id: string.h,v 1.20 2007/08/19 09:46:44 jonnymind Exp $
 
    Core falcon string representation.
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: ven nov 19 2004
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -310,6 +304,14 @@ protected:
     */
    byte m_gcStatus;
 
+   /**
+    * True if this string is exportable - importable in GC context.
+    */
+   bool m_bExported;
+
+   // Reserved for future usage.
+   byte m_bFlags;
+
    /**
     * Creates the core string.
     *
@@ -346,6 +348,7 @@ public:
       m_size( 0 ),
       m_storage( 0 ),
       m_allocated( 0 ),
+      m_bExported( false ),
       m_class( &csh::handler_static ),
       m_garbageable( false )
    {}
@@ -678,9 +681,11 @@ public:
    void setCharAt( uint32 pos, uint32 chr ) { m_class->setCharAt( this, pos, chr ); }
    String subString( int32 start, int32 end ) const { return String( *this, start, end ); }
    String subString( int32 start ) const { return String( *this, start, length() ); }
-   bool change( int32 start, const String *other ) { return m_class->change( this, start, csh::npos, other ); }
-   bool change( int32 start, int32 end, const String *other ) {
-      return m_class->change( this, start, end, other );
+   bool change( int32 start, const String &other ) {
+      return m_class->change( this, start, csh::npos, &other );
+   }
+   bool change( int32 start, int32 end, const String &other ) {
+      return m_class->change( this, start, end, &other );
    }
    void insert( uint32 pos, uint32 len, const String &source ) { m_class->insert( this, pos, len, &source ); }
    void remove( uint32 pos, uint32 len ) { m_class->remove( this, pos, len ); }
@@ -768,12 +773,16 @@ public:
    String & operator+=( const wchar_t *other ) { append( String( other ) ); return *this; }
 
    String & operator=( const String &other ) {
-      if ( m_allocated != 0 )
-         m_class->destroy( this );
       copy( other );
       return *this;
    }
 
+   String & operator=( uint32 chr ) {
+      m_size = 0;
+      append( chr );
+      return *this;
+   }
+
    /** Assign from a const char string.
       If this string is not empty, its content are destroyed; then
       this object is changed into a static zero terminated C string and
@@ -955,6 +964,8 @@ public:
     */
     void writeNumber( double number, const String &format );
 
+    void writeNumber( int64 number, const String &format );
+
     /** Checks the position to be in the string, and eventually changes it if it's negative.
       This is just a nice inline shortuct so that the string constructor for substrings
       can be called safely.
@@ -1075,6 +1086,25 @@ public:
       \return true on success, false if the sequence is invalid.
    */
    bool fromUTF8( const char *utf8 );
+
+   /** Access to a single character.
+      Please, notice that Falcon strings are polymorphic in assignment,
+      so they cannot support the following syntax:
+      \code
+         s[n] = c; // can't work with Falcon strings.
+      \endcode
+
+      This operator is provided as a candy grammar for getCharAt().
+   */
+   const uint32 operator []( uint32 pos ) const { return getCharAt( pos ); }
+
+   /** Return wether this exact string instance should be internationalized.
+      \note exported() attribute is not copied across string copies.
+   */
+   bool exported() const { return m_bExported; }
+   /** Sets wether this string should be exported in international context or not.
+   */
+   void exported( bool e ) { m_bExported = e; }
 };
 
 
@@ -1126,7 +1156,7 @@ public:
 
    This forces the garbage collector to have two chains of garbageable items,
    the normal items and the strings, but as a great deal of strings in a falcon based
-   applications are not menant to be garbage collected, the differentiation between
+   applications are not meant to be garbage collected, the differentiation between
    garbageable strings and program strings pays back the extra complexity of this
    management.
 
