@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: flc_symbol.h
-   $Id: symbol.h,v 1.10 2007/08/19 09:46:44 jonnymind Exp $
 
    Short description
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: mar ago 3 2004
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #ifndef FLC_SYMBOL_H
@@ -54,17 +48,27 @@ public:
    typedef enum {
       t_nil,
       t_int,
+      t_bool,
       t_num,
       t_string,
       t_symbol,
       t_base,
-      t_reference
+      t_reference,
+      t_reflective
    } t_type;
 private:
    t_type m_val_type;
 
+   struct t_reflect {
+      int16 offset;
+      int16 size;
+      bool isSigned;
+   };
+
    union {
+      bool val_bool;
       uint64 val_int;
+      struct t_reflect val_reflect;
       numeric val_num;
       const String *val_str;
       const Symbol *val_sym;
@@ -76,6 +80,12 @@ public:
       m_val_type(t_nil)
    {}
 
+   explicit VarDef( bool val ):
+      m_val_type(t_bool)
+   {
+      m_value.val_bool = val;
+   }
+
    VarDef( int64 val ):
       m_val_type(t_int)
    {
@@ -114,6 +124,7 @@ public:
 
    t_type type() const { return m_val_type; }
    void setNil() { m_val_type = t_nil; }
+   void setBool( bool val ) { m_val_type = t_bool; m_value.val_bool = val; }
    void setInteger( uint64 val ) { m_val_type = t_int; m_value.val_int = val; }
    void setString( const String *str ) { m_val_type = t_string; m_value.val_str = str; }
    void setSymbol( const Symbol *sym ) { m_val_type = t_symbol; m_value.val_sym = sym; }
@@ -121,18 +132,110 @@ public:
    void setBaseClass( const Symbol *sym ) { m_val_type = t_base; m_value.val_sym = sym; }
    void setReference( const Symbol *sym ) { m_val_type = t_reference; m_value.val_sym = sym; }
 
+   /** Declares this vardef to be reflective.
+      This means this vardef is relative to a C or C++ structure that
+      can be directly filled, or that can fill directly, data from
+      and to CoreObjects instances.
+
+      VarDefs may represent class properties. When the class defined
+      by the vardefs is istantiated, the reflectivity information
+      passes to the properties in the final core object, and they
+      can be used to transfer automatically data to and from
+      C structures through CoreObject::configureFrom() and
+      CoreObject::configureTo().
+
+      Example:
+
+      \code
+         Symbol *someClass = self->addClass( "SomeClass" );
+         someClass->setWKS( true );
+         some_c_structure cstruct;
+
+         self->addClassProperty( someClass, "field1" )->
+            setReflective( (uint16)(&cstruct.field1 - &cstruct), // offset of the field
+            sizeof( cstruct.field1 ), // size of the field; 1,2,4 or 8 bytes
+            true ); //signed or unsigned data.
+
+         // ... later on, in the program...
+         // when the application wants to pass the data to a falcon program:
+
+         CoreObject *BuildFalconObjectFromStruct( VMachine *vm, const some_c_structure &cstruct )
+         {
+            CoreClass *cls = vm->findWKI( "SomeClass" );
+            CoreObject *myobject = cls->createInstance();
+            myobject->configureFrom( &cstruct );
+            return myobject;
+         }
+
+         // And when we want to read back the data...
+
+         void ReadBackData( some_c_structure &cstruct, CoreObject *co );
+         {
+            co->configureTo( &cstruct );
+         }
+      \endcode
+
+      \param offset distance from base data of the reflective data
+      \param size size of reflective binary data
+      \param is True is signed, false if signed.
+   */
+   void setReflective( uint16 offset, uint16 size, bool is = false ) {
+      m_val_type = t_reflective;
+      m_value.val_reflect.offset = offset;
+      m_value.val_reflect.size = size;
+      m_value.val_reflect.isSigned = is;
+   }
+
+   /** Declares this vardef to be reflective.
+      This is just a shortcut that calculates the offset distance between
+      the data (base address) and the dest (field address).
+
+      So, it actually calls setReflective( uint16, uint16, bool ),
+      passing (dest - data) as offset.
+      Example:
+      \code
+         Symbol *someClass = self->addClass( "SomeClass" );
+
+         some_c_structure cstruct;
+
+         self->addClassProperty( someClass, "field1" )->
+            setReflective( &cstruct,         // base address of a structure
+                           &cstruct.field1,  // position of the field in that structure
+                           sizeof( cstruct.field1 ), // size of the field; 1,2,4 or 8 bytes
+                           true ); //signed or unsigned data.
+      \endcode
+
+      \param data base data address for the reflective field
+      \param dest address of a field in the structure
+      \param size size of reflective binary data
+      \param is true is signed, false if signed.
+   */
+   void setReflective( void *data, void *dest, uint16 size, bool is = false )
+   {
+      byte *p1 = (byte *) data;
+      byte *p2 = (byte *) dest;
+      setReflective( (uint16) (p2 - p1), size, is );
+   }
+
+
+   bool asBool() const { return m_value.val_bool; }
    int64 asInteger() const { return m_value.val_int; }
    const String *asString() const { return m_value.val_str; }
    const Symbol *asSymbol() const { return m_value.val_sym; }
    numeric asNumeric() const { return m_value.val_num; }
+   int16 asReflectiveOffset() { return m_value.val_reflect.offset; }
+   int16 asReflectiveSize() { return m_value.val_reflect.size; }
+   bool asReflectiveIsSigned() { return m_value.val_reflect.isSigned; }
 
    bool isNil() const { return m_val_type == t_nil; }
+   bool isBool() const { return m_val_type == t_bool; }
    bool isInteger() const { return m_val_type == t_int; }
    bool isString() const { return m_val_type == t_string; }
    bool isNumeric() const { return m_val_type == t_num; }
    bool isSymbol() const { return m_val_type == t_symbol || m_val_type == t_base; }
    bool isBaseClass() const { return m_val_type == t_base; }
    bool isReference() const { return m_val_type == t_reference; }
+   bool isReflective() const { return m_val_type == t_reflective; }
 
    bool save( Stream *out ) const;
    bool load( Module *mod, Stream *in );
@@ -178,14 +281,16 @@ class FALCON_DYN_CLASS FuncDef: public BaseAlloc
    /** Count of local variables */
    uint16 m_locals;
 
+   /** Count of local variables - still undefined */
+   uint16 m_undefined;
+
    /** Cache item for functions with static elements.
       If the function is not static, it will be NO_STATE.
    */
    uint32 m_onceItemId;
 
 public:
-
-	enum {
+   enum {
 		NO_STATE = 0xFFFFFFFF
 	} enum_NO_STATE;
 
@@ -197,11 +302,17 @@ public:
 
    Symbol *addParameter( Symbol *sym );
    Symbol *addLocal( Symbol *sym );
+   Symbol *addUndefined( Symbol *sym );
 
    uint32 offset() const { return m_offset; }
    void offset( uint32 o ) { m_offset = o; }
    uint16 params() const { return m_params; }
    uint16 locals() const { return m_locals; }
+   uint16 undefined() const { return m_undefined; }
+   void params( uint16 p ) { m_params = p; }
+   void locals( uint16 l ) { m_locals = l; }
+   void undefined( uint16 u ) { m_undefined = u; }
+
    /** Counts the parameters and the local variables that this funcdef has in its symtab.
       Useful when the function object is created by the compiler, that won't use addParameter()
       and addLocal(). Those two metods are for API implementors and extensions, while the
@@ -449,7 +560,9 @@ private:
    typedef enum {
       FLAG_EXPORTED=0x1,
       FLAG_ETAFUNC=0x2,
-      FLAG_WELLKNOWN=0x4
+      FLAG_WELLKNOWN=0x4,
+      FLAG_IMPORTED=0x8,
+      FLAG_ENUM=0x16
    }
    e_flags;
 
@@ -563,10 +676,25 @@ public:
       return *this;
    }
 
+   /** Sets the symbol import class.
+      Import class is prioritary to export class; that is, if a symbol is imported,
+      exported() will always return false. Also, imported symbols report their type as unknonw,
+      no matter what local setting is provided.
+      \param exp true if the symbol must be imported, false otherwise.
+      \return itself
+   */
+   Symbol &imported( bool exp ) {
+      if ( exp )
+         m_flags |= FLAG_IMPORTED;
+      else
+         m_flags &=~FLAG_IMPORTED;
+      return *this;
+   }
+
    /** Declares the symbol as an "eta function".
       Eta functions are self-managed functions in Sigma-evaluation
       (functional evaluation).
-      \param exp true if the symbol must be exported, false otherwise.
+      \param exp true if the symbol is an ETA function, false otherwise.
       \return itself
    */
    Symbol &setEta( bool exp ) {
@@ -604,6 +732,22 @@ public:
       return *this;
    }
 
+
+   /** Declares the symbol as an "enum class".
+      Enum classes are classes that only store constant values, or serve
+      otherwise as namespaces.
+      They cannot be instantiated.
+      \param exp true if the symbol is an enumeration.
+      \return itself
+   */
+   Symbol &setEnum( bool exp ) {
+      if ( exp )
+         m_flags |= FLAG_ENUM;
+      else
+         m_flags &=~FLAG_ENUM;
+      return *this;
+   }
+
    void setUndefined() { clear(); m_type = tundef; }
    void setLocalUndef() { clear(); m_type = tlocalundef; }
    void setGlobal() { clear(); m_type = tglobal; }
@@ -621,13 +765,15 @@ public:
    const String &name() const { return *m_name; }
    uint32 id() const { return m_id; }
    type_t type() const { return m_type; }
-   bool exported() const { return (m_flags & FLAG_EXPORTED) == FLAG_EXPORTED; }
+   bool exported() const { return (! imported()) && ((m_flags & FLAG_EXPORTED) == FLAG_EXPORTED); }
+   bool imported() const { return (m_flags & FLAG_IMPORTED) == FLAG_IMPORTED; }
    uint16 itemId() const { return m_itemPos; }
    void itemId( uint16 ip ) { m_itemPos = ip; }
    bool isEta() const { return (m_flags & FLAG_ETAFUNC) == FLAG_ETAFUNC; }
    bool isWKS() const { return (m_flags & FLAG_WELLKNOWN) == FLAG_WELLKNOWN; }
+   bool isEnum() const { return (m_flags & FLAG_ENUM) == FLAG_ENUM; }
 
-   bool isUndefined() const { return m_type == tundef; }
+   bool isUndefined() const { return imported() || m_type == tundef; }
    bool isLocalUndef() const { return m_type == tlocalundef; }
    bool isGlobal() const { return m_type == tglobal; }
    bool isLocal() const { return m_type == tlocal; }
