@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: syntree.h
-   $Id: syntree.h,v 1.19 2007/08/01 13:32:26 jonnymind Exp $
 
    Syntactic tree item definitions.
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: ven giu 4 2004
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #ifndef FALCON_SYNTREE_H
@@ -73,11 +67,14 @@ class FALCON_DYN_CLASS RangeDecl: public BaseAlloc
 {
    Value *m_rstart;
    Value *m_rend;
+   Value *m_step;
+
 
 public:
-   RangeDecl( Value *start, Value *end = 0 ):
+   RangeDecl( Value *start, Value *end = 0, Value *step = 0 ):
       m_rstart( start ),
-      m_rend( end )
+      m_rend( end ),
+      m_step( step )
    {}
 
    RangeDecl( const RangeDecl &other );
@@ -87,6 +84,7 @@ public:
    bool isOpen() const { return m_rend == 0; }
    Value *rangeStart() const { return m_rstart; }
    Value *rangeEnd() const { return m_rend; }
+   Value *rangeStep() const { return m_step; }
    RangeDecl *clone() const { return new RangeDecl( *this ); }
 };
 
@@ -98,6 +96,7 @@ class FALCON_DYN_CLASS Value: public BaseAlloc
 public:
    typedef enum {
       t_nil,
+      t_imm_bool,
       t_imm_integer,
       t_imm_string,
       t_imm_num,
@@ -105,6 +104,7 @@ public:
       t_symdef,
       t_self,
       t_sender,
+      t_funcvar,  //!< Inner functional variable.
 
       t_byref,
       t_array_decl,
@@ -117,6 +117,7 @@ public:
 private:
    type_t m_type;
    union {
+      bool asBool;
       int64 asInteger;
       numeric asNumeric;
       String *asString;
@@ -138,6 +139,12 @@ public:
       copy( other );
    }
 
+   explicit Value( bool val ):
+      m_type( t_imm_bool )
+   {
+      m_content.asBool = val;
+   }
+
    explicit Value( int64 val ):
       m_type( t_imm_integer )
    {
@@ -236,6 +243,7 @@ public:
 
    bool isImmediate() const {
       return m_type == t_nil ||
+             m_type == t_imm_bool ||
              m_type == t_imm_integer ||
              m_type == t_imm_string ||
              m_type == t_imm_num;
@@ -256,6 +264,7 @@ public:
       return false;
    }
 
+   bool asBool() const { return m_content.asBool; }
    int64 asInteger() const { return m_content.asInteger; }
    numeric asNumeric() const { return m_content.asNumeric; }
    String *asString() const { return m_content.asString; }
@@ -269,6 +278,7 @@ public:
    Expression *asExpr() const { return m_content.asExpr; }
 
    void setNil() { m_type = t_nil; }
+   void setBool( bool val ) { m_type = t_imm_bool; m_content.asBool = val; }
    void setInteger( int64 val ) { m_type = t_imm_integer; m_content.asInteger = val; }
    void setNumeric( numeric val ) { m_type = t_imm_num; m_content.asNumeric = val; }
    void setString( String *val ) { m_type = t_imm_string; m_content.asString = val; }
@@ -283,7 +293,8 @@ public:
    void setSender() { m_type = t_sender; }
 
 
-   bool isNil() const {return m_type == t_nil; }
+   bool isNil() const { return m_type == t_nil; }
+   bool isBool() const { return m_type == t_imm_bool; }
    bool isInteger() const { return m_type == t_imm_integer; }
    bool isNumeric() const { return m_type == t_imm_num; }
    bool isString() const { return m_type == t_imm_string; }
@@ -389,17 +400,30 @@ public:
       t_provides,
 
       t_iif,
-      t_let,
       t_lambda,
 
       t_obj_access,
       t_funcall,
-      t_funval,
       t_inherit,
       t_array_access,
       t_array_byte_access,
       t_strexpand,
       t_indirect,
+
+      t_assign,
+
+      t_aadd,
+      t_asub,
+      t_amul,
+      t_adiv,
+      t_amod,
+      t_apow,
+      t_aband,
+      t_abor,
+      t_abxor,
+      t_ashl,
+      t_ashr,
+
       /** An optimized expression is like an unary operator */
       t_optimized,
    } operator_t;
@@ -430,6 +454,42 @@ public:
 
    Value *third() const { return m_third; }
    void third( Value *t ) { delete m_third; m_third = t; }
+
+   bool isBinaryOperator() const
+   {
+      switch( m_operator )
+      {
+         case t_bin_and:
+         case t_bin_or:
+         case t_bin_xor:
+         case t_shift_left:
+         case t_shift_right:
+         case t_and:
+         case t_or:
+
+         case t_plus:
+         case t_minus:
+         case t_times:
+         case t_divide:
+         case t_modulo:
+         case t_power:
+
+         case t_gt:
+         case t_ge:
+         case t_lt:
+         case t_le:
+         case t_eq:
+         case t_neq:
+
+         case t_has:
+         case t_hasnt:
+         case t_in:
+         case t_notin:
+         case t_provides:
+            return true;
+      }
+      return false;
+   }
 };
 
 
@@ -451,23 +511,10 @@ public:
       t_raise,
       t_give,
       t_unref,
-      t_assignment,
-      t_autoadd,
-      t_autosub,
-      t_automul,
-      t_autodiv,
-      t_automod,
-      t_autopow,
-      t_autoband,
-      t_autobor,
-      t_autobxor,
-      t_autoshl,
-      t_autoshr,
       t_if,
       t_elif,
       t_while,
       t_forin,
-      t_for,
       t_try,
       t_catch,
       t_switch,
@@ -711,13 +758,13 @@ public:
 
 class FALCON_DYN_CLASS StmtGive: public Statement
 {
-   Value *m_object;
+   ArrayDecl *m_objects;
    ArrayDecl *m_attribs;
 
 public:
-   StmtGive( uint32 line, Value *object, ArrayDecl *attribs ):
+   StmtGive( uint32 line, ArrayDecl *objects, ArrayDecl *attribs ):
       Statement( line, t_give ),
-      m_object( object ),
+      m_objects( objects ),
       m_attribs( attribs )
    {}
 
@@ -725,204 +772,12 @@ public:
 
    virtual ~StmtGive();
 
-   Value *object() const { return m_object; }
+   ArrayDecl *objects() const { return m_objects; }
    ArrayDecl *attributes() const { return m_attribs; }
 
    virtual Statement *clone() const;
 };
 
-class FALCON_DYN_CLASS StmtBaseAssign: public StmtExpression
-{
-   Value *m_dest;
-
-public:
-   StmtBaseAssign( uint32 line, type_t t, Value *target, Value *expr ):
-      StmtExpression( line, t, expr ),
-      m_dest( target )
-   {}
-
-   StmtBaseAssign( const StmtBaseAssign &other );
-
-   virtual ~StmtBaseAssign();
-
-   Value *destination() const { return m_dest; }
-
-   // base assign is a pure virtual class; no implementation for clone().
-};
-
-
-class FALCON_DYN_CLASS StmtAssignment: public StmtBaseAssign
-{
-
-public:
-   StmtAssignment( uint32 line, Value *target, Value *expr ):
-      StmtBaseAssign( line, t_assignment, target, expr )
-   {}
-
-   StmtAssignment( const StmtAssignment &other ):
-      StmtBaseAssign( other )
-   {}
-
-   virtual Statement *clone() const;
-};
-
-class FALCON_DYN_CLASS StmtAutoAdd: public StmtBaseAssign
-{
-public:
-   StmtAutoAdd( uint32 line, Value *target, Value *expr ):
-      StmtBaseAssign( line, t_autoadd, target, expr )
-   {}
-
-   StmtAutoAdd( const StmtAutoAdd &other ):
-      StmtBaseAssign( other )
-   {}
-
-   virtual Statement *clone() const;
-};
-
-class FALCON_DYN_CLASS StmtAutoSub: public StmtBaseAssign
-{
-public:
-   StmtAutoSub( uint32 line, Value *target, Value *expr ):
-      StmtBaseAssign( line, t_autosub, target, expr )
-   {}
-
-   StmtAutoSub( const StmtAutoSub &other ):
-      StmtBaseAssign( other )
-   {}
-
-   virtual Statement *clone() const;
-};
-
-class FALCON_DYN_CLASS StmtAutoMul: public StmtBaseAssign
-{
-public:
-   StmtAutoMul( uint32 line, Value *target, Value *expr ):
-      StmtBaseAssign( line, t_automul, target, expr )
-   {}
-
-   StmtAutoMul( const StmtAutoMul &other ):
-      StmtBaseAssign( other )
-   {}
-
-   virtual Statement *clone() const;
-};
-
-class FALCON_DYN_CLASS StmtAutoDiv: public StmtBaseAssign
-{
-public:
-   StmtAutoDiv( uint32 line, Value *target, Value *expr ):
-      StmtBaseAssign( line, t_autodiv, target, expr )
-   {}
-
-   StmtAutoDiv( const StmtAutoDiv &other ):
-      StmtBaseAssign( other )
-   {}
-
-   virtual Statement *clone() const;
-};
-
-
-class FALCON_DYN_CLASS StmtAutoMod: public StmtBaseAssign
-{
-public:
-   StmtAutoMod( uint32 line, Value *target, Value *expr ):
-      StmtBaseAssign( line, t_automod, target, expr )
-   {}
-
-   StmtAutoMod( const StmtAutoMod &other ):
-      StmtBaseAssign( other )
-   {}
-
-   virtual Statement *clone() const;
-};
-
-class FALCON_DYN_CLASS StmtAutoPow: public StmtBaseAssign
-{
-public:
-   StmtAutoPow( uint32 line, Value *target, Value *expr ):
-      StmtBaseAssign( line, t_autopow, target, expr )
-   {}
-
-   StmtAutoPow( const StmtAutoPow &other ):
-      StmtBaseAssign( other )
-   {}
-
-   virtual Statement *clone() const;
-};
-
-class FALCON_DYN_CLASS StmtAutoBAND: public StmtBaseAssign
-{
-public:
-   StmtAutoBAND( uint32 line, Value *target, Value *expr ):
-      StmtBaseAssign( line, t_autoband, target, expr )
-   {}
-
-   StmtAutoBAND( const StmtAutoBAND &other ):
-      StmtBaseAssign( other )
-   {}
-
-   virtual Statement *clone() const;
-};
-
-
-class FALCON_DYN_CLASS StmtAutoBOR: public StmtBaseAssign
-{
-public:
-   StmtAutoBOR( uint32 line, Value *target, Value *expr ):
-      StmtBaseAssign( line, t_autobor, target, expr )
-   {}
-
-   StmtAutoBOR( const StmtAutoBOR &other ):
-      StmtBaseAssign( other )
-   {}
-
-   virtual Statement *clone() const;
-};
-
-
-class FALCON_DYN_CLASS StmtAutoBXOR: public StmtBaseAssign
-{
-public:
-   StmtAutoBXOR( uint32 line, Value *target, Value *expr ):
-      StmtBaseAssign( line, t_autobxor, target, expr )
-   {}
-
-   StmtAutoBXOR( const StmtAutoBXOR &other ):
-      StmtBaseAssign( other )
-   {}
-
-   virtual Statement *clone() const;
-};
-
-class FALCON_DYN_CLASS StmtAutoSHL: public StmtBaseAssign
-{
-public:
-   StmtAutoSHL( uint32 line, Value *target, Value *expr ):
-      StmtBaseAssign( line, t_autoshl, target, expr )
-   {}
-
-   StmtAutoSHL( const StmtAutoSHL &other ):
-      StmtBaseAssign( other )
-   {}
-
-   virtual Statement *clone() const;
-};
-
-class FALCON_DYN_CLASS StmtAutoSHR: public StmtBaseAssign
-{
-public:
-   StmtAutoSHR( uint32 line, Value *target, Value *expr ):
-      StmtBaseAssign( line, t_autoshr, target, expr )
-   {}
-
-   StmtAutoSHR( const StmtAutoSHR &other ):
-      StmtBaseAssign( other )
-   {}
-
-   virtual Statement *clone() const;
-};
-
 
 /** Loop control statements (break and continue) */
 class FALCON_DYN_CLASS StmtLoopCtl: public Statement
@@ -1065,7 +920,7 @@ class FALCON_DYN_CLASS StmtForin: public StmtBlock
 {
    StatementList m_first;
    StatementList m_last;
-   StatementList m_all;
+   StatementList m_middle;
 
    Value *m_source;
    Value *m_dest;
@@ -1085,8 +940,8 @@ public:
    StatementList &firstBlock() { return m_first; }
    const StatementList &lastBlock() const { return m_last; }
    StatementList &lastBlock() { return m_last; }
-   const StatementList &allBlock() const { return m_all; }
-   StatementList &allBlock() { return m_all; }
+   const StatementList &middleBlock() const { return m_middle; }
+   StatementList &middleBlock() { return m_middle; }
 
    Value *source() const { return m_source; }
    Value *dest() const { return m_dest; }
@@ -1095,34 +950,6 @@ public:
 };
 
 
-class FALCON_DYN_CLASS StmtFor: public StmtBlock
-{
-   Value *m_counter;
-   Value *m_from;
-   Value *m_to;
-   Value *m_step;
-
-public:
-   StmtFor( uint32 line, Value *counter, Value *from, Value *to, Value *step = 0 ):
-      StmtBlock( line, t_for ),
-      m_counter( counter ),
-      m_from( from ),
-      m_to( to ),
-      m_step( step )
-   {}
-
-   StmtFor( const StmtFor &other );
-
-   virtual ~StmtFor();
-
-   Value *counter() const { return m_counter; }
-   Value *from() const { return m_from; }
-   Value *to() const { return m_to; }
-   Value *step() const { return m_step; }
-
-   virtual Statement *clone() const;
-};
-
 class FALCON_DYN_CLASS StmtCaseBlock: public StmtBlock
 {
 public:
