@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: transcoding.h
-   $Id: transcoding.h,v 1.4 2007/08/03 13:17:06 jonnymind Exp $
 
    Declarations of encoders and decoders.
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: dom ago 20 2006
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -158,12 +152,12 @@ public:
       return m_stream->errorDescription( description );
    }
 
-   virtual int32 readAvailable( int32 msecs_timeout ) {
-      return m_stream->readAvailable( msecs_timeout );
+   virtual int32 readAvailable( int32 msecs_timeout, const Sys::SystemData *sysData = 0 ) {
+      return m_stream->readAvailable( msecs_timeout, sysData );
    }
 
-   virtual int32 writeAvailable( int32 msecs_timeout ) {
-      return m_stream->writeAvailable( msecs_timeout );
+   virtual int32 writeAvailable( int32 msecs_timeout, const Sys::SystemData *sysData ) {
+      return m_stream->writeAvailable( msecs_timeout, sysData );
    }
 
    virtual int64 lastError() const
@@ -199,7 +193,7 @@ public:
    virtual bool get( uint32 &chr );
    virtual bool put( uint32 chr );
    virtual const String encoding() const { return "EOL"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 /** Transparent byte oriented encoder.
@@ -236,7 +230,7 @@ public:
    virtual bool get( uint32 &chr );
    virtual bool put( uint32 chr );
    virtual const String encoding() const { return "byte"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 
@@ -253,7 +247,7 @@ public:
    virtual bool get( uint32 &chr );
    virtual bool put( uint32 chr );
    virtual const String encoding() const { return "utf-8"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 /** UTF-16 encoding transcoder. */
@@ -312,7 +306,7 @@ public:
    virtual bool put( uint32 chr );
    virtual const String encoding() const { return "utf-16"; }
    t_endianity endianity() const { return m_streamEndian; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 /** UTF-16LE encoding transcoder. */
@@ -371,7 +365,7 @@ public:
 
    TranscoderCP1252( Stream *s, bool bOwn=false );
    virtual const String encoding() const { return "cp1252"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 /** Latin-1 (ISO8859_1) transcoder. */
@@ -384,7 +378,7 @@ public:
 
    TranscoderISO8859_1( Stream *s, bool bOwn=false );
    virtual const String encoding() const { return "iso8859-1"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 class FALCON_DYN_CLASS TranscoderISO8859_2: public TranscoderISO_CP
@@ -396,7 +390,7 @@ public:
 
    TranscoderISO8859_2( Stream *s, bool bOwn=false );
    virtual const String encoding() const { return "iso8859-2"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 class FALCON_DYN_CLASS TranscoderISO8859_3: public TranscoderISO_CP
@@ -408,7 +402,7 @@ public:
 
    TranscoderISO8859_3( Stream *s, bool bOwn=false );
    virtual const String encoding() const { return "iso8859-3"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 
@@ -421,7 +415,7 @@ public:
 
    TranscoderISO8859_4( Stream *s, bool bOwn=false );
    virtual const String encoding() const { return "iso8859-4"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 class FALCON_DYN_CLASS TranscoderISO8859_5: public TranscoderISO_CP
@@ -433,7 +427,7 @@ public:
 
    TranscoderISO8859_5( Stream *s, bool bOwn=false );
    virtual const String encoding() const { return "iso8859-5"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 class FALCON_DYN_CLASS TranscoderISO8859_6: public TranscoderISO_CP
@@ -445,7 +439,7 @@ public:
 
    TranscoderISO8859_6( Stream *s, bool bOwn=false );
    virtual const String encoding() const { return "iso8859-6"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 class FALCON_DYN_CLASS TranscoderISO8859_7: public TranscoderISO_CP
@@ -457,7 +451,7 @@ public:
 
    TranscoderISO8859_7( Stream *s, bool bOwn=false );
    virtual const String encoding() const { return "iso8859-7"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 class FALCON_DYN_CLASS TranscoderISO8859_8: public TranscoderISO_CP
@@ -469,7 +463,7 @@ public:
 
    TranscoderISO8859_8( Stream *s, bool bOwn=false );
    virtual const String encoding() const { return "iso8859-8"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 class FALCON_DYN_CLASS TranscoderISO8859_9: public TranscoderISO_CP
@@ -481,7 +475,7 @@ public:
 
    TranscoderISO8859_9( Stream *s, bool bOwn=false );
    virtual const String encoding() const { return "iso8859-9"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 class FALCON_DYN_CLASS TranscoderISO8859_10: public TranscoderISO_CP
@@ -493,7 +487,7 @@ public:
 
    TranscoderISO8859_10( Stream *s, bool bOwn=false );
    virtual const String encoding() const { return "iso8859-10"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 class FALCON_DYN_CLASS TranscoderISO8859_11: public TranscoderISO_CP
@@ -505,7 +499,7 @@ public:
 
    TranscoderISO8859_11( Stream *s, bool bOwn=false );
    virtual const String encoding() const { return "iso8859-11"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 class FALCON_DYN_CLASS TranscoderISO8859_13: public TranscoderISO_CP
@@ -517,7 +511,7 @@ public:
 
    TranscoderISO8859_13( Stream *s, bool bOwn=false );
    virtual const String encoding() const { return "iso8859-13"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 class FALCON_DYN_CLASS TranscoderISO8859_14: public TranscoderISO_CP
@@ -529,7 +523,7 @@ public:
 
    TranscoderISO8859_14( Stream *s, bool bOwn=false );
    virtual const String encoding() const { return "iso8859-14"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 class FALCON_DYN_CLASS TranscoderISO8859_15: public TranscoderISO_CP
@@ -541,7 +535,7 @@ public:
 
    TranscoderISO8859_15( Stream *s, bool bOwn=false );
    virtual const String encoding() const { return "iso8859-15"; }
-   virtual UserData *clone();
+   virtual UserData *clone() const;
 };
 
 
