@@ -6,16 +6,11 @@
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: dom ago 8 2004
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #ifndef flc_VM_H
@@ -42,6 +37,7 @@
 #include <falcon/genericlist.h>
 #include <falcon/fassert.h>
 #include <falcon/bommap.h>
+#include <falcon/vm_sys.h>
 
 #define FALCON_MAX_ATTRIBUTES 64
 #define FALCON_VM_DFAULT_CHECK_LOOPS 1000
@@ -67,24 +63,6 @@ public:
    {}
 };
 
-class TryFrame
-{
-   uint32 m_pc;
-   uint32 m_stackBase;
-   uint32 m_frameBase;
-   Module *m_module;
-
-   friend class VMachine;
-public:
-
-   TryFrame( int32 pc, int32 sb, int32 fb, Module *mod ):
-      m_pc( pc ),
-      m_stackBase( sb ),
-      m_frameBase( fb ),
-      m_module( mod )
-      {}
-};
-
 
 void opcodeHandler_END ( register VMachine *vm );
 void opcodeHandler_NOP ( register VMachine *vm );
@@ -184,15 +162,15 @@ void opcodeHandler_STVR( register VMachine *vm );
 void opcodeHandler_TRAV( register VMachine *vm );
 
 void opcodeHandler_SJMP( register VMachine *vm );
-void opcodeHandler_FORI( register VMachine *vm );
-void opcodeHandler_FORN( register VMachine *vm );
+void opcodeHandler_INCP( register VMachine *vm );
+void opcodeHandler_DECP( register VMachine *vm );
 
 void opcodeHandler_SHL( register VMachine *vm );
 void opcodeHandler_SHR( register VMachine *vm );
 void opcodeHandler_SHLS( register VMachine *vm );
 void opcodeHandler_SHRS( register VMachine *vm );
-void opcodeHandler_LDVR( register VMachine *vm );
-void opcodeHandler_LDPR( register VMachine *vm );
+//void opcodeHandler_LDVR( register VMachine *vm );
+//void opcodeHandler_LDPR( register VMachine *vm );
 void opcodeHandler_POWS( register VMachine *vm );
 void opcodeHandler_LSB( register VMachine *vm );
 void opcodeHandler_UNPS( register VMachine *vm );
@@ -201,7 +179,7 @@ void opcodeHandler_INDI( register VMachine *vm );
 void opcodeHandler_STEX( register VMachine *vm );
 void opcodeHandler_TRAC( register VMachine *vm );
 void opcodeHandler_WRT( register VMachine *vm );
-
+void opcodeHandler_STO( register VMachine *vm );
 
 /** The Falcon virtual machine.
 
@@ -316,6 +294,8 @@ protected:
    Item m_regB;
    Item m_regS1;
    Item m_regS2;
+   Item m_regL1;
+   Item m_regL2;
 
    /** Space for immediate operands. */
    Item m_imm[4];
@@ -401,9 +381,6 @@ protected:
    /** Context currently being executed. */
    VMContext *m_currentContext;
 
-   /** Modules held in this VM */
-   //ModuleVector m_modules;
-
    /** Ready to run contexts. */
    ContextList m_contexts;
    /** Contexts willing to sleep for a while */
@@ -538,6 +515,11 @@ protected:
    */
    void *m_userData;
 
+   /** System specific data.
+      \see systemData();
+   */
+   Sys::SystemData m_systemData;
+
    /** Utility for switch opcode.
       Just pretend it's not here.
    */
@@ -729,9 +711,16 @@ public:
             for start symbol(s) by prepare() function.
       \param rt the runtime to be linked
       \param isMainModule false to prevent this module to be chosen as startup module.
-      \return false on link time error, true on success
+      \return 0 time error, the internally built LiveModule instance on success.
    */
-   bool link( Module *module, bool isMainModule=true );
+   LiveModule *link( Module *module, bool isMainModule=true );
+
+   /** Returns the main module, if it exists.
+      Returns an instance of the LiveModule class, that is the local representation
+      of a module linked in the VM, that holds informations about the "main" module
+      in this virtual machine.
+   */
+   LiveModule *mainModule() const { return m_mainModule; }
 
    /** Unlinks all the modules in the runtime.
 
@@ -1141,6 +1130,24 @@ public:
    const Item &sender() const { return m_regS2; }
    Item &sender() { return m_regS2; }
 
+   /** Latch item.
+      Generated on load property/vector instructions, it stores the accessed object.
+   */
+   const Item &latch() const { return m_regL1; }
+   /** Latch item.
+      Generated on load property/vector instructions, it stores the accessed object.
+   */
+   Item &latch() { return m_regL1; }
+
+   /** Latcher item.
+      Generated on load property/vector instructions, it stores the accessor item.
+   */
+   const Item &latcher() const { return m_regL2; }
+   /** Latcher item.
+      Generated on load property/vector instructions, it stores the accessor item.
+   */
+   Item &latcher() { return m_regL2; }
+
    void requestQuit() { m_event = eventQuit; }
    void requestSuspend() { m_event = eventSuspend; }
    tEvent lastEvent() const { return m_event; }
@@ -1945,6 +1952,43 @@ public:
    */
    Attribute *findAttribute( const Symbol *sym ) const;
 
+   /** Interrupts pending I/O on this machine from a separate thread.
+      Interrupts compliant streams I/O and wait operations. The next I/O or
+      the next wait, or the next compliant system blocking operation
+      will cause an Interrupted exception to be raised.
+
+      This method does not generate an "interrupt request" on the VM, that will
+      keep on running until a blocking operation is encountered.
+
+      This method can be safely called from other threads in the same application
+      where the is currently executed, as well as from inside the same thread.
+   */
+   void interrupt() { m_systemData.interrupt(); }
+
+   /** Returns true if the VM has been interrupted.
+      If an asynchronous interrupt has been generated, this method will return true.
+
+      This is meant to be called from those compliant operations that require polling,
+      or just before starting waiting on interruptable code to avoid useless calculations,
+      or right after to know if the VM has been interrupted during the computation.
+      \param raise if true, prepare an Interrupted exception in the VM context if the VM has been
+                   interrupted.
+      \param reset if true, reset interrupted status now.
+      \param dontCheck suppse interruption test is already done, and just raise the error and reset.
+      \return true if the machine has been interrupted through interrupt() request.
+   */
+   bool interrupted( bool raise = false, bool reset = false, bool dontCheck = false );
+
+   /** Get System Specific data.
+      \returns system specific data bound with this machine.
+   */
+   const Sys::SystemData &systemData() const { return m_systemData; }
+
+   /** Get System Specific data (non const).
+      \returns system specific data bound with this machine.
+   */
+   Sys::SystemData &systemData() { return m_systemData; }
+
 //==========================================================================
 //==========================================================================
 //==========================================================================
@@ -2050,16 +2094,15 @@ public:
    friend void opcodeHandler_STVR( register VMachine *vm );
    friend void opcodeHandler_TRAV( register VMachine *vm );
 
-   friend void opcodeHandler_SJMP( register VMachine *vm );
-   friend void opcodeHandler_FORI( register VMachine *vm );
-   friend void opcodeHandler_FORN( register VMachine *vm );
+   friend void opcodeHandler_INCP( register VMachine *vm );
+   friend void opcodeHandler_DECP( register VMachine *vm );
 
    friend void opcodeHandler_SHL( register VMachine *vm );
    friend void opcodeHandler_SHR( register VMachine *vm );
    friend void opcodeHandler_SHLS( register VMachine *vm );
    friend void opcodeHandler_SHRS( register VMachine *vm );
-   friend void opcodeHandler_LDVR( register VMachine *vm );
-   friend void opcodeHandler_LDPR( register VMachine *vm );
+//   friend void opcodeHandler_LDVR( register VMachine *vm );
+//   friend void opcodeHandler_LDPR( register VMachine *vm );
    friend void opcodeHandler_POWS( register VMachine *vm );
    friend void opcodeHandler_LSB( register VMachine *vm );
    friend void opcodeHandler_UNPS( register VMachine *vm );
@@ -2068,6 +2111,7 @@ public:
    friend void opcodeHandler_STEX( register VMachine *vm );
    friend void opcodeHandler_TRAC( register VMachine *vm );
    friend void opcodeHandler_WRT( register VMachine *vm );
+   friend void opcodeHandler_STO( register VMachine *vm );
 };
 
 }
