@@ -7,16 +7,11 @@
    Author: Giancarlo Niccolai
    Begin: lun ago  9 22:25:35 CEST 2004
 
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #include <falcon/module.h>
@@ -25,24 +20,91 @@
 
 #include "falcon_rtl_ext.h"
 #include "version.h"
+#include <falcon/time_sys.h>
+#include "rtl_messages.h"
+
+using namespace Falcon;
+
+/*#
+   @module falcon_rtl The Runtime Library module
+   @brief Main Falcon module.
+
+   The runtime library represents the set of standard commonly
+   available functionalities that the Falcon users are usually
+   expecting to be always loaded in the virtual machine. It includes
+   important functions that manipulates types, as the array sorting
+   function, or the dictionary traversal functions. Also, it provides basic
+   I/O support, as file and directory manipulation.
+
+   Some of the functions that are provided in the RTL, as printl, are a common
+   hook for embedders. In the case of printl, which provides a common and comfortable
+   simple output for scripts, embedders usually redirect it to their own functions to
+   intercept script output request and redirect them. In some cases, embedders may
+   may find useful to disable some functionalities, i.e. removing file/directory functions.
+
+   RTL also provides many functions that duplicate operator functionalities.
+   Array partitioning and string concatenations are some example. Those duplicate
+   functions are meant both because they are usually more flexible than the pre-defined
+   operators, and because, being Falcon a second order language, it is then possible to
+   pass those functions as function parameters, or to save those functions in variables;
+   of course, operators cannot be used that way.
+
+   However, operators are faster than function calls, so, when possible, use the
+   operators rather than the functions. Functions duplicating operators are mean to be
+   used when operators cannot possibly be employed, or when the limited functionalities
+   of the operators are not enough.
+
+   As Falcon grows, some of this functions may be moved in more specific locations;
+   in example, serialization, or system I/O functions, or mathematic functions, may
+   be moved in other more specific modules. This depends on the decisions of the community
+   for the best usage patterns.
+*/
+
+/*#
+   @beginModule falcon_rtl
+*/
+
+/*#
+   @funset rtl_basic_io Basic I/O
+   @brief Functions providing basic interface.
+
+   RTL Basic I/O functions are mainly meant to provide scripts with a
+   very basic interface to interact with the outside world.
+*/
+
+/*#
+   @funset rtl_general_purpose General purpose
+   @brief Generic functions
+*/
+
+/*#
+   @group rtl_syssupport System Support
+   @brief Function and classes supporting OS and environment.
+
+   This group of functions and classes is meant to provide OS and
+   enviromental basic support to Falcon scripts.
+*/
 
 FALCON_MODULE_DECL( const Falcon::EngineData &data )
 {
+   #define FALCON_DECLARE_MODULE self
+
    // setup DLL engine common data
    data.set();
 
    // Module declaration
    Falcon::Module *self = new Falcon::Module();
    self->name( "falcon_rtl" );
+   self->language( "en_US" );
    self->engineVersion( FALCON_VERSION_NUM );
    self->version( RTL_VERSION_MAJOR, RTL_VERSION_MINOR, RTL_VERSION_REVISION );
 
+
    //=======================================================================
    // Message setting
    //=======================================================================
-   // TODO: load proper messages...
-   // ... and fallback to english:
-   self->stringTable().build( Falcon::Ext::message_table );
+
+   #include "rtl_messages.h"
 
    //=======================================================================
    // RTL basic functionality
@@ -50,6 +112,7 @@ FALCON_MODULE_DECL( const Falcon::EngineData &data )
 
    self->addExtFunc( "print", Falcon::Ext::print );
    self->addExtFunc( "inspect", Falcon::Ext::inspect );
+   self->addExtFunc( "inspectShort", Falcon::Ext::inspectShort );
    self->addExtFunc( "input", Falcon::Ext::input );
    self->addExtFunc( "printl", Falcon::Ext::printl );
    self->addExtFunc( "seconds", Falcon::Ext::seconds );
@@ -64,6 +127,7 @@ FALCON_MODULE_DECL( const Falcon::EngineData &data )
    self->addExtFunc( "randomWalk", Falcon::Ext::flc_randomWalk );
    self->addExtFunc( "randomGrab", Falcon::Ext::flc_randomGrab );
    self->addExtFunc( "randomSeed", Falcon::Ext::flc_randomSeed );
+   self->addExtFunc( "randomDice", Falcon::Ext::flc_randomDice );
 
    //=======================================================================
    // RTL math
@@ -108,6 +172,8 @@ FALCON_MODULE_DECL( const Falcon::EngineData &data )
    self->addExtFunc( "strLower", Falcon::Ext::strLower );
    self->addExtFunc( "strCmpIgnoreCase", Falcon::Ext::strCmpIgnoreCase );
    self->addExtFunc( "strWildcardMatch", Falcon::Ext::strWildcardMatch );
+   self->addExtFunc( "strToMemBuf", Falcon::Ext::strToMemBuf );
+   self->addExtFunc( "strFromMemBuf", Falcon::Ext::strFromMemBuf );
 
    //=======================================================================
    // RTL array API
@@ -161,6 +227,7 @@ FALCON_MODULE_DECL( const Falcon::EngineData &data )
    self->addExtFunc( "fileChown", Falcon::Ext::fileChown );
    self->addExtFunc( "fileChmod", Falcon::Ext::fileChmod );
    self->addExtFunc( "fileChgroup", Falcon::Ext::fileChgroup );
+   self->addExtFunc( "fileCopy", Falcon::Ext::fileCopy );
 
    self->addExtFunc( "dirMake", Falcon::Ext::dirMake );
    self->addExtFunc( "dirChange", Falcon::Ext::dirChange );
@@ -203,6 +270,7 @@ FALCON_MODULE_DECL( const Falcon::EngineData &data )
    Falcon::Symbol *stream_class = self->addClass( "Stream" );
    stream_class->setWKS(true);
    self->addClassMethod( stream_class, "close", Falcon::Ext::Stream_close );
+   self->addClassMethod( stream_class, "flush", Falcon::Ext::Stream_flush );
    self->addClassMethod( stream_class, "read", Falcon::Ext::Stream_read );
    self->addClassMethod( stream_class, "readLine", Falcon::Ext::Stream_readLine );
    self->addClassMethod( stream_class, "write", Falcon::Ext::Stream_write );
@@ -276,6 +344,9 @@ FALCON_MODULE_DECL( const Falcon::EngineData &data )
    self->addClassMethod( tstamp_class, "toLongFormat", Falcon::Ext::TimeStamp_toLongFormat );
    self->addClassMethod( tstamp_class, "fromLongFormat", Falcon::Ext::TimeStamp_fromLongFormat );
    self->addClassMethod( tstamp_class, "compare", Falcon::Ext::TimeStamp_compare );
+   self->addClassMethod( tstamp_class, "fromRFC2822", Falcon::Ext::TimeStamp_fromRFC2822 );
+   self->addClassMethod( tstamp_class, "toRFC2822", Falcon::Ext::TimeStamp_toRFC2822 );
+   self->addClassMethod( tstamp_class, "changeZone", Falcon::Ext::TimeStamp_changeZone );
 
    // properties // setting to default 0
    self->addClassProperty( tstamp_class, "year" )->setInteger( 0 );
@@ -285,9 +356,60 @@ FALCON_MODULE_DECL( const Falcon::EngineData &data )
    self->addClassProperty( tstamp_class, "minute" )->setInteger( 0 );
    self->addClassProperty( tstamp_class, "second" )->setInteger( 0 );
    self->addClassProperty( tstamp_class, "msec" )->setInteger( 0 );
+   self->addClassProperty( tstamp_class, "timezone" )->setInteger( 0 );
+
+   Falcon::Symbol *c_timezone = self->addClass( "TimeZone" );
+   self->addClassMethod( c_timezone, "getDisplacement", Falcon::Ext::TimeZone_getDisplacement );
+   self->addClassMethod( c_timezone, "describe", Falcon::Ext::TimeZone_describe );
+   self->addClassMethod( c_timezone, "getLocal", Falcon::Ext::TimeZone_getLocal );
+   self->addClassProperty( c_timezone, "local" )->setInteger( Falcon::tz_local );
+   self->addClassProperty( c_timezone, "UTC" )->setInteger( Falcon::tz_UTC );
+   self->addClassProperty( c_timezone, "GMT" )->setInteger( Falcon::tz_UTC );
+   self->addClassProperty( c_timezone, "E1" )->setInteger( Falcon::tz_UTC_E_1 );
+   self->addClassProperty( c_timezone, "E2" )->setInteger( Falcon::tz_UTC_E_2 );
+   self->addClassProperty( c_timezone, "E3" )->setInteger( Falcon::tz_UTC_E_3 );
+   self->addClassProperty( c_timezone, "E4" )->setInteger( Falcon::tz_UTC_E_4 );
+   self->addClassProperty( c_timezone, "E5" )->setInteger( Falcon::tz_UTC_E_5 );
+   self->addClassProperty( c_timezone, "E6" )->setInteger( Falcon::tz_UTC_E_6 );
+   self->addClassProperty( c_timezone, "E7" )->setInteger( Falcon::tz_UTC_E_7 );
+   self->addClassProperty( c_timezone, "E8" )->setInteger( Falcon::tz_UTC_E_8 );
+   self->addClassProperty( c_timezone, "E9" )->setInteger( Falcon::tz_UTC_E_9 );
+   self->addClassProperty( c_timezone, "E10" )->setInteger( Falcon::tz_UTC_E_10 );
+   self->addClassProperty( c_timezone, "E11" )->setInteger( Falcon::tz_UTC_E_11 );
+   self->addClassProperty( c_timezone, "E12" )->setInteger( Falcon::tz_UTC_E_12 );
+
+   self->addClassProperty( c_timezone, "W1" )->setInteger( Falcon::tz_UTC_W_1 );
+   self->addClassProperty( c_timezone, "W2" )->setInteger( Falcon::tz_UTC_W_2 );
+   self->addClassProperty( c_timezone, "W3" )->setInteger( Falcon::tz_UTC_W_3 );
+   self->addClassProperty( c_timezone, "W4" )->setInteger( Falcon::tz_UTC_W_4 );
+   self->addClassProperty( c_timezone, "EDT" )->setInteger( Falcon::tz_UTC_W_4 );
+   self->addClassProperty( c_timezone, "W5" )->setInteger( Falcon::tz_UTC_W_5 );
+   self->addClassProperty( c_timezone, "EST" )->setInteger( Falcon::tz_UTC_W_5 );
+   self->addClassProperty( c_timezone, "CDT" )->setInteger( Falcon::tz_UTC_W_5 );
+   self->addClassProperty( c_timezone, "W6" )->setInteger( Falcon::tz_UTC_W_6 );
+   self->addClassProperty( c_timezone, "CST" )->setInteger( Falcon::tz_UTC_W_6 );
+   self->addClassProperty( c_timezone, "MDT" )->setInteger( Falcon::tz_UTC_W_6 );
+   self->addClassProperty( c_timezone, "W7" )->setInteger( Falcon::tz_UTC_W_7 );
+   self->addClassProperty( c_timezone, "MST" )->setInteger( Falcon::tz_UTC_W_7 );
+   self->addClassProperty( c_timezone, "PDT" )->setInteger( Falcon::tz_UTC_W_7 );
+   self->addClassProperty( c_timezone, "W8" )->setInteger( Falcon::tz_UTC_W_8 );
+   self->addClassProperty( c_timezone, "PST" )->setInteger( Falcon::tz_UTC_W_8 );
+   self->addClassProperty( c_timezone, "W9" )->setInteger( Falcon::tz_UTC_W_9 );
+   self->addClassProperty( c_timezone, "W10" )->setInteger( Falcon::tz_UTC_W_10 );
+   self->addClassProperty( c_timezone, "W11" )->setInteger( Falcon::tz_UTC_W_11 );
+   self->addClassProperty( c_timezone, "W12" )->setInteger( Falcon::tz_UTC_W_12 );
+
+   self->addClassProperty( c_timezone, "NFT" )->setInteger( Falcon::tz_NFT );
+   self->addClassProperty( c_timezone, "ACDT" )->setInteger( Falcon::tz_ACDT );
+   self->addClassProperty( c_timezone, "ACST" )->setInteger( Falcon::tz_ACST );
+   self->addClassProperty( c_timezone, "HAT" )->setInteger( Falcon::tz_HAT );
+   self->addClassProperty( c_timezone, "NST" )->setInteger( Falcon::tz_NST );
+
+   self->addClassProperty( c_timezone, "NONE" )->setInteger( Falcon::tz_NST );
 
    // A factory function that creates a timestamp already initialized to the current time:
    self->addExtFunc( "CurrentTime", Falcon::Ext::CurrentTime );
+   self->addExtFunc( "ParseRFC2822", Falcon::Ext::ParseRFC2822 );
 
    //=======================================================================
    // Directory class
@@ -310,6 +432,7 @@ FALCON_MODULE_DECL( const Falcon::EngineData &data )
    self->addConstant( "FILE_TYPE_DEVICE", (Falcon::int64) Falcon::FileStat::t_device );
    self->addConstant( "FILE_TYPE_SOCKET", (Falcon::int64) Falcon::FileStat::t_socket );
    self->addConstant( "FILE_TYPE_UNKNOWN", (Falcon::int64) Falcon::FileStat::t_unknown );
+   self->addConstant( "FILE_TYPE_NOTFOUND", (Falcon::int64) Falcon::FileStat::t_notFound );
 
    //=======================================================================
    // FileStat class
@@ -356,6 +479,38 @@ FALCON_MODULE_DECL( const Falcon::EngineData &data )
    self->addClassMethod( list_class, "insert", Falcon::Ext::List_insert );
    self->addClassMethod( list_class, "clear", Falcon::Ext::List_clear );
 
+   //=======================================================================
+   // The path class
+   //=======================================================================
+   Falcon::Symbol *path_class = self->addClass( "Path", Falcon::Ext::Path_init );
+   path_class->setWKS(true);
+
+   self->addClassProperty( path_class, "path" );
+   self->addClassProperty( path_class, "unit" );
+   self->addClassProperty( path_class, "location" );
+   self->addClassProperty( path_class, "file" );
+   self->addClassProperty( path_class, "extension" );
+   self->addClassProperty( path_class, "filename" );
+
+   //=======================================================================
+   // The path class
+   //=======================================================================
+   Falcon::Symbol *uri_class = self->addClass( "URI", Falcon::Ext::URI_init );
+   uri_class->setWKS(true);
+
+   self->addClassProperty( uri_class, "scheme" );
+   self->addClassProperty( uri_class, "userInfo" );
+   self->addClassProperty( uri_class, "host" );
+   self->addClassProperty( uri_class, "port" );
+   self->addClassProperty( uri_class, "path" );
+   self->addClassProperty( uri_class, "query" );
+   self->addClassProperty( uri_class, "fragment" );
+   self->addClassProperty( uri_class, "uri" );
+   self->addClassMethod( uri_class, "encode", Falcon::Ext::URI_encode );
+   self->addClassMethod( uri_class, "decode", Falcon::Ext::URI_decode );
+   self->addClassMethod( uri_class, "getFields", Falcon::Ext::URI_getFields );
+   self->addClassMethod( uri_class, "setFields", Falcon::Ext::URI_setFields );
+
    //=======================================================================
    // The command line parser class
    //=======================================================================
