@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: itemcopy.cpp
-   $Id: itemcopy.cpp,v 1.6 2007/08/03 13:17:07 jonnymind Exp $
 
    Short description
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: gio mar 16 2006
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -31,12 +25,24 @@
 #include <falcon/cobject.h>
 #include <falcon/vm.h>
 
+/*#
+   @beginmodule falcon_rtl
+*/
+
 namespace Falcon {
 
 namespace Ext {
 
-/**
-   itemCopy( item ) --> shallow copy
+/*#
+   @function itemCopy
+   @inset rtl_general_purpose
+   @brief Performs a shallow copy of one item.
+   @param item The item to be copied.
+   @return A copy of the item.
+   @raise CloneError if the item is not cloneable.
+
+   This function works as the FBOM method @a BOM.clone.
+   @note Deprecated; used the BOM method instead.
 */
 FALCON_FUNC  itemCopy( ::Falcon::VMachine *vm )
 {
