@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: time.cpp
-   $Id: seconds.cpp,v 1.1.1.1 2006/10/08 15:05:06 gian Exp $
 
    Time function
    -------------------------------------------------------------------
    Author: $AUTHOR
    Begin: $DATE
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #include <falcon/module.h>
@@ -28,9 +22,26 @@
 #include <windows.h>
 #endif
 
+/*#
+   @beginmodule falcon_rtl
+*/
+
 namespace Falcon { namespace Ext {
 
 
+/*#
+   @function seconds
+   @inset rtl_general_purpose
+   @brief Returns the number of seconds since the “epoch” as reported by the system.
+   @return The number of seconds and fractions of seconds in a floating point value.
+
+   Actually, this function returns a floating point number which represents seconds
+   and fraction of seconds elapse since a conventional time. This function is mainly
+   meant to be used to take intervals of time inside the script,
+   with a millisecond precision.
+*/
+
+
 FALCON_FUNC  seconds ( ::Falcon::VMachine *vm )
 {
 #ifdef WIN32
