@@ -1,23 +1,17 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: verion.h
-   $Id: version.h,v 1.1 2007/07/25 12:23:08 jonnymind Exp $
 
    Falcon RTL module version informations
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: 2007-07-25
 
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #ifndef RTL_VERSION_H
