@@ -1,22 +1,16 @@
 /*
-   AFALCON - The Falcon Programming Language
+   FALCON - The Falcon Programming Language
    FILE: confparser_mod.cpp
-   $Id: confparser_mod.cpp,v 1.7 2007/01/02 09:14:28 gian Exp $
 
    Configuration parser module -- module service classes
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: sab nov 25 2006
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -103,7 +97,7 @@ bool ConfigFileLine::parseLine()
                m_type = t_section;
                state = section; // section name
             }
-            else if ( chr != ' ' || chr != '\t' )
+            else if ( chr != ' ' && chr != '\t' )
             {
                m_type = t_keyval;
                m_key = new String;
@@ -156,6 +150,7 @@ bool ConfigFileLine::parseLine()
                state = value;
                m_value = new String;
                m_value->append( chr );
+               trimValuePos = m_value->size();
             }
          break;
 
@@ -408,7 +403,7 @@ bool ConfigFile::load()
    if ( ! stream.open( m_fileName, GenericStream::e_omReadOnly, GenericStream::e_smShareRead ) )
    {
       stream.errorDescription( m_errorMsg );
-      m_fsError = stream.lastError();
+      m_fsError = (long) stream.lastError();
       return false;
    }
 
@@ -511,7 +506,7 @@ bool ConfigFile::load( Stream *input )
    // error handling
 
 error:
-   m_fsError = input->lastError();
+   m_fsError = (long) input->lastError();
    input->errorDescription( m_errorMsg );
    return false;
 }
@@ -527,7 +522,7 @@ bool ConfigFile::save()
       FileStream::e_aUserRead | FileStream::e_aReadOnly,
       FileStream::e_smShareRead ) )
    {
-      m_fsError = stream.lastError();
+      m_fsError = (long) stream.lastError();
       stream.errorDescription( m_errorMsg );
       return false;
    }
@@ -607,7 +602,7 @@ bool ConfigFile::save( Stream *output )
 
    if ( !output->good() )
    {
-      m_fsError = output->lastError();
+      m_fsError = (long) output->lastError();
       output->errorDescription( m_errorMsg );
       return false;
    }
