@@ -1,31 +1,25 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: verion.h
-   $Id: version.h,v 1.1 2007/07/25 12:23:08 jonnymind Exp $
 
    Regex module version informations
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: 2007-07-25
 
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 #ifndef VERSION_H
 #define VERSION_H
 
 #define VERSION_MAJOR        1
-#define VERSION_MINOR        0 
-#define VERSION_REVISION     0
+#define VERSION_MINOR        0
+#define VERSION_REVISION     1
 
 #endif
 
